<?php
    namespace yoda_tools\includes\core;
    use PHPMailer\PHPmailer\PHPMailer;
    use PHPMailer\PHPMailer\Exception;
    use yoda_tools\includes\core\utilities;

    require_once YODA_TOOLS_PATH.'includes/extensions/mailgun/mailgun.php';
    use Mailgun\Mailgun;
use WP_Error;

    /* Does User Belong Here */
    if (!defined('YODA_TOOLS_PATH')) {
        header('Status: 403 Forbidden');
        header('HTTP/1.1 403 Forbidden');
        exit();
    }

    if ( version_compare( get_bloginfo( 'version' ), '5.5-alpha', '<' ) ) {
        if ( ! class_exists( '\PHPMailer', false ) ) {
            require_once ABSPATH . WPINC . '/class-phpmailer.php';
        }

    } else {
        if ( ! class_exists( '\PHPMailer\PHPMailer\PHPMailer', false ) ) {
            require_once ABSPATH . WPINC . '/PHPMailer/PHPMailer.php';
        }

        if ( ! class_exists( '\PHPMailer\PHPMailer\Exception', false ) ) {
            require_once ABSPATH . WPINC . '/PHPMailer/Exception.php';
        }

        if ( ! class_exists( '\PHPMailer\PHPMailer\SMTP', false ) ) {
            require_once ABSPATH . WPINC . '/PHPMailer/SMTP.php';
        }
    }

    class Mail_Handler extends PHPMailer{
        private $mailArgs = array();
        private $options;
        private $email_data;
        private $error;
        private $transcript = '';
	    private $mailgunMessage;
        private $sendTo;
        private $tableLog;
        private $smtp_status;


        public function __construct($exceptions = null)
        {
            parent::__construct($exceptions);
            $this->mailgunMessage = array(
			    'from'    => '',
			    'to'      => '',
			    'subject' => '',
			);
            
            $this->set_vars();

            $mode_tracking = $this->options->getOption('mode-tracking');

            if (!$mode_tracking) {
                //if mode_tracking returns empty let's double check by retrieving the settings data
                $mode_tracking = ($this->options->getSettings('yoda_tools_events'))? 'on' : 'off';
            }

            $this->smtp_status = $this->options->getOption('mode-smtp-tools');    
            
            if($this->smtp_status == 'on'){

                if ( $this->options->getOption('transport_type') == 'smtp' ) {
                    add_action( 'phpmailer_init', array( $this, 'init_smtp' ), 999 );
                }
                
                add_action( 'wp_mail_failed', array( $this, 'yoda_tools_wp_mail_failed' ) );
                add_action( 'wp_mail_succeeded', array( $this, 'yoda_tools_wp_mail_succeeded' ) );

	            add_action('admin_post_export_email_logs', array( $this, 'export_logs_to_csv'));


                $this->hooks();
                
            }
        }

        public static function register(): void { }

        public function set_vars() {
            $this->options = settings::getInstance();
            $this->Debugoutput = function($str, $level) {
                $this->transcript .= $str;
            };
        }

        function hooks(){
            add_filter( 'wp_mail', array( $this, 'yoda_tools_wp_mail' ) );

            //let's override the from email
            add_filter( 'wp_mail_from', function($from){
                $new_from = $this->options->getOption('envelope_sender');
                if ($new_from) {
                    return $new_from;
                } else {
                    return 'no@reply.com';
                }  
                return $from;
            }, 10, 1);
        }

        public function yoda_tools_wp_mail( $args ) {
            
            $this->mailArgs = array();
            $this->mailArgs['to'] = $args['to'];
            $this->mailArgs['subject'] = $args['subject'];
            $this->mailArgs['message'] = $args['message'];
            $this->mailArgs['headers'] = $args['headers'];
            $this->mailArgs['attachments'] = $args['attachments'];

            return $this->mailArgs;
        }

        function yoda_tools_wp_mail_failed($error) {
            if ( ! empty( $error->errors ) && ! empty( $error->errors['wp_mail_failed'] ) && is_array( $error->errors['wp_mail_failed'] ) ) {
                $log_more_data = $this->detailed_log($error);
                $this->logData('log', $this->mailArgs, $error->get_error_message(), $log_more_data);
            }
        }

        function yoda_tools_wp_mail_succeeded() {
            //this will duplicates the log
            //$this->logData('log', $this->mailArgs, "Email successfully delivered!");
        }

        /**
         * @param PHPMailer $phpmailer
         */
        public function init_smtp( $phpmailer ) {
           
            /* Set the mailer type as per config above, this overrides the already called isMail method */
            $phpmailer->isSMTP();
            $phpmailer->Host = $this->options->getOption('hostname');
            /* Set the other options */
            $phpmailer->Port = $this->options->getOption('port');
            $phpmailer->SMTPDebug = 2;
            $phpmailer->CharSet  = "utf-8";
            
            $from_name = ! empty( $this->FromName ) ? $this->FromName : $this->options->getOption('envelope_sender_name');
            $from_email = ! empty($this->options->getOption('envelope_sender')) ? $this->options->getOption('envelope_sender') : 'no@reply.com';
            $phpmailer->From     = $from_email;
            $phpmailer->FromName = $from_name;

            //set ReplyTo option if needed
            //this should be set before SetFrom, otherwise might be ignored
            //$phpmailer->addReplyTo( 'no@reply.com', $from_name );
            if ( ! empty( $this->options->getOption('reply_to') ) ) {
                $phpmailer->addReplyTo( $this->options->getOption('reply_to'), $from_name );
            }

            $phpmailer->setFrom( $phpmailer->From, $phpmailer->FromName );
            //This should set Return-Path header for servers that are not properly handling it, but needs testing first
            /* Set the SMTPSecure value */
            if ( 'none' !== $this->options->getOption('enc_type') ) {
                $phpmailer->SMTPSecure = $this->options->getOption('enc_type');
            }

            /* If we're using smtp auth, set the username & password */
            $phpmailer->SMTPAuth = false;
            if ( 'none' !== $this->options->getOption('auth_type') ) {
                $phpmailer->SMTPAuth = true;
                $phpmailer->Username = $this->options->getOption('username');

                //can't authenticate because for some reason they put the password inside MD5
                // $phpmailer->Password = md5($this->options->getOption('password'));
                $phpmailer->Password = utilities::ssl_encrypt(['data' => $this->options->getOption('password'), 'action' => 'decrypt']);
            }   
                     
        }

        function send(){

            if ($this->smtp_status == 'off') {
                return;
            }

            $this->sendTo = $this->to[0][0];
            if(count($this->to[0]) > 2){
                $this->sendTo = implode(', ', $this->to[0]);
            }
                        
            $this->mailArgs = array();
            $this->mailArgs['to'] = $this->sendTo;
            $this->mailArgs['subject'] = $this->Subject;
            $this->mailArgs['message'] = $this->Body;
            $this->mailArgs['attachments'] = $this->attachment;
            $this->mailArgs['from_headers'] = $this->FromName;
            $this->mailArgs['from'] = $this->From;
            $this->mailArgs['cc'] = $this->cc;
            $this->mailArgs['bcc'] = $this->bcc;
            $this->mailArgs['ReplyTo'] = $this->ReplyTo;
            
            if ($this->options->getOption('transport_type') == 'mailgun') {
                $this->mailArgs['delivery_uri'] = $this->options->getOption('mailgun_domain_name');
            } elseif ($this->options->getOption('transport_type') == 'smtp') {
                $this->mailArgs['delivery_uri'] = $this->options->getOption('hostname').':'.$this->options->getOption('port');
            } else {
                $this->mailArgs['delivery_uri'] = wp_parse_url( network_home_url(), PHP_URL_HOST );
            }

            $contentType = $this->ContentType;
			if ( ! empty( $contentType ) ) {
				$this->addHeader( 'Content-Type', $contentType );
			}

            $senderText = ! empty( $this->FromName ) ? $this->FromName : $this->From;
            $this->mailgunMessage ['from'] = "{$senderText} <{$this->From}>";

			$this->addHeader( 'Sender', $this->options->getOption('envelope_sender') );

            // add the to recipients
            $recipients = array();
			foreach ( ( array ) $this->all_recipients as $recipient=>$value ) {
				$recipients[] = $recipient;
			}
			$this->mailgunMessage['to'] = $recipients;



            // add the cc recipients
            // refactored by juls 11/08/23
            $cc_recipients = null;
			foreach ( ( array ) $this->cc as $recipient => $value ) {
                if (is_array($value)) {
                    $cc_recipients = implode(',', $value);
                }
			}
			$this->mailgunMessage['cc'] = $cc_recipients;


            // add the bcc recipients
            // refactored by juls 11/08/23
			$bcc_recipients = null;
			foreach ( ( array ) $this->bcc as $recipient => $value ) {
                if (is_array($value)) {
                    $bcc_recipients = implode(',', $value);
                }
			}
			$this->mailgunMessage['bcc'] = $bcc_recipients;


            // add the reply-to
            // refactored by juls 11/08/23
			$replyTo = $this->ReplyTo;
            $replyTo_recipients = [];
            if (is_array($this->ReplyTo)) {
                foreach ($this->ReplyTo as $rep_index => $repto_email) {
                    $replyTo_recipients[] = $rep_index;
                }
            }
            
            if (!empty($replyTo_recipients)) {
                $replyTo = implode(',', $replyTo_recipients);
                $this->mailgunMessage['reply-to'] = $replyTo;
            }
            
            //juls modification ends here

            // add the date
			$date = $this->MessageDate;
			if ( ! empty( $date ) ) {
				$this->addHeader( 'date', $this->MessageDate );
			}

            // add the messageId
			$messageId = $this->MessageID;
			if ( ! empty( $messageId ) ) {
				$this->addHeader( 'message-id', $messageId );
			}

            // add the subject
			if ( null !== $this->Subject ) {
				$this->mailgunMessage ['subject'] = $this->Subject;
			}

            $attachments = $this->attachment;
            
			if ( ! is_array( $attachments ) ) {
				// WordPress may a single filename or a newline-delimited string list of multiple filenames
				$attArray[] = explode( PHP_EOL, $attachments );
			} else {
				$attArray = $attachments;
			}

			$attachments = array();
			foreach ( $attArray as $file ) {
				if ( ! empty( $file ) ) {

                    //@since 1.5.7
                    //Support multidimensional array where the filepath is located
                    if (is_array($file)) {
                        //we only taking the first index
                        //disregard the rest
                        //@since 1.5.7
                        $attachments[] = ['filePath' => $file[0]];
                    } else {
                        $attachments[] = array( 'filePath' => $file );
                    }
					
				}
			}

			if ( ! empty( $attachments ) ) {
				$this->mailgunMessage['attachment'] = $attachments;
			}
			
            if ( $this->options->getOption('transport_type') == 'mailgun' ) {
                //let's return the status of mailgun so that we can have a proper response from wp_mail else it will always return false
                return $this->init_mailgun($this->mailArgs);
            }
            
            if($this->options->getOption('transport_type') != 'mailgun'){
            
                try {
                    $result = false;
                    if ( $this->options->getOption('transport_type') != 'mailgun' ) {
                        $result = $this->sendSmtp();
                    }
                    if($result){
                        
                        $this->logData('log', $this->mailArgs, "Email successfully delivered!");
                    }

                    return $result;

                } catch (Exception $exc) {

                    
                    $log_more_data = $this->detailed_log($exc);

                    $this->error = $exc;

                    $this->mailHeader = '';

                    $this->logData('log', $this->mailArgs, $exc->getMessage(), $log_more_data );

                    return false;

                }
            }
            
        }

        function init_mailgun($mail_data){

            $this->mailArgs = $mail_data;

            $apikey = $this->options->getOption('mailgun_api_key');
            $domain = $this->options->getOption('mailgun_domain_name');

            $attachments = [];
            if (isset($this->mailgunMessage['attachment'])) {
                $attachments = $this->mailgunMessage['attachment']; //@since 1.5.7 we prepared the attachment in ->mailgunMessage why not using it?
            }

            $params = array(
                'from'	        => $this->mailgunMessage['from'],
                'to'            => $this->mailArgs['to'],
                'subject'       => $this->mailArgs['subject'],
                'text'          => $this->mailArgs['subject'],
                'html'          => $this->mailArgs['message'],
                'attachment'    => $attachments,
            );

            if (isset($this->mailgunMessage['cc'])) {
                $params['cc'] = $this->mailgunMessage['cc'];
            }

            if (isset($this->mailgunMessage['bcc'])) {
                $params['bcc'] = $this->mailgunMessage['bcc'];
            }

            //add optional to header
            if (isset($this->mailgunMessage['reply-to']) ) {
                $params['h:Reply-To'] = $this->mailgunMessage['reply-to'];
            }




            try {

                $this->api_endpoint = ! is_null( $this->options->getOption('mailgun_region') ) ? 'https://api.eu.mailgun.net' : 'https://api.mailgun.net';

                $mgClient = Mailgun::create($apikey, $this->api_endpoint);
                
                # Make the call to the client.
                $result = $mgClient->messages()->send($domain, $params);

                $this->mailArgs["transcript"] = print_r( $result, true );
                
                if($result){
                    $this->logData('log', $this->mailArgs, "Email successfully delivered!");
                    return true;
                }

            } catch ( \Mailgun\Exception $e ) {

                $log_more_data = $this->detailed_log($e);

                $this->logData('log', $this->mailArgs, $e->getMessage(), $log_more_data);
                return false;
            }

        }

        function logData($log, $data, $status, $report = null){
            
            global $wpdb;
            $this->tableLog = $wpdb->prefix . 'yoda_tools_email_logs';

            $transcript = (!$report)? json_encode(['message' => $this->mailgunMessage, 'report' => []]) : json_encode(['message' => $this->mailgunMessage, 'report' => $report]);
            
            $timezone_string = get_option('timezone_string');
            date_default_timezone_set($timezone_string);
            $local_date = date('Y-m-d H:i:s');
            $gmt_date = gmdate('Y-m-d H:i:s', strtotime($local_date));

            $query = "INSERT INTO 
                        {$this->tableLog} 
                    SET 
                        email_subject = %s, 
                        sent_to = %s, 
                        email_status = %s, 
                        date_time_sent = '$gmt_date',
                        email_data = %s,
                        transcript = %s";
            $wpdb->query($wpdb->prepare(
                    $query,
                    array(
                        $data['subject'],
                        $data['to'],
                        $status,
                        json_encode($this->mailArgs),
                        $transcript
                    )
                )
            );
        }

        public function sendSmtp() {
            if (!$this->preSend()) {
                return false;
            }

            return $this->postSend();
        }

        private function addHeader( $name, $value, $deprecated = '' ) {
			if ( ! empty( $value ) ) {
                if (is_array($value)) {
                    foreach ($value as $v) {
                        $value = (is_array($v))? implode(',', $v) : $v;
                    }
                }
				$this->mailgunMessage['h:' . $name] = preg_replace('/.*:\s?/', '', $value);
			}
		}

	    private function generate_snippet($email_content) {
		    // Snippet generation (e.g., first 100 characters)
		    return mb_strimwidth(strip_tags($email_content), 0, 100, '...');
	    }


	    public function export_logs_to_csv() {
		    global $wpdb;
		    $this->tableLog = $wpdb->prefix . 'yoda_tools_email_logs';
		    $logs = $wpdb->get_results("SELECT * FROM {$this->tableLog}");

		    $filename = 'email_logs_' . date('Ymd') . '.csv';
		    header('Content-Type: text/csv');
		    header('Content-Disposition: attachment;filename="'.$filename.'"');

		    $fp = fopen('php://output', 'w');
		    fputcsv($fp, array('Id', 'Subject', 'Recipient', 'Status', 'Date', 'Data')); // Adjust headers

		    foreach ($logs as $log) {
			    $body = '';
				if(!empty($log->transcript)) {
					$email_data = json_decode($log->email_data);

					if (!empty($email_data) && !empty($email_data->message)){
						$body = preg_replace('/<style[^>]*>.*?<\/style>/is', '', $email_data->message); // Strip CSS and style-related tags using regex
						$body = strip_tags($body); // Remove HTML tags
						$body = htmlspecialchars($body, ENT_QUOTES | ENT_SUBSTITUTE, 'UTF-8'); // Convert HTML special characters to entities
						$body = str_replace(array("\r", "\n"), ' ', $body); // Remove newlines
						$body = preg_replace('/\s+/', ' ', $body); // Remove excessive whitespace (spaces, newlines, tabs)
						$body = trim($body); // Trim any leading or trailing whitespace
					}
				}
			    fputcsv($fp, array($log->log_id, $log->email_subject, $log->sent_to, $log->email_status, $log->date_time_sent, $body));
		    }
		    fclose($fp);
		    exit;
	    }

        public function detailed_log( $exc )
        {

            if ($exc instanceof WP_Error) {

                $message = '';
                if (strpos($exc->errors['wp_mail_failed'][0], 'wordpress@localhost') !== false) {
                    $message = '"wordpress@localhost" is not a valid email address as "localhost" is a reserved domain name that is not meant for use in email addresses.To fix this error, you should update the email address used by WordPress to a valid email address associated with your domain name. This can usually be done in the WordPress settings or by editing the site\'s configuration files.';
                }

                if (is_array($exc->error_data['wp_mail_failed']['to'])) {
                    $args = 'To: '.implode(',',$exc->error_data['wp_mail_failed']['to'])."\r\n";
                }
                
                if (is_string($exc->error_data['wp_mail_failed']['subject'])) {
                    $args .= 'Subject: '.$exc->error_data['wp_mail_failed']['subject']."\r\n";
                }
                
                if (is_array($exc->error_data['wp_mail_failed']['headers'])) {
                    $args .= 'Headers: '.implode(',',$exc->error_data['wp_mail_failed']['headers'])."\r\n";
                }
                
                if (is_array($exc->error_data['wp_mail_failed']['attachments'])) {
                    $args .= 'Attachments: '.implode(',',$exc->error_data['wp_mail_failed']['attachments']);
                }

                $failed_msg = '';
                if (is_array($exc->error_data['wp_mail_failed']['message'])) {
                    $failed_msg = $exc->error_data['wp_mail_failed']['message'];
                }

                $details[] = [
                    'function' => 'wp_mail',
                    'header' => $args,
                    'content' => $failed_msg
                ];

                $error_name = '';
                if (is_string($exc->errors['wp_mail_failed'][0])) {
                    $error_name = $exc->errors['wp_mail_failed'][0];
                }
                return [
                    'error' => $error_name,
                    'error_code' => 'WP_error',
                    'human_message' => $message,
                    'details' => $details
                ];
            }

            $traces = $exc->getTrace();

            $acceptable_source = ['smtpSend', 'mailSend', 'init_mailgun'];
            $acceptable_source_desc = ['smtpSend' => 'SMTP', 'mailSend' => 'Default', 'init_mailgun' => 'Mailgun'];
                    
            $details = [];
            foreach ($traces as $trace) {
               
                if (!in_array($trace['function'], $acceptable_source))
                    continue;

                $args = null;
                $content = null;
                
                if ( isset($trace['class']) ) {
                    //check for mailgun
                    if ( $trace['function'] == 'init_mailgun' ) {

                        if (!empty($trace['args'])) {
                            foreach ($trace['args'] as $arg) {
                                $args = 'to: '.$arg['to']."\r\n";
                                $args .= 'subject: '.$arg['subject']."\r\n";
                                $args .= 'attachments: '.implode(',', $arg['attachments'])."\r\n";
                                $args .= 'from_headers: '.$arg['from_headers']."\r\n";
                                $args .= 'from: '.$arg['from']."\r\n";
                                $args .= 'cc: '.implode(',', $arg['cc'])."\r\n";
                                $args .= 'bcc: '.implode(',', $arg['bcc'])."\r\n";
                                $args .= 'ReplyTo: '.implode(',', $arg['ReplyTo'])."\r\n";
                                $args .= 'Delivery URI: '.$arg['delivery_uri']."\r\n";
                                $content = $arg['message'];
                            }
                        }

                    } else {

                        if (!empty($trace['args'])) {
                            if (isset($trace['args'][0]) && !empty($trace['args'][0])) {
                                $args = $trace['args'][0];
                            }
                            if (isset($trace['args'][1]) && !empty($trace['args'][1])) {
                                $content = $trace['args'][1];
                            }
                        }
        
                    }
                }

                if (empty($args)) 
                    continue;
                
                $details[] = [
                    'function' => $acceptable_source_desc[$trace['function']],
                    'header' => $args,
                    'content' => $content
                ];
            }

            $human_message = '';
        
            switch (trim($exc->getMessage())) {
                case 'Could not instantiate mail function.':
                    $human_message = 'Is an error message that typically appears when there is an issue with the mail server configuration on a web server. Problem with host provider or if you are in localhost check your mailer.ini and php.ini for correct configuration.';
                    break;
                case 'SMTP Error: Could not connect to SMTP host. Failed to connect to server':
                    $human_message = 'Is an error message that typically indicates that the email client or application was unable to connect to the SMTP (Simple Mail Transfer Protocol) server to send an email. You may need to check the SMTP settings in your email client or application, verify that the SMTP server is correctly configured, and ensure that the necessary ports are open on your server.';
                    break;
                case 'The following From address failed: customer@codecorun.com : MAIL FROM command failed,Error: authentication Required,530,SMTP server error: MAIL FROM command failed Detail: Error: authentication Required SMTP code: 530':
                    $human_message = 'When this command fails, it usually indicates that the email server requires authentication to accept emails from the sender\'s address. To resolve this error, you should ensure that the sender\'s email address is properly authenticated with the SMTP server. You can do this by providing valid login credentials for the SMTP server in the email client or application settings.';
                    break;
                case 'SMTP Error: Could not authenticate.':
                    $human_message = 'Is an error message that typically indicates that the email client or application was unable to authenticate with the SMTP (Simple Mail Transfer Protocol) server during the sending of an email. To resolve this error, you should ensure that the login credentials used to authenticate with the SMTP server are correct and up-to-date.';
                    break;
                case 'SMTP Error: Could not connect to SMTP host.Connection failed.stream_socket_enable_crypto(): SSL operation failed with code 1.OpenSSL Error messages: error: 0 A000086: SSL routines: : certificate verify failed':
                    $human_message = 'Is an error message that typically indicates a problem with SSL/TLS certificate verification when attempting to connect to an SMTP (Simple Mail Transfer Protocol) server. <p>To resolve this error, you can try the following steps:</p>
                        <ol>
                        <li>Verify that the SMTP server is correctly configured with a valid SSL/TLS certificate.</li>
                        <li>Ensure that the SSL/TLS certificate is not expired or revoked.<li>
                        <li>Check that the date and time settings on your computer are correct as an incorrect time and date can cause SSL/TLS certificate verification failure.<li>
                        <li>Configure your email client or application to ignore SSL/TLS certificate verification if necessary, although this is not recommended as it could pose a security risk.<li>
                        <li>If you are not sure how to resolve this issue, you can contact your email provider or server administrator for assistance.<li></ul>';
                    break;
                case 'SMTP Error: Could not connect to SMTP host. Connection failed. stream_socket_client(): SSL: An existing connection was forcibly closed by the remote host':
                    $human_message = 'Is an error message that typically indicates that the email client or application was unable to connect to the SMTP (Simple Mail Transfer Protocol) server because the connection was closed by the server.
                    <p>To resolve this error, you can try the following steps:</p>
                    <ol>
                        <li>Check your internet connection and ensure that you have a stable and reliable network connection.</li>
                        <li>Check that the SMTP server is running and accepting connections.</li>
                        <li>Ensure that the SSL/TLS certificate on the SMTP server is valid and not expired.</li>
                        <li>Check that the ports required for SMTP communication are not blocked by a firewall or other security software.</li>
                        <li>If you continue to encounter this error, you may want to contact your email provider or server administrator for assistance.</li>
                    </ol>';
                    break;
                case 'SMTP Error: Could not connect to SMTP host.':
                case 'Could not connect to SMTP host.':
                    $human_message = 'Is an error message that typically appears when an email client or application is unable to establish a connection to the SMTP (Simple Mail Transfer Protocol) server.';
                    break;
                case 'Your credentials are incorrect.':
                    $human_message = 'The API key and/or API domain provided by the user are not valid or do not match the credentials associated with their Mailgun account. To resolve this error, the user should double-check that they are using the correct API key and domain associated with their Mailgun account. ';
                    break;
                case 'Expected a non-empty value. Got: ""':
                    $human_message = 'The domain name in your mailgun configuration is likely empty.';
                    break;
            }

            return [
                'error' => $exc->getMessage(),
                'error_code' => $exc->getCode(),
                'human_message' => $human_message,
                'details' => $details
            ];
        }

    }
