<?php 
    /**
     *
     *
     * adminNotices
     * class for admin notices
     *
     * @since 1.3.3
     * @author - maennche
     *         - Juls Terobias
     *
     *
     */
    namespace yoda_tools\includes\core;
    use yoda_tools\includes\core\settings;

    use WP_Error;

    /* Does User Belong Here */
    if (!defined('YODA_TOOLS_PATH')) {
        header('Status: 403 Forbidden');
        header('HTTP/1.1 403 Forbidden');
        exit();
    }

    class adminNotices
    {

        /**
         *
         *
         * @since 1.3.3
         *
         *
         */
        private array $settings = [];


        public function __construct()
        {


            $yoda_options = settings::getInstance();
            $settings = $yoda_options->getOption('an-settings');


            if (!empty($settings)) {
                foreach ($settings as $index => $set) {
                    if (isset($set)) {
                        $this->settings[$index] = $set;
                    }
                }
            }


            add_action( 'admin_enqueue_scripts', [$this,'load_assets'] );

            if (isset($_GET["page"]) && $_GET["page"] == "yoda-tools-admin-notices") {
                add_action('admin_init', [$this, 'save_settings']);
                add_action('admin_init', [$this, 'show_notice']);
            }

            // Priority of 0 to render before any notices.
            add_action( 'network_admin_notices', array( $this, 'wrap_head' ), 0 );
            add_action( 'user_admin_notices', array( $this, 'wrap_head' ), 0 );
            add_action( 'admin_notices', array( $this, 'wrap_head' ), 0 );
            add_action( 'all_admin_notices', array( $this, 'wrap_tail' ), 999999999999 );
            //add admin tab panel


            add_action( 'admin_bar_menu', [$this, 'admin_tab'], 100 );
            //add notification widget
            add_action('admin_footer', [$this, 'load_notice_area']);

            $this->load_ajax();
        }

        /**
         *
         *
         * wrap_head
         * @since 1.3.3
         *
         *
         */
        public function wrap_head()
        {
            echo '<!--start--><div class="yoda_an_wrapps">';
        }

        /**
         *
         *
         * wrap_tail
         * @since 1.3.3
         *
         *
         */
        public function wrap_tail()
        {
            echo '</div> <!--end-->';
        }

        /**
         *
         *
         * notice_display_css
         * @since 1.3.3
         *
         *
         */
        public function notice_display_css()
        {

            $classes = [];
            $display = 'display: block !important';
            if (!empty($this->settings)) {

                switch ($this->settings['yoda_an_sln_options']) {
                    case '3':
                        $classes[] = '.notice.notice-success';
                        $classes[] = '.success';
                        $classes[] = '.notice.notice-updated';
                        $classes[] = '.updated';
                        break;
                }

                switch ($this->settings['yoda_an_eln_options']) {
                    case '3':
                        $classes[] = '.notice.notice-error';
                        $classes[] = '.error';
                        break;
                }

                switch ($this->settings['yoda_an_wln_options']) {
                    case '3':
                        $classes[] = '.notice.notice-warning';
                        $classes[] = '.warning';
                        break;
                }

                switch ($this->settings['yoda_an_iln_options']) {
                    case '3':
                        $classes[] = '.notice.notice-info';
                        $classes[] = '.info';
                        break;
                }

                if (!empty($this->settings['yoda_an_css_selector'])) {
                    $classes[] = $this->settings['yoda_an_css_selector'];
                }

            }

            if (!empty($classes)) {
                $classes =  implode(',', $classes);
                $classes .= '{ '.$display.' }';
                return $classes;
            }

            return '';
        }

        /**
         *
         *
         * load assets
         * load assets to admin notices page only
         * @since 1.3.2
         *
         *
         */
        public function load_assets() : void
        {
            if (!$this->disable_to_user()) {
                //load select2 assets in admin notices page only
                wp_enqueue_style( YODA_TOOLS_SLUG.'-admin-notices-css',YODA_TOOLS_URL.'css/adminNotices.css');
                wp_enqueue_script( YODA_TOOLS_SLUG.'-admin-notices-js', YODA_TOOLS_URL.'js/adminNotices.js');
                wp_localize_script( YODA_TOOLS_SLUG.'-admin-notices-js', 'yodaAdminNotices', ['yodaANSettings' => json_encode($this->settings)]);

                //hide notices
                $system_notices = null;
                if (!empty($this->settings['yoda_an_css_selector'])) {
                    if ($this->settings['yoda_an_css_selector'] === "1") {
                        $system_notices = ", .wrap .notice,
                        .wrap .error,
                        .wrap .updated";
                    }
                }

                $hide_notices = '.yoda_an_wrapps .notice, 
                .yoda_an_wrapps .error, 
                .yoda_an_wrapps .updated'.$system_notices.'{
                    display: none;
                }'.$this->notice_display_css(); // override the display base from settings

                wp_add_inline_style(YODA_TOOLS_SLUG.'-admin-notices-css', $hide_notices);
	            
            }
        }


        /**
         *
         *
         * load_ajax
         * load all ajax hooks
         * @since 1.3.3
         *
         *
         */
        public function load_ajax()
        {
            add_action('wp_ajax_yoda_an_search_user', [$this, 'search_users']);
        }


        /**
         *
         *
         * search_users
         * load users when search
         * @since 1.3.3
         *
         *
         */
        public function search_users()
        {
            if ( ! wp_verify_nonce( $_GET['nonce'], 'yoda-an-nonce' ) ) {
                $error = new \WP_Error('yoda notice error:','Invalid nonce');
                wp_send_json_error($error);
                exit();
            }

            $search = sanitize_text_field($_GET['search']);
            if ($search) {
                $args = array(
                    'search'         => "*$search*",
                    'search_columns' => array( 'ID', 'display_name', 'user_email' ),
                );

                $user_query = new \WP_User_Query( $args );

                // Get the results
                $users = $user_query->get_results();

                $user_details = [];
                foreach ( $users as $user ) {
                    $user_details[] = [
                        'id' => $user->ID,
                        'text' => $user->display_name
                    ];
                }

                wp_send_json_success(json_encode($user_details));

            } else {
                $error = new \WP_Error('yoda notice error:','No input was given');
                wp_send_json_error($error);
            }

            exit();

        }

        /**
         *
         *
         * save_settings
         * save settings data to wp_option "yoda_an_settings"
         * @since 1.3.3
         * @param
         * @return void
         *
         *
         *
         */
        public function save_settings(): void {

            if (!empty($_POST)) {

                $data = $_POST;

                foreach ($data as $index => $post) {
                    if (!is_array($post)) {
                        $data[$index] = sanitize_text_field($post);
                    } else {
                        $data[$index] = array_map('sanitize_text_field', $post);
                    }
                }
                $return_page = $data['yoda_tool_page'];
                unset($data['yoda_tool_page']);

                //save data
                $res = update_option('yoda-tools-an-settings', $data);
                delete_option('yoda_tools_an_settings');

                if ($res) {
                    wp_redirect(get_admin_url().'admin.php?page='.$return_page.'&success=1');
                } else {
                    wp_redirect(get_admin_url().'admin.php?page='.$return_page);
                }

                exit();

            }
        }

        /**
         *
         *
         * set_field_status
         * @since 1.3.3
         * @param string - $data
         *        string - $tocheck
         *        string - $expected
         *
         * @return mixed [string | null]
         *
         *
         */
        public static function set_field_status($data, $tocheck, $expected, $default = false)
        {
            if (isset($data) && !empty($data)) {
                if ((int)$data === (int)$tocheck) {
                    return $expected;
                }
            } else {
                if ($default) {
                    return $expected;
                }
            }

            return null;
        }


        /**
         *
         *
         * admin_tab
         * @since 1.3.3
         *
         *
         */
        public function admin_tab( $admin_bar ) {

            if ($this->disable_to_user()) {
                return;
            }

            $admin_bar->add_menu(
                array(
                    'id'     => 'yoda_an_tab_number',
                    'title'  => esc_html__( 'Admin Notices', 'yoda-tools' ),
                    'href'   => '#',
                    'parent' => 'top-secondary',
                )
            );
        }


        /**
         *
         * load_notice_area
         * load notification html somewhere in the footer
         *
         *
         */
        public function load_notice_area()
        {
            if ($this->disable_to_user()) {
                return;
            }
            ?>
            <div class="yoda_an_notice_widget"></div>
            <?php
        }

        /**
         *
         *
         * show_notice
         * @since 1.3.3
         *
         *
         */
        public function show_notice ()
        {
            if (isset($_GET['success'])) {
                add_action( 'admin_notices', function(){
                    $class = 'notice notice-success is-dismissible';
                    $message = __('Changes has been saved','yoda-tools');
                    printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $message );
                } );
            }

        }

        /**
         *
         *
         * disable_to_user
         * @since 1.3.3
         *
         *
         */
        public function disable_to_user() : bool
        {
            if (!empty($this->settings)) {
                $user_id = get_current_user_id();
                switch ($this->settings['yoda_an_visibility_options']) {
                    case "2":
                        if (!empty($this->settings['yoda_an_hide_to_users'])) {
                            if (!in_array($user_id, $this->settings['yoda_an_hide_to_users'])) {
                                return true;
                            }
                        } else {
                            return true;
                        }
                        break;
                    case "3":
                        if (!empty($this->settings['yoda_an_hide_not_to_users'])) {
                            if (in_array($user_id, $this->settings['yoda_an_hide_not_to_users'])) {
                                return true;
                            }
                        } else {

                            return true;
                        }

                        break;
                }
            }
            return false;
        }

    }
?>