<?php 
	namespace yoda_tools\includes\core;
	use yoda_tools\includes\core\register;

	/* Does User Belong Here */
	if (!defined('YODA_TOOLS_PATH')) {
	    header('Status: 403 Forbidden');
	    header('HTTP/1.1 403 Forbidden');
	    exit();
	}

	class advancedSpamGuard
	{

	    /**
	     *
	     *
	     * $api
	     * will be our seo api object
	     * @since 1.5.5
	     *
	     */
	    private $api = null;


	    /**
	     *
	     *
	     * $apiKey
	     * this will be the user's api key
	     * @since 1.5.5
	     *
	     *
	     */
	    private string $apiKey = '';


	    /**
	     *
	     *
	     * spam_message
	     * @since 1.5.5
	     *
	     *
	     */
	    private string $spam_message = '';


	    public function __construct($apiKey)
	    {
	        $this->apiKey = $apiKey;
	        $this->api = new register();
	        $this->spam_message = __('YODA tools detected your submission as spam. If you find this a mistake please contact administrator', 'yoda-tools');
	    }

		/**
		 *
		 *
		 * user_registration_check
		 * Catch the user registration to check
		 * @param $user_login - string
		 *        $user_email - string
		 * @param $user_email
		 * @return boolean
		 * @since 1.5.5
		 */
	    function user_registration_check( $user_login, $user_email ): bool {
	        //let's process the user registration
	        $input = [
	            'type'              => 'user_registration',
	            'user_ip'           => $this->get_client_ip(),
	            'user_referrer'     => $this->get_client_referer(),
	            'user_agent'        => $this->detectBrowser(),
	            'data'              => [
	                'user_name'         => $user_login,
	                'user_email'        => $user_email
	            ]
	        ];

	        $this->api->set_api_key($this->apiKey);
	        $this->api->set_body(
	            [
	                'action'            => 'create',
	                'typeId'            => 'D0891CF5-5DE3-4D8E-9F1C-3552DE9C7CEE',
	                'input'             => json_encode($input),
	                'submissionDate'    => date('Y-m-d H:i:s'),
	                'scoreNotes'        => 'Passed Honey Pot',
					'report'			=> 'true'
	            ]
	        );
	        $response = $this->api->execute_seo_curl('spam');

	        if (!empty($response)) {
		        $response = json_decode($response);
	            return $response->rows->isSpam;
	        }

	        return false;

	    }

	    /**
	     *
	     *
	     * comment_check
	     * catch the incoming comment message to check
	     * @param array $data
	     * @return boolean
	     *
	     *
	     *@since 1.5.5
	     */
	    public function comment_check(array $data): bool {
	        $input = [
	            'type'      => 'comment',
	            'user_ip'           => $this->get_client_ip(),
	            'user_referrer'     => $this->get_client_referer(),
	            'user_agent'        => $this->detectBrowser(),
	            'data'      => [
	                'comment_author'        => $data['comment_author'],
	                'comment_author_email'  => $data['comment_author_email'],
	                'comment_author_url'    => $data['comment_author_url'],
	                'comment_content'       => $data['comment_content']
	            ]
	        ];

	        $this->api->set_api_key($this->apiKey);
	        $this->api->set_body(
	            [
	                'action'            => 'create',
	                'typeId'            => '3D4B6959-5565-408D-9FF0-35E1574973F2',
	                'input'             => json_encode($input),
	                'submissionDate'    => date('Y-m-d H:i:s'),
	                'scoreNotes'        => 'Passed Honey Pot',
					'report'			=> 'true'
	            ]
	        );
		    $response = $this->api->execute_seo_curl('spam');

	        if (!empty($response)) {
		        $response = json_decode($response);
	            return $response->rows->isSpam;
	        }

	        return false;
	    }

		/**
		 *
		 * DEPRECATED
		 * Reason - multiple calls occurred
		 * contact_form_seven
		 * let's check the contact form 7 submission for non ajax
		 *
		 * @param $contact_form
		 * @param $abort
		 * @return void
		 *
		 * @since 1.5.5
		 *
		 */
	    function check_contact_form_seven($contact_form, $abort)
	    {

	        $submission = \WPCF7_Submission::get_instance();
	        $posted_data = $submission->get_posted_data();

	        //prepare the data
	        $input = [
	            'type'      => 'form-submission',
	            'from'      => 'contact-form-7',
	            'user_ip'           => $this->get_client_ip(),
	            'user_referrer'     => $this->get_client_referer(),
	            'user_agent'        => $this->detectBrowser(),
	            'data'      => $posted_data
	        ];

	        $this->api->set_api_key($this->apiKey);
	        $this->api->set_body(
	            [
	                'action'            => 'create',
	                'typeId'            => '5EB94FED-DBD2-43BD-9EF1-C86C2CAD706F',
	                'input'             => json_encode($input),
	                'submissionDate'    => date('Y-m-d H:i:s'),
	                'scoreNotes'        => 'Passed Honey Pot',
					'report'			=> 'true'
	            ]
	        );
		    $response = $this->api->execute_seo_curl('spam');

	        if (!empty($response)) {
		        $response = json_decode($response);
	            if ($response->rows->isSpam) {

	                add_filter('wpcf7_skip_mail', function(){
	                    return true;
	                });

	                add_filter( 'wpcf7_form_response_output', function( $output, $class, $content, $object ){
	                    return sprintf(
	                        '<div class="wpcf7-response-output wpcf7-display-none wpcf7-mail-sent-ng" 
	                            role="alert" style="display: block;">%s</div>',
	                            $this->spam_message
	                    );
	                }, 10, 4 );

	            }
	        }


	    }

	    /**
	     *
	     *
	     * check_contact_form_seven_ajax
	     * check the submission via ajax
	     * @param array $post_data
	     * @return boolean
	     *
	     * @since 1.5.5
	     *
	     *
	     */

	    public function check_contact_form_seven_ajax($post_data): bool {
	        $posted_data = $post_data;

			//@since 1.6.5
			$form_id = (!empty($post_data['_wpcf7']))? $post_data['_wpcf7'] : 0;
			$form_name = '';
			if ($form_id > 0) {
				$form_post = get_post($form_id);
				$form_name = $form_post->post_title;
			}

	        $unset_these = ['_wpcf7', '_wpcf7_version', '_wpcf7_locale', '_wpcf7_unit_tag', '_wpcf7_container_post', '_wpcf7_posted_data_hash'];
	        foreach ($unset_these as $unset) {
	            unset($posted_data[$unset]);
	        }

	        //prepare the data
			//@since 1.6.5
	        $input = [
	            'type'      		=> 'form-submission',
	            'from_plugin'     	=> 'contact-form-7',
				'from'				=> $form_name,
				'from_form_id'		=> $form_id,
	            'user_ip'           => $this->get_client_ip(),
	            'user_referrer'     => $this->get_client_referer(),
	            'user_agent'        => $this->detectBrowser(),
	            'data'      		=> $posted_data
	        ];

	        $this->api->set_api_key($this->apiKey);
	        $this->api->set_body(
	            [
	                'action'            => 'create',
	                'typeId'            => '5EB94FED-DBD2-43BD-9EF1-C86C2CAD706F',
	                'input'             => json_encode($input),
	                'submissionDate'    => date('Y-m-d H:i:s'),
	                'scoreNotes'        => 'Passed Honey Pot',
					'report'			=> 'true'
	            ]
	        );
	        $response = $this->api->execute_seo_curl('spam');

	        if (!empty($response)) {
	            $response = json_decode($response);
	            return $response->rows->isSpam;
	        }
	        return false;

	    }

		/**
		 *
		 *
		 * gravity_form_check
		 * check all submission for gravity form
		 * @param array $data - $data
		 * @param $form
		 * @return boolean
		 *
		 * @since 1.5.5
		 *
		 */
	    public function gravity_form_check($data, $form): bool {
	        $unset_these = ['gform_ajax', 'is_submit_2', 'gform_submit', 'gform_unique_id', 'state_2', 'gform_target_page_number_2','gform_source_page_number_2','gform_field_values','WwVkQdP62N','version_hash'];

	        $get_posted = $data;

	        foreach ($unset_these as $unset) {
	            if (isset($get_posted[$unset])) {
	                unset($get_posted[$unset]);
	            }
	        }

			//@since 1.6.5
	        $input = [
	            'type'      		=> 'form-submission',
	            'from_plugin'      	=> 'gravity-forms',
				'from'				=> $form['title'],
				'from_form_id'		=> $form['id'],
	            'user_ip'           => $this->get_client_ip(),
	            'user_referrer'     => $this->get_client_referer(),
	            'user_agent'        => $this->detectBrowser(),
	            'data'      		=> $get_posted
	        ];

	        $this->api->set_api_key($this->apiKey);
	        $this->api->set_body(
	            [
	                'action'            => 'create',
	                'typeId'            => '5EB94FED-DBD2-43BD-9EF1-C86C2CAD706F',
	                'input'             => json_encode($input),
	                'submissionDate'    => date('Y-m-d H:i:s'),
	                'scoreNotes'        => 'Passed Honey Pot',
					'report'			=> 'true'
	            ]
	        );
	        $response = $this->api->execute_seo_curl('spam');

	        if (!empty($response)) {
	            $response = json_decode($response);
	            return $response->rows->isSpam;
	        }
	        return false;
	    }


	    /**
	     *
	     *
	     * NOTE: This is disabled for now
	     * check_login
	     * check all wp login form for BOT
	     * @param array $posted_data - $posted_data
	     * @return boolean
	     *
	     * @since 1.5.5
	     *
	     *
	     */
	    public function check_login($posted_data = []): bool {

	        if (empty($posted_data))
	            return false;

	        $input = [
	            'type'      => 'wp_login',
	            'user_ip'           => $this->get_client_ip(),
	            'user_referrer'     => $this->get_client_referer(),
	            'user_agent'        => $this->detectBrowser(),
	            'data'      => [
	                'username'      => $posted_data['log'],
	                'password'      => $posted_data['pwd']
	            ]
	        ];

	        $this->api->set_api_key($this->apiKey);
	        $this->api->set_body(
	            [
	                'action'            => 'create',
	                'typeId'            => 'D0891CF5-5DE3-4D8E-9F1C-3552DE9C7CEE',
	                'input'             => json_encode($input),
	                'submissionDate'    => date('Y-m-d H:i:s'),
	                'scoreNotes'        => 'Passed Honey Pot'
	            ]
	        );
	        $response = $this->api->execute_seo_curl('spam');

	        if (!empty($response)) {
	            $response = json_decode($response);
	            return $response->rows->isSpam;
	        }

	        return false;
	    }

	    /**
	     *
	     *
	     * get_client_ip
	     * get client's IP address
	     * @param
	     * @return mixed - bool|array|string
	     *
	     * @since 1.5.5
	     *
	     *
	     */
	    public function get_client_ip()
	    {
	        $environments = ['HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR'];
	        $ipaddress = 'UNKNOWN';
	        foreach ($environments as $env) {
	            if ( getenv($env) ) {
	                $ipaddress = getenv($env);
	                break;
	            }
	        }
	        return filter_var($ipaddress, FILTER_VALIDATE_IP);
	    }

	    /**
	     *
	     *
	     * get_client_referer
	     * get client's doman or referrer of the submission
	     * @param
	     * @return string
	     *
	     * @since 1.5.5
	     *
	     *
	     */
	    public function get_client_referer() {
	        if ( isset($_SERVER['HTTP_REFERER'] ) ) {
	            $fnDomain = $_SERVER['HTTP_REFERER'];
	        } else if(getenv('HTTP_REFERER')) {
	            $fnDomain = getenv('HTTP_REFERER');
	        } else {
	            $fnDomain = 'UNKNOWN';
	        }
	        return esc_url($fnDomain);
	    }


		/**
		 *
		 *
		 * detectBrowser
		 * get client user agent or browser
		 * @return string
		 * @since 1.5.5
		 *
		 */
	    public static function detectBrowser()
	    {
	        if( !isset($_SERVER['HTTP_USER_AGENT']) ) {
	            return "UNKNOWN";
	        } else {
	            return sanitize_text_field($_SERVER['HTTP_USER_AGENT']);
	        }
	    }

	}
