<?php
	namespace yoda_tools\includes\core;

	/* Does User Belong Here */
	if (!defined('YODA_TOOLS_PATH')) {
		header('Status: 403 Forbidden');
		header('HTTP/1.1 403 Forbidden');
		exit();
	}


	class register{

		private string $logout_url = "https://studio.maennche.com/api/v2/api/logout/";

		/**
		 *
		 *
		 * auth_url
		 * this will be the api url where we request the authentication
		 * @since 1.4.0
		 *
		 *
		 */

		private string $auth_url = "https://studio.maennche.com/api/v2/api/authenticate";


		/**
		 *
		 *
		 * seo_url
		 * this will be the api url to generate SEO title and descriptions
		 * @since 1.4.0
		 *
		 *
		 */

		private string $seo_url = "https://studio.maennche.com/api/v2/seo-tools/generate/";


		/**
		 *
		 *
		 * credits_url
		 * api url to get the credits available
		 * @since 1.4.0
		 *
		 *
		 */
		//Deprecating Credits
		//private string $credits_url = 'https://studio.maennche.com/api/v2/credits/total/?fields=id';


		/**
		 *
		 *
		 * spam_guard
		 * api url to process the input to check if spam
		 * @since 1.5.5
		 *
		 *
		 */
		private string $spamguard_url = 'https://studio.maennche.com/api/v2/spam-guard-submission/';

		/**
		 * 
		 * 
		 * site_details
		 * the api endpoint of site details
		 * @since 1.5.6
		 * 
		 * 
		 */
		private string $site_details = 'https://studio.maennche.com/api/v2/tenant-domain/site-details/';

		/**
		 *
		 *
		 * auth_username
		 * username of the user
		 * @since 1.4.0
		 *
		 *
		 */
		private string $auth_username = '';


		/**
		 *
		 *
		 * auth_password
		 * username of the user
		 * @since 1.4.0
		 *
		 *
		 */
		private string $auth_password = '';

		/**
		 *
		 *
		 * auth_deviceID
		 * username of the user
		 * @since 1.4.0
		 *
		 *
		 */
		private string $auth_deviceID = '';


		/**
		 *
		 *
		 * seo_body
		 * body parameters of SEO request
		 * @since 1.4.0
		 *
		 *
		 */
		private array $seo_body = [];


		/**
		 *
		 *
		 * api_key
		 * api key of the SEO request
		 * @since 1.4.0
		 *
		 *
		 */
		private string $api_key = '';

		/**
		 *
		 *
		 * set_username
		 * @since 1.4.0
		 * @param $username - string
		 *
		 *
		 */
		public function set_username($username) {
			$this->auth_username = $username;
		}

		/**
		 *
		 *
		 * set_password
		 * @since 1.4.0
		 * @param $password - string
		 *
		 *
		 */
		public function set_password($password) {
			$this->auth_password = $password;
		}

		/**
		 *
		 *
		 * set_deviceID
		 * @since 1.4.0
		 * @param $deviceID - string
		 *
		 *
		 */
		public function set_deviceID($deviceID) {
			$this->auth_deviceID = $deviceID;
		}


		/**
		 *
		 *
		 * set_api_key
		 * set the api key value
		 * @since 1.4.0
		 * @param $api_key - string
		 *
		 *
		 */
		public function set_api_key($api_key)
		{
			$this->api_key = $api_key;
		}


		/**
		 *
		 *
		 * set_body
		 * @since 1.4.0
		 * @param $body - array
		 *
		 *
		 */
		public function set_body($body)
		{
			$this->seo_body = $body;
		}


		/**
		 *
		 *
		 * execute_curl
		 * @param $url - string
		 * @param null $header
		 * @return bool|string
		 * @since 1.4.0
		 */
		private function execute_curl($url, $header = null)
		{
			$ch = curl_init();

			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

			if ($header) {
				curl_setopt($ch, CURLOPT_HTTPHEADER, array(
					'api_key: ' . $this->api_key
				));
			}

			$response = curl_exec($ch);
			curl_close($ch);
			return $response;
		}


		/**
		 *
		 *
		 * execute_seo_curl
		 * @param string $endpoint
		 * @return string
		 *
		 * @since 1.4.0
		 */
		public function execute_seo_curl(string $endpoint = 'seo'): string {

			switch ($endpoint) {
				case 'site_details':
					$endpoint_url = $this->site_details;
					break;
				case 'spam':
					$endpoint_url = $this->spamguard_url;
					break;
				default:
					$endpoint_url = $this->seo_url;
					break;
			}

			// Initialize curl
			$ch = curl_init();

			// Set curl options
			curl_setopt($ch, CURLOPT_URL, $endpoint_url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($this->seo_body));
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
				'api_key: ' . $this->api_key,
			));

			// Execute the curl request
			$response = curl_exec($ch);
			$status_code = curl_getinfo($ch, CURLINFO_RESPONSE_CODE);
			error_log('code: '.$status_code);

			// Close curl
			curl_close($ch);

			return $response;

		}

		/**
		 *
		 *
		 * authenticate		 *  the user's credentials
		 * @since 1.4.0
		 * @param
		 * @return string array
		 *
		 *
		 */
		/**
		 *
		 *
		 * authenticate
		 * authenticates the user's credentials
		 * @return array|false|string
		 *
		 * @since 1.4.0
		 */
		public function authenticate() {
			// Set the endpoint URL and query parameters

			if (!$this->auth_password || !$this->auth_username) {
				return json_encode(['status' => 'failed', 'msg' => __('No credentials found','yoda-tools')]);
			}
			$parameters = http_build_query(
				[
					'fields' => 'id',
					'password' => $this->auth_password,
					'MobileDeviceId' => $this->auth_deviceID,
					'email' => $this->auth_username
				]
			);

			$url = $this->auth_url."/?{$parameters}";
			$response = $this->execute_curl($url);
			$response_ = $response;
			if (!empty($response)) {
				if (is_array($response)) {
					return $response;
				} else {
					$response = stripslashes($response); // is this the trick that fixes the issue? hmmmm
					$response = str_replace(["\n", "\r"],'',$response); //removing the new line from response time
					$res = json_decode($response);
					$apikey = null;

					try {
						//the response json seems changing. I have to catch the possible error.
						if (isset($res->rows)) {
							if (isset($res->rows[0])) {
								$apikey = $res->rows[0]->apiKey;
							} else if (isset($res->rows->apiKey)){
								$apikey = $res->rows->apiKey;
							}
						}
					} catch(\Exception $e) {
						error_log('Message: ' .$e->getMessage()) ;
					}

					if ($apikey) {
						return ['status' => 'success', 'apiKey' => $apikey];
					} else {
						return ['status' => 'failed', 'msg' => __('No apiKey is found', 'yoda-tools')];
					}
				}

			} else {
				return ['status' => 'failed', 'msg' => __('Invalid credential', 'yoda-tools')];
			}

		}


		/**
		 *
		 *
		 * logout
		 * @since 1.4.0
		 *
		 *
		 */
		public function logout()
		{
			$url = $this->logout_url;
			$response = $this->execute_curl($url, true);
			return $response;
		}

		/**
		 *
		 * get_credits
		 * @since 1.4.0
		 *
		 */
		public function get_credits(): int {
			//$response = $this->execute_curl($this->credits_url, true);
			//if ($response) {
				//$remaining = json_decode($response);
				//if (!empty($remaining->rows)) {
					//return $remaining->rows[0];
				//}
			//}
			//Deprecating Credits
			return 1000;
		}


	}