<?php 
    namespace yoda_tools\includes\core;

    use ParagonIE\Sodium\Core\Curve25519\Ge\P2;
    use yoda_tools\includes\core\register;
    use yoda_tools\includes\cron_tracking;
    use yoda_tools\includes\core\settings;
    use yoda_tools\includes\core\utilities;

    /* Does User Belong Here */
    if (!defined('YODA_TOOLS_PATH')) {
        header('Status: 403 Forbidden');
        header('HTTP/1.1 403 Forbidden');
        exit();
    }

    class seoSmartValidator
    {

        /**
         *
         *
         * seo_plugins
         * the list of supported 3rd party wordpress SEO plugins
         * feel free to add if new SEO plugin will come out.
         * This is subject to update if the plugin listed made changes to their classes
         * @since 1.4.0
         *
         *
         */
        private array $seo_plugin_metas = [
            'wordpress-seo/wp-seo.php' => [
                'title' => '_yoast_wpseo_title',
                'description' => '_yoast_wpseo_metadesc',
                'tax_titles' => [
                    'categories' => [
                        'title' => 'title-tax-category',
                        'description' => 'metadesc-tax-category',
                        'yoda_tax_title_fix' => '%%term_title%% Archives %%page%% %%sep%% %%sitename%%',
                        'yoda_tax_description_fix' => 'Are you looking tax for %%term_title%% from %%sitename%%? Then this is the link that you need to click!'
                    ],
                    'tags' => [
                        'title' => 'title-tax-post_tag',
                        'description' => 'metadesc-tax-post_tag',
                        'yoda_tag_title_fix' => '%%term_title%% Archives %%page%% %%sep%% %%sitename%%',
                        'yoda_tag_description_fix' => 'Are you looking tag for %%term_title%% from %%sitename%%? Then this is the link that you need to click!'
                    ]
                ],
                'no_index' => '_yoast_wpseo_meta-robots-noindex'
            ],
            'all-in-one-seo-pack/all_in_one_seo_pack.php' => [
                'title' => '_aioseo_title',
                'description' => '_aioseo_description'
            ]
        ];
        private string $aioseo_id = 'all-in-one-seo-pack/all_in_one_seo_pack.php';


        /**
         *
         *
         * to_log
         * let's store all data here
         * @since 1.4.0
         *
         *
         */
        private array $to_log = [];


        /**
         *
         * all_posts
         * @since 1.4.0
         *
         */
        private array $all_posts = [];



        /**
         *
         *
         * has_supported
         * @since 1.4.0
         *
         *
         */
        private bool $has_supported = false;



        /**
         *
         *
         * maximumt_meta_length
         * @since 1.4.0
         *
         *
         */
        private int $maximumt_meta_length = 160;




        /**
         *
         *
         * supported_post_type
         * @since 1.4.0
         *
         *
         */
        private array $supported_post_type = ['page', 'post'];



        /**
         *
         *
         * seo_api
         * @since 1.4.0
         *
         *
         */
        private $seo_api = null;



        /**
         *
         *
         * seo_api_key
         * @since 1.4.0
         *
         *
         */
        private string $seo_api_key = '';



        /**
         *
         *
         * credits
         * @since 1.4.0
         *
         *
         */
        private $credits = null;


        /**
         *
         *
         * auto_process
         * @since 1.4.0
         *
         *
         */
        private $auto_process = null;


        /**
         *
         *
         * statuses
         * @since 1.4.0
         *
         *
         */
        private array $statuses = [
            'pending'               => 'Pending',
            'processing'            => 'Processing',
            'failed'                => 'Failed',
            'dismissed'             => 'Dismissed',
            'completed'             => 'Completed',
            'requires-attention'    => 'Requires Attention'
        ];



        /**
         *
         *
         * seo_meta_key
         * where we save the log meta data
         * @since 1.4.0
         *
         *
         */
        private string $seo_meta_key = 'yoda-seo-log-details';



        /**
         *
         *
         * seo_custom_meta_key
         * this meta key for our own generated metas if no 3rd party plugin installed
         * @since 1.4.0
         *
         *
         */
        private string $seo_custom_meta_key = 'yoda_tools_smart_seo_description';


        /**
         *
         *
         * seo_custom_title_meta_key
         * this meta key for our own generate metas if no 3rd party plugin installed
         * @since 1.4.0
         *
         *
         */
        private string $seo_custom_title_meta_key = 'yoda_tools_smart_seo_title';


        /**
         *
         *
         * pending_schedules
         * store all pending schedules here and load once only
         * @since 1.5.0
         *
         *
         */
        private array $pending_schedules = [];


        /**
         *
         * max_schuduled_number
         * store the number of logs to process
         * @since 1.5.0
         *
         */
        private int $max_schuduled_number = 10;


        /**
         *
         *
         * $to_length_review
         * store all posts that have title and description but have to check the length
         * @since 1.5.3
         *
         *
         */
        private array $to_length_review = [];


        /**
         *
         *
         * max_run_Time
         * let's store the max_run_time_here
         * @since 1.5.3
         *
         *
         */
        private int $max_run_Time = 80;


        /**
         *
         *
         * post_with_title_meta
         * holds all post with supported title
         * @since 1.5.3
         *
         *
         */
        private array $post_with_title_meta = [];


        /**
         *
         *
         * post_with_description_meta
         * holds all post with supported description
         * @since 1.5.3
         *
         *
         */
        private array $post_with_description_meta = [];



        public function __construct($apiKey = null, $credit_settings = null)
        {
            //customize cron interval
            add_filter( 'cron_schedules', [$this, 'event_processor_schedule'] );

            $this->seo_api_key = $apiKey;
            $this->seo_api = new register();

            $this->credits = (isset($credit_settings['credits']))? $credit_settings['credits'] : 'all';
            $this->auto_process = (isset($credit_settings['auto_process']))? $credit_settings['auto_process'] : 'N';

            //lets load the yodaSEO js
            add_action( 'admin_enqueue_scripts', [$this, 'load_assets']);

            //build the table
            $this->register_post();
            $this->register_statuses();
            add_action('admin_footer-post.php', [$this, 'render_custom_status']);
            add_action('admin_head', [$this, 'remove_add_new_button']);

            //create crone
            $this->create_seo_validator_schedule();
            add_action('yoda_seo_smart_validator', [$this, 'do_validation']);
            add_action('yoda_seo_smart_processor', [$this, 'do_processing']);

            if ($this->auto_process === 'Y') {
                //load pending schedules
                $this->pending_schedules = $this->event_schedule_next_call();
            }

            //we render the custom description
            add_action('wp_head', [$this, 'render_custom_meta_tags']);

            //we render the custom title for seo
            add_filter( 'document_title_parts', [$this, 'render_custom_meta_title'], 10, 1 );

            //hide quick edit links
            add_filter('post_row_actions', [$this, 'hide_edit_link'], 10, 1);

            //hide edit from dropdown
            add_filter('bulk_actions-edit-yoda-seo', [$this, 'hide_edit_dropdown'], 10, 1);

            //add custom columns
            add_filter('manage_yoda-seo_posts_columns', [$this, 'new_column']);
            add_action('manage_yoda-seo_posts_custom_column', [$this, 'manage_column'], 10, 2);
            add_filter('manage_edit-yoda-seo_sortable_columns', [$this, 'sortable_columns'] );

            //add custom link under title column
            add_filter('post_row_actions', [$this, 'add_action_links'], 10, 2 );

            //add custom option in bulk action
            //we don't need this for now
            add_filter('bulk_actions-edit-yoda-seo', [$this, 'add_bulk_process']);

            //let's customize the secondary save button
            add_filter('gettext', [$this,'rename_save_draft_button'], 20, 3 );

            //let's remove the add new button and link - for the reason we are adding theme during scanning
            add_action('admin_menu', [$this, 'remove_add_new'] );

            //dismiss log
            add_action('wp_ajax_yoda_tools_seo_dismiss', [$this, 'dismiss_log']);
            //process via ajax
            add_action('wp_ajax_yoda_tools_seo_process', [$this, 'process_log']);

            //show meta data to log
            add_action('add_meta_boxes', [$this,'meta_box']);

            //add custom button
            add_action('manage_posts_extra_tablenav', [$this, 'custom_ui'], 10, 1);

            //render AIOSEO Title
            add_filter('aioseo_title', [$this, 'aio_render_ai_title']);

            //render AIOSEO Description
            add_filter( 'aioseo_description', [$this, 'aioseo_render_ai_description'] );

            //create cron for site details
            $this->create_site_details_cron();
            add_action('yoda_site_details_check', [$this, 'send_site_details']);

            //process the bulk action
            add_action('handle_bulk_actions-edit-yoda-seo',[$this, 'process_bulk'], 0, 3);

            //process the bulk from new location
            add_action('admin_init', [$this, 'process_yoda_bulk']);

            //add widget to dashboard
            add_action('wp_dashboard_setup', [$this, 'required_attention']);

            //SEO save overview without generating
            add_action('wp_ajax_yoda_tools_save_overview', [$this, 'save_overview']);

            //get seo suggestion
            add_action('wp_ajax_yoda_seo_ask_suggestion', [$this, 'get_suggestion']);

            //apply seo suggestion
            add_action('wp_ajax_yoda_seo_apply_suggestion', [$this, 'apply_suggestion']);

        }

        /**
         *
         *
         * load_assets
         * @since 1.4.0
         *
         *
         */
        public function load_assets()
        {
            if (!is_admin()) {
                return;
            }

            $screen = get_current_screen();
            $yoda_seo_page = (isset($_GET['page']))? $_GET['page'] : null;
            if ( ($screen->base === 'edit'  && $screen->post_type === 'yoda-seo') ||  $yoda_seo_page === 'yoda-tools-seo-tools' || $screen->base === 'dashboard') {
                wp_register_script( YODA_TOOLS_SLUG.'-yoda-seo-js' , YODA_TOOLS_URL.'js/yodaSeo.js', array('jquery') );
                wp_enqueue_script( YODA_TOOLS_SLUG.'-yoda-seo-js' );
                wp_enqueue_style( YODA_TOOLS_SLUG.'-yoda-seo-css', YODA_TOOLS_URL.'css/yodaSeo.css' );
                wp_localize_script( YODA_TOOLS_SLUG.'-yoda-seo-js', 'yodaSeoNonce', [wp_create_nonce('yoda-seo-nonce')]);
                wp_localize_script( YODA_TOOLS_SLUG.'-yoda-seo-js', 'yodaSeoAjaxURL', [admin_url( 'admin-ajax.php' )]);
                wp_enqueue_script('jquery-ui-core');
                wp_enqueue_script('jquery-ui-datepicker');
                wp_enqueue_style('jquery-datepicker-ui-css', YODA_TOOLS_URL.'includes/lib/datepicker/datepicker.css');
            }

        }

        /**
         *
         *
         * meta_box
         * show the meta data of log post
         * @since 1.4.0
         *
         *
         */
        public function meta_box()
        {
            add_meta_box('yoda-seo-metadata', __('Log Data', 'yoda-seo'), [$this,'metda_details'], 'yoda-seo', 'normal', 'core' );
        }

        /**
         *
         *
         * metda_details
         * render the meta data to html
         * @since 1.4.0
         *
         *
         */
        public function metda_details($post)
        {
            $details = get_post_meta($post->ID, $this->seo_meta_key, true);

            $output = [];
            if ($details['scope'] === 'global') {
                $desc = get_post_meta($details['post_id'], $this->seo_custom_meta_key, true);
                if (!empty($desc)) {
                    $output[] = ['description' => $desc];
                }
                $title = get_post_meta($details['post_id'], $this->seo_custom_title_meta_key, true);
                if (!empty($title)) {
                    $output[] = ['title' => $title];
                }
                if (empty($output)) {
                    //let's take the content from supported plugins
                    $installed_metas = $this->check_supported_plugins();

                    foreach ($installed_metas as $metas) {
                        $title = get_post_meta($details['post_id'], $metas['title'], true);
                        if ($title && $details['attributes'] === 'title') {
                            $output[] = ['title' => $title];
                        }

                        $description = get_post_meta($details['post_id'], $metas['description'], true);
                        if ($description && $details['attributes'] === 'description') {
                            $output[] = ['description' => $description];
                        }
                    }
                }
            }
            if ($details['object'] === 'menu') {
                //get excerpt of the menu
                $excerpt = get_the_excerpt($details['post_id']);
                $output[] = ['title' => $excerpt];
            }
            if ($details['object'] === 'meta' && empty($details['scope'])) {
                //get supported plugin meta details
                foreach ($this->seo_plugin_metas as $key) {

                    $get_title = get_post_meta($details['post_id'], $key['title'], true);
                    if ($get_title && $details['attributes'] === 'title') {
                        $output[] = ['title' => $get_title];
                    }

                    $get_description = get_post_meta($details['post_id'], $key['description'], true);
                    if ($get_description && $details['attributes'] === 'description') {
                        $output[] = ['description' => $get_title];
                    }

                }

            }
            include_once(YODA_TOOLS_PATH.'/views/seo-log.php');
        }


        /**log_details
         *
         *
         * remove_add_new
         * let's remove the add new button and link - for the reason we are adding theme during scanning
         * @since 1.4.0
         *
         *
         */
        public function remove_add_new()
        {
            remove_submenu_page( 'edit.php?post_type=yoda-seo', 'post-new.php?post_type=yoda-seo' );
        }


        /**
         *
         *
         * rename_save_draft_button
         * let's rename the draft button according to current post status
         * @since 1.4.0
         *
         *
         */
        public function rename_save_draft_button( $translated_text, $text, $domain)
        {
            global $post;
            if (isset($post->post_type)) {
                if ( $post->post_type === 'yoda-seo' && 'Save Draft' === $text ) {
                    if ($post->post_status) {
                        $status = ($post->post_status === 'auto-draft')? 'Draft' : $this->statuses[$post->post_status];
                    } else {
                        $status = 'Draft';
                    }

                    $translated_text = __( 'Save as '.$status, 'yoda-tools' );
                }
            }

            return $translated_text;

        }


        /**
         *
         *
         * hide_edit_link
         * let's hide the quick edit
         * @since 1.4.0
         *
         *
         */
        public function hide_edit_link($actions)
        {
            global $post_type;
            if ( $post_type === 'yoda-seo' ) {
                unset( $actions['inline hide-if-no-js'] );
                unset( $actions['edit'] );
                unset( $actions['trash'] );
            }
            return $actions;
        }

        /**
         *
         *
         * hide_edit_dropdown
         * let's hide edit from dropdown
         * @since 1.4.0
         *
         *
         */
        public function hide_edit_dropdown($actions)
        {
            global $post_type;
            if ( $post_type === 'yoda-seo' ) {
                unset( $actions['edit'] );
            }
            return $actions;
        }

        /**
         *
         *
         * new_column
         * lets add custom columns
         * @since 1.4.0
         *
         *
         */
        function new_column($columns)
        {
            $columns = [
                'cb' => $columns['cb'],
                'title' => $columns['title'],
                'yoda_seo_object' => 'Object',
                'yoda_seo_attributes' => 'Attributes',
                'yoda_seo_type' => 'Post type',
                'yoda_seo_schedule' => 'Schedule',
                'date' => $columns['date'],
            ];
            return $columns;
        }

        /**
         *
         *
         * manage_column
         * manage the content of custom columns
         * @since 1.4.0
         *
         *
         */
        public function manage_column($column_name, $post_id)
        {

            $seo_details = get_post_meta($post_id,$this->seo_meta_key, true);

            if ( $column_name === 'yoda_seo_object' ) {
                if (isset($seo_details['object'])) {
                    echo $seo_details['object'];
                }
            }

            if ( $column_name === 'yoda_seo_attributes' ) {
                if (isset($seo_details['attributes'])) {
                    echo $seo_details['attributes'];
                }
            }

            if ( $column_name === 'yoda_seo_type' ) {
                if (isset($seo_details['post_id'])) {
                    echo get_post_type($seo_details['post_id']);
                }
            }

            if ( $column_name === 'yoda_seo_schedule' ) {
                $status = get_post_status($post_id);
                if ($status === 'pending') {
                    if ($this->auto_process === 'Y') {
                        echo (!empty($this->pending_schedules[$post_id]))? $this->pending_schedules[$post_id] : 'Preparing schedule...';
                    } else {
                        echo "Not scheduled";
                    }

                }

            }
        }

        /**
         *
         *
         * sortable_columns
         * let's make the custom columns sortable
         * @since 1.4.0
         *
         *
         */
        public function sortable_columns($columns)
        {
            $columns['yoda_seo_object'] = __('Object','yoda-tools');
            $columns['yoda_seo_attributes'] = __('Attributes','yoda-tools');
            $columns['yoda_seo_type'] = __('Post Type','yoda-tools');
                return $columns;
        }

        /**
         *
         *
         * add_action_links
         * let's add custom action link under title column
         * @since 1.4.0
         *
         *
         */
        public function add_action_links($actions, $post)
        {
            if ( $post->post_type == 'yoda-seo' ) {

                // add process and dismissed link for pending and draft???
                if ($post->post_status === 'pending' || $post->post_status === 'draft' || $post->post_status === 'processing') {
                    $actions['yoda-seo-process'] = '<a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-process" data-id="'.$post->ID.'" data-nonce="'.wp_create_nonce('yoda-seo-report-action').'">'.esc_attr__('Process','yoda-tools').'</a>';
                    $actions['yoda-seo-dismiss'] = '<a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-dismiss" data-id="'.$post->ID.'" data-nonce="'.wp_create_nonce('yoda-seo-report-action').'">'.esc_attr__('Dismiss','yoda-tools').'</a>';
                }

                //add process link for dismissed status
                if ($post->post_status === 'dismissed') {
                    $actions['yoda-seo-process'] = '<a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-process" data-id="'.$post->ID.'" data-nonce="'.wp_create_nonce('yoda-seo-report-action').'">'.esc_attr__('Process','yoda-tools').'</a>';
                }

                //add view action link when the log is completed
                $log_details = get_post_meta($post->ID,$this->seo_meta_key, true);
                switch ($log_details['object']) {
                    case 'duplicate':
                    case 'rewrite':
                    case 'meta':
                        $edit_url = get_edit_post_link($log_details['post_id']);
                        $actions['yoda-seo-view'] = '<a href="'.$edit_url.'" class="yoda-seo-action">'.esc_attr__('View Post','yoda-tools').'</a>';
                        break;
                    case 'menu':
                        $parent_menu_id = $this->get_parent_menu($log_details['post_id']);
                        $edit_url = admin_url('nav-menus.php?action=edit&menu='.$parent_menu_id);
                        $actions['yoda-seo-view'] = '<a href="'.$edit_url.'" class="yoda-seo-action">'.esc_attr__('View Post','yoda-tools').'</a>';
                        break;
                    case 'image':
                        //we don't do image for now let's come back later.
                        break;
                }


            }
            return $actions;
        }

        /**
         *
         *
         * add_bulk_process
         * let's add process option inside bulk action dropdown
         * @since 1.4.0
         *
         *
         */
        function add_bulk_process( $actions ) {


            global $post_type;
            if ( $post_type == 'yoda-seo' && !isset($actions['untrash']) ) {
                // Add your custom action
                $actions = [
                    'yoda-bulk-process' => __('Process', 'yoda-tools'),
                    'yoda-bulk-dismiss' => __('Dismiss', 'yoda-tools'),
                    'trash' => $actions['trash']
                ];

            }
            return $actions;

        }



        /**
         *
         *
         * build_table
         * build the log table for missing SEO elements
         * @since 1.4.0
         * @param
         * @return
         *
         *
         */
        public function register_post()
        {
            $labels = array(
                'name'               => _x( 'SEO Suggestions', 'SEO Suggestions', 'yoda-tools' ),
                'singular_name'      => _x( 'SEO Suggestion', 'SEO Suggestion', 'yoda-tools' ),
                'menu_name'          => _x( 'SEO Suggestions', 'yoda-seo', 'yoda-tools' ),
                'name_admin_bar'     => _x( 'SEO Suggestions', 'add new yoda seo', 'yoda-tools' ),
                'add_new'            => _x( 'Add New', 'SEO Suggestion', 'yoda-tools' ),
                'add_new_item'       => __( 'Add New SEO Suggestion', 'yoda-tools' ),
                'new_item'           => __( 'New SEO Suggestion', 'yoda-tools' ),
                'edit_item'          => __( 'SEO Suggestion', 'yoda-tools' ),
                'view_item'          => __( 'View SEO Suggestions', 'yoda-tools' ),
                'all_items'          => __( 'All SEO Suggestions', 'yoda-tools' ),
                'search_items'       => __( 'Search SEO Suggestions', 'yoda-tools' ),
                'parent_item_colon'  => __( 'Parent SEO Suggestion:', 'yoda-tools' ),
                'not_found'          => __( 'No SEO Suggestions Found.', 'yoda-tools' ),
                'not_found_in_trash' => __( 'No SEO Suggestions Found in trash.', 'yoda-tools' )
            );

            $args = array(
                'labels'             => $labels,
                'description'        => __( 'Description.', 'yoda-tools' ),
                'public'             => false,
                'publicly_queryable' => false,
                'show_ui'            => true,
                'show_in_menu'       => false,
                'query_var'          => false,
                'rewrite'            => array( 'slug' => 'yoda-seo' ),
                'menu_icon'			 => 'dashicons-code-standards',
                'capability_type'    => 'post',
                'has_archive'        => false,
                'hierarchical'       => false,
                'menu_position'      => null,
                'supports'           => array( 'title' )
            );
            register_post_type( 'yoda-seo', $args );
            add_filter( 'display_post_states', [$this,'render_list_status'], 10, 1);
        }

        /**
         *
         *
         * register_statuses
         * @since 1.4.0
         *
         *
         */
        public function register_statuses()
        {
            foreach ($this->statuses as $i =>  $status) {

                register_post_status( $i, array(
                    'label'                     => _x( $status, 'yoda-tools' ),
                    'public'                    => true,
                    'exclude_from_search'       => true,
                    'show_in_admin_all_list'    => true,
                    'show_in_admin_status_list' => true,
                    'label_count'               => _n_noop( $status.' <span class="count">(%s)</span>', $status.' <span class="count">(%s)</span>' ),
                    'post_type'                 => 'yoda-seo'
                ) );

            }


        }

        /**
         *
         *
         * render_custom_status
         * //customize report HTML UI by changing the dropdown content of the status and button labels
         * @since 1.4.0
         *
         *
         */
        public function render_custom_status()
        {
            global $post;

            if (!isset($post)) {
                return;
            }

            if($post->post_type === 'yoda-seo'){
                $options = '';
                foreach ($this->statuses as $i => $status) {
                    $selected = ($post->post_status === $i)? 'selected' : null;
                    $options .= '<option value="'.$i.'" '.$selected.'>'.$status.'</span>';
                }
                $label = $post->post_status;
                ?>
                <script type="text/javascript">
                    jQuery(document).ready(function(){
                        jQuery('#post_status option').each(function(){
                            if (jQuery(this).val() === 'draft' || jQuery(this).text() === 'Pending Review') {
                                jQuery(this).remove();
                            }
                        });
                        jQuery('.save-post-status').click(function(event){
                            let new_label = jQuery('#post_status option:selected').text();
                            jQuery('#save-post').val('Save as '+new_label);
                        });
                        jQuery("select#post_status").append('<?php echo $options; ?>');
                        jQuery(".misc-pub-section label").append("<?php echo $label; ?>");
                        <?php if (isset($this->statuses[$label])): ?>
                            jQuery("#post-status-display").text('<?php echo $this->statuses[$label]; ?>');
                        <?php endif; ?>
                    });
                </script>
                <?php
            }
        }

        /**
         *
         *
         * remove_add_new_button
         * let's remove the add new button next to yoda seo title
         * we are doing this because we are adding these stuffs inside even scheduler
         *
         * in here we also hack the menu to appear the YODA tools is the current
         *
         * @since 1.4.0
         *
         *
         */
        public function remove_add_new_button()
        {
            $screen = get_current_screen();
            $post = (isset($_GET['post']))? $_GET['post'] : 0;
            if ( ($screen->base === 'edit'  && $screen->post_type === 'yoda-seo') || ($screen->base === 'post' && get_post_type($post) === 'yoda-seo')) {
                ?>
                <style>a.page-title-action { display: none; } #publishing-action input#publish {display: none;}</style>
                <script type="text/javascript">
                    jQuery(document).ready(function(){
                        jQuery('a.page-title-action').remove();
                        jQuery('#publishing-action input#publish').remove();

                        //let's set the YODA menu as current
                        jQuery('#toplevel_page_yoda-tools-dashboard, #toplevel_page_yoda-tools-dashboard a.toplevel_page_yoda-tools-dashboard').removeClass('wp-not-current-submenu');
                        jQuery('#toplevel_page_yoda-tools-dashboard, #toplevel_page_yoda-tools-dashboard a.toplevel_page_yoda-tools-dashboard').addClass('wp-has-current-submenu');
                        jQuery('#toplevel_page_yoda-tools-dashboard .wp-submenu li').each(function(){
                            let get_a = jQuery(this).find('a').text();
                            if (get_a === 'SEO Suggestions') {
                                jQuery(this).addClass('current');
                                jQuery(this).find('a').addClass('current');
                            }
                        });
                    });
                </script>
                <?php
            }


        }

        /**
         *
         *
         * render_list_status
         * @since 1.4.0
         *
         *
         */
        public function render_list_status($states)
        {
            global $post;
            if (!isset($post))
                return $states;

            if(isset($this->statuses[$post->post_status])){
                return [$this->statuses[$post->post_status]];
            }
            return $states;
        }

        /**
         *
         * register event scheduler
         * @since 1.4.0
         *
         */
        public function create_seo_validator_schedule()
        {
            //let's run the validator every day midnight
            if (!wp_next_scheduled('yoda_seo_smart_validator')) {
                wp_schedule_event(strtotime('midnight'), 'daily', 'yoda_seo_smart_validator');
            }

            //do not create the cron job is auto_process is off
            if ($this->auto_process === 'Y') {
                //let's run the processor every 30 minutes
                if (!wp_next_scheduled('yoda_seo_smart_processor')) {
                    $next_event_time = time() + ( 5 * 60 ); // 6 minutes from now
                    wp_schedule_event($next_event_time, 'yoda_interval', 'yoda_seo_smart_processor');
                }
            } else {
                //clear process if the auto_process is off.
                wp_clear_scheduled_hook('yoda_seo_smart_processor');
            }


        }

        /**
         *
         *
         * yoda_processor_schedule
         * @since 1.4.0
         *
         *
         */
        public function event_processor_schedule()
        {
            $schedules['yoda_interval'] = array(
                'interval' => 300,
                'display' => __( 'YODA SEO interval' )
            );
            return $schedules;
        }


        /**
         *
         * get_to_log
         * @since 1.4.0
         * @return array
         *
         */
        public function get_to_log()
        {
            return $this->to_log;
        }

        /**
         *
         *
         * do_smart_validation
         * @since 1.4.0
         *
         *
         */
        public function do_validation()
        {

            //get all post at once to avoid redundant request
            $this->get_all_posts();

            //scan for metas from supported plugins
            //do the 3rd party scanning
            $this->scan_all_plugin_metas();

            //scan for no images in posts/pages
            //let's not scan the images for now, let's wait for matt's API
            //$this->scan_all_images();

            //scan all menus for titles
            //$this->scan_all_menus();
            $this->scan_wp_menus();

            //scan for no supported plugin installed
            //let's scan the custom if no supported plugins are installed
            if (!$this->has_supported) {
                $this->scan_all_custom_metas();
            }

            //scan for title and description duplications
            $this->scan_duplications();

            //save scanned data
            $this->save_scanned_data();

            //scan category meta and description
            $this->scan_meta_category_tag();

        }


        /**
         *
         *
         * get_all_posts
         * get all post and pages data
         * @since 1.4.0
         * @param $_in - array
         *
         * @return mixed void | object
         *
         *
         */
        private function get_all_posts()
        {

            $args = [
                'posts_per_page' => -1,
                'post_type' => ['post','page'],
                'post_status' => 'publish'
            ];

            $data = get_posts($args);

            if (!empty($data)) {
                if (!empty($_in)) {
                    return $data;
                } else {
                    foreach ($data as $row) {
                        $this->all_posts[$row->ID] = $row->post_content;
                    }
                }
            }



        }

        /**
         *
         *
         * check_supported_plugins
         * let's check if supported plugins are installed.
         * @since 1.4.0
         * @param $plugin string
         *
         *
         */
        public function check_supported_plugins($plugin = '')
        {
            //fix for other setup that doesn't have is_plugin_active function
            if ( ! function_exists( 'get_plugins' ) ) {
                require_once ABSPATH . 'wp-admin/includes/plugin.php';
            }

            $installed_metas = [];
            if (!empty($plugin)) {
                if (is_plugin_active($plugin)) {
                        $installed_metas[] = $this->check_supported_plugins($plugin);
                    }
            } else {
                foreach ($this->seo_plugin_metas as $id => $meta) {
                    if (is_plugin_active($id)) {
                        $installed_metas[] = $meta;
                    }
                }
            }

            return $installed_metas;
        }


        /**
         *
         *
         * get_all_plugin_metas
         * check the metas from supported plugins
         * @since 1.4.0
         * @param
         * @return void
         *
         *
         */

        public function scan_all_plugin_metas() : void
        {
            $installed_metas = $this->check_supported_plugins();

            if (!empty($installed_metas)) {
                $ids = $this->all_posts;
                $this->has_supported = true;
                foreach ($ids as $id => $content) {
                    foreach ($installed_metas as $meta) {

                        $title = $this->get_seo_rendered_meta($id, $meta['title']);
                        $description = $this->get_seo_rendered_meta($id, $meta['description']);

                        if (!$title) {
                            $this->to_log[] = ['obj' => 'meta', 'attr' => 'title', 'post_id' => $id, 'src' => $meta['title']];
                        } else {
                            if (strlen($title) < 20 || strlen($title) > 70) {
                                $this->to_log[] = ['obj' => 'rewrite', 'attr' => 'title', 'post_id' => $id, 'src' => $meta['title'], 'rewrite_type' => 'length'];
                            }

                            //store and check for duplication later
                            $this->post_with_title_meta[$id] = ['title' => $title, 'src' => $meta['title']];
                        }
                        if (!$description) {
                            $this->to_log[] = ['obj' => 'meta', 'attr' => 'description', 'post_id' => $id, 'src' => $meta['description']];
                        } else {
                            if (strlen($description) < 55 || strlen($description) > 350) {
                                $this->to_log[] = ['obj' => 'rewrite', 'attr' => 'description', 'post_id' => $id, 'src' => $meta['description'], 'rewrite_type' => 'length'];
                            }

                            //store and check for duplication later
                            $this->post_with_description_meta[$id] = ['description' => $description, 'src' => $meta['description']];
                        }
                    }
                }
            }

        }


        /**
         *
         *
         * scan_all_custom_metas
         * this will scan the posts content if there's meta fields
         * @since 1.4.0
         * @param
         * @return void
         *
         *
         */
        public function scan_all_custom_metas() : void
        {

            libxml_use_internal_errors(true);
            ob_start();
            get_header();
            $header_html = ob_get_contents();
            ob_get_clean();

            $dom = new \DOMDocument();
            $dom->loadHTML($header_html);
            //metas
            $metaTags = $dom->getElementsByTagName('meta');
            if ($metaTags->length > 0) {

                $have_meta_desc_and_have_text = false;
                $have_meta_title_and_have_text = false;
                foreach ($metaTags as $meta) {
                    if ($meta->getAttribute('name') === 'description' && $meta->nodeValue) {
                        $have_meta_desc_and_have_text = true;
                        break;
                    }

                    if ($meta->getAttribute('name') === 'title' && $meta->nodeValue) {
                        $have_meta_title_and_have_text = true;
                        break;
                    }
                }

                if (!$have_meta_desc_and_have_text || !$have_meta_title_and_have_text) {
                    //description exists do nothing
                    foreach ($this->all_posts as $id => $post) {

                        $has_yoda_title = get_post_meta($id, $this->seo_custom_title_meta_key, true);
                        $has_yoda_desc = get_post_meta($id, $this->seo_custom_meta_key, true);

                        if (!$have_meta_title_and_have_text && empty($has_yoda_title)) {
                            $this->to_log[] = ['obj' => 'meta', 'attr' => 'title', 'post_id' => $id, 'scope' => 'global'];
                        }

                        if (!$have_meta_desc_and_have_text && empty($has_yoda_desc)) {
                            $this->to_log[] = ['obj' => 'meta', 'attr' => 'description', 'post_id' => $id, 'scope' => 'global'];
                        }

                    }
                }

            }
            libxml_clear_errors();

        }

        /**
         *
         *
         * scan_all_images
         * scan all image post/pages images
         * @since 1.4.0
         * @param
         * @return void
         *
         *
         */
        public function scan_all_images() : void
        {
	        // Fetch all image attachments
	        $args = [
		        'post_type'   => 'attachment',
		        'post_mime_type' => 'image',
		        'post_status' => 'inherit',
		        'posts_per_page' => -1,
	        ];
	        $images = get_posts($args);

	        foreach ($images as $attachment) {
		        $alt = get_post_meta($attachment->ID, '_wp_attachment_image_alt', true);
		        $src = wp_get_attachment_url($attachment->ID);

		        if (empty($alt)) {
			        $this->to_log[] = [
				        'obj'     => 'image',
				        'attr'    => 'alt',
				        'src'     => $src,
				        'post_id' => $attachment->ID
			        ];
		        }
	        }
        }

        /**
         *
         *
         * scan_all_menus
         * scan only menus that are assigned to locations
         * @since 1.4.0
         * @param
         * @return void
         *
         * @deprecated
         * In give way to new method called scan_wp_menus
         * @since 1.6.7
         *
         */

        // this method is subject to refactor. Instead of scanning the menu using DOMDocument why not do WP_query by nav_menu_item post type
        public function scan_all_menus()
        {

            $active_ids = get_nav_menu_locations();
            foreach ( $active_ids as $menu_id ) {
                $menu_html = wp_nav_menu( array( 'menu' => $menu_id, 'echo' => false ) );
                if ($menu_html) {
                    $dom = new \DOMDocument();
                    $dom->loadHTML($menu_html);
                    $anchors = $dom->getElementsByTagName('a');
                    $nav_li = $dom->getElementsByTagName('li');

                    if ($anchors->length > 0) {
                        foreach ($anchors as $index => $anchor) {
                            $title = $anchor->getAttribute('title');
                            if (empty($title) && !empty($anchor->nodeValue)) {
                                $nav_id = $nav_li[$index]->getAttribute('id');
                                $nav_id = explode('-',$nav_id);
                                $this->to_log[] = ['obj' => 'menu', 'attr' => 'title', 'src' => $menu_id, 'nav_text' => $anchor->nodeValue, 'post_id' => end($nav_id)];
                            }
                        }

                    }

                }
            }

        }


        /**
         *
         *
         * scan_wp_menus
         * scan the entire menu for title
         * @since 1.6.7
         *
         *
         */
        public function scan_wp_menus()
        {
            $active_locations = get_nav_menu_locations();
            if ($active_locations) {
                foreach ($active_locations as $location) {
                    $menu_items = wp_get_nav_menu_items($location);
                    if (!empty($menu_items)) {
                        foreach ($menu_items as $item) {
                            $this->to_log[] = ['obj' => 'menu', 'attr' => 'title', 'src' => $location, 'nav_text' => $item->title, 'post_id' => $item->ID];
                        }
                    }

                }
            }

        }


        /**
         *
         *
         * save_scanned_data
         * save the scanned data when found missing
         * @since 1.4.0
         * @param
         * @return void
         *
         */
        public function save_scanned_data()
        {

            if (!empty($this->to_log)) {

                $to_save = [];

                $get_logs = $this->check_duplicated();

                foreach ($this->to_log as $log) {
                    $data = [];
                    if (isset($log['post_id'])) {
                        //check if duplicated
                        if (!in_array($log['post_id'], $get_logs)) {
                            $data['post_id'] = sanitize_text_field($log['post_id']);
                        }
                    } else {
                        //let's not save if no post id
                        continue;
                    }

                    if (isset($log['obj'])) {
                        $data['obj'] = sanitize_text_field($log['obj']);
                    }
                    if (isset($log['attr'])) {
                        $data['attr'] = sanitize_text_field($log['attr']);
                    }
                    if (isset($log['src'])) {
                        $data['src'] = sanitize_text_field($log['src']);
                    }
                    if (isset($log['nav_text'])) {
                        $data['nav_text'] = sanitize_text_field($log['nav_text']);
                    }
                    if (isset($log['scope'])) {
                        $data['scope'] = sanitize_text_field($log['scope']);
                    }
                    if (isset($log['size'])) {
                        $data['size'] = sanitize_text_field($log['size']);
                    }
                    if (isset($log['rewrite_type'])) {
                        $data['rewrite_type'] = sanitize_text_field($log['rewrite_type']);
                    }

                    $to_save[] = $data;
                }



                if (!empty($to_save)) {

                    foreach ($to_save as $log_row) {

                        if (!isset($log_row['post_id'])) {
                            continue;
                        }

                        $post_type = get_post_type($log_row['post_id']);
                        switch ($post_type) {
                            case 'page':
                            case 'post':
                                $log_title = get_the_title($log_row['post_id']);
                                break;
                            case 'nav_menu_item':
                                $log_title = $log_row['nav_text'];
                                break;
                            case 'attachment':
                                $log_title = 'Image';
                                break;
                        }


                        $log_status = 'pending';
                        //let's check if the parent post has the context required to call the API
                        $feed = $this->pre_ai_process($log_row['post_id'], get_the_title($log_row['post_id']), $log_row['obj']);
                        if (empty($feed)) {
                            //no context found
                            $log_status = 'requires-attention';
                        }

                        // Set up the post arguments
                        $post_args = array(
                            'post_type' => 'yoda-seo',
                            'post_title' => $log_title,
                            'post_status' => $log_status
                        );
                        //save log
                        $log_id = wp_insert_post( $post_args );

                        //yoda_seo_process_feed
                        $feed = $this->pre_ai_process($log_row['post_id'], get_the_title($log_row['post_id']), $log_row['obj']);
                        if (!empty($feed)) {
                            update_post_meta($log_id, 'yoda_seo_process_feed', $feed);
                        }

                        if ($log_id > 0) {
                            //save log details
                            $details = [
                                'post_id'       => (isset($log_row['post_id']))? $log_row['post_id'] : null,
                                'object'        => (isset($log_row['obj']))? $log_row['obj'] : null,
                                'attributes'    => (isset($log_row['attr']))? $log_row['attr'] : null,
                                'source'        => (isset($log_row['src']))? $log_row['src'] : null,
                                'nav_text'      => (isset($log_row['nav_text']))? $log_row['nav_text'] : null,
                                'scope'         => (isset($log_row['scope']))? $log_row['scope'] : null,
                                'size'          => (isset($log_row['size']))? $log_row['size'] : null,
                                'rewrite_type'  => (isset($log_row['rewrite_type']))? $log_row['rewrite_type'] : null
                            ];
                            update_post_meta($log_id, $this->seo_meta_key, $details);

                        }


                    }


                }


            }
        }


        /**
         *
         *
         * do_processing
         * let's process the pending log
         * @since 1.4.0
         * @param
         * @return
         *
         *
         */
        public function do_processing( $post_id = 0 )
        {

            //before we process let's check if the credit is still above from limit
            if ($this->limit_reached()) {
                //bail out
                error_log('error cron 1: credit limit reached');
                return json_encode(['status' => 'error', 'message' => __('You have insufficient credits', 'yoda-tools')]);
            }


            //get supported metas
            $installed_metas = $this->check_supported_plugins();


            //process log outside event schedule
            //let's process multiple pendings per run
            $log_details_array = [];

            if ($post_id > 0) {

                $this->update_status($post_id, 'processing');
                $log_details_array[$post_id] = get_post_meta($post_id, $this->seo_meta_key, true);


            } else {
                //get 30 pending log and process
                $args = [
                    'posts_per_page' => $this->max_schuduled_number,
                    'post_type' => 'yoda-seo',
                    'post_status' => ['pending','processing'],
                    'orderby' => 'date',
                    'order' => 'ASC'
                ];
                $to_process = get_posts($args);

                if (!empty($to_process)) {
                    foreach ($to_process as $process) {
                        //update the status
                        $log_details_array[$process->ID] = get_post_meta($process->ID, $this->seo_meta_key, true);
                        $this->update_status($process->ID, 'processing');
                    }
                }

            }


            //let's extract the IDs so that we are querying only the posts that has been logged
            if (!empty($log_details_array)) {


                $time_start = microtime(true);

                $processed_data = []; //let's hold the processed data here

                foreach ($log_details_array as $logid => $log_details) {

                    $parent_post = get_post($log_details['post_id']);

                    //check for no index pages
                    //@since 1.5.7
                    $is_no_index = false;
                    if (!empty($installed_metas)) {
                        foreach ($installed_metas as $in_meta) {
                            //let's check if the parent page is no-index for yoast
                            if (isset($in_meta['no_index']) && $parent_post) {
                                $is_no_index = get_post_meta($parent_post->ID, $in_meta['no_index'], true);
                                if ($is_no_index) {
                                    $is_no_index = true;
                                    break;
                                }
                            }
                        }
                    }

                    if ($is_no_index) {
                        $this->update_status($logid, 'failed');
                        //return if single
                        if ($post_id > 0) {
                            return json_encode(['status' => 'failed', 'message' => __("YODA can't process if the post is no-index", 'yoda-tools')]);
                        }
                        continue;
                    }


                    //let's check if the process takes too long and terminate
                    if ((microtime(true) - $time_start) > $this->max_run_Time) {
                        error_log("Cron runtime error: Runtime exceeds at {$this->max_run_Time} request is terminated");
                        break;
                    }

                    if (empty($parent_post)) {
                        error_log('error cron 6: No parent post found');
                        update_post_meta( $logid, 'yoda_seo_scan_error', ['status' => 'error', 'error' => 'No parent post found']);
                        continue;
                    }

                    $ai_content = null;

                    switch ($log_details['object']) {
                        case 'duplicate':
                        case 'rewrite':
                                $feed = $this->get_seo_rendered_meta($parent_post->ID, $log_details['source']);
                                if (isset($log_details['rewrite_type'])) {
                                    if ($log_details['rewrite_type'] === 'length') {
                                        //get the feed from content
                                        $post_content_feed = get_post_field('post_content', $parent_post->ID);
                                        //strip all html tags
                                        $feed_body = wp_strip_all_tags($post_content_feed);
                                        if (!empty($feed_body)) {
                                            $feed = $feed_body;
                                        }
                                    }
                                }

                                $to_rewrite = ($log_details['attributes'] === 'title')? 'Title' : 'Description';
                                $ai_content = $this->request_ai_content(
                                    [
                                            'type'      => 'Page: '.$to_rewrite,
                                            'value'     => $feed,
                                            'previous'  => $feed
                                    ]
                                );

                                if (!$ai_content) {
                                    $this->update_status($logid, 'failed');

                                    //this is questionable investigate this code
                                    if ($post_id > 0) {
                                        return json_encode(['status' => 'failed', 'message' => __('No relevant response from the server', 'yoda-tools')]);
                                    }

                                } else {
                                    update_post_meta($parent_post->ID, $log_details['source'], $ai_content);

                                    //save the previous text
                                    $this->save_previous_text(
                                        [
                                            'post_id'   => $parent_post->ID,
                                            'record'      => [
                                                'previous'  => $feed,
                                                'new'       => $ai_content
                                            ]
                                        ]
                                    );

                                }

                                //update the log data status
                                $this->update_status($logid, 'completed');

                                //return json if individual
                                if ($post_id > 0) {
                                    return json_encode(['status' => 'success', 'message' => __('Completed', 'yoda-tools')]);
                                }

                            break;
                        case 'meta' :
                                //do the meta processing
                                //for meta and description let's request the API only once

                                 //now let's check if the post already been processed by checking the cached data and take the ai content from there instead.

                                $log_status_failed = 'failed';

                                if (isset($processed_data[$parent_post->ID])) {
                                    $ai_content = $processed_data[$parent_post->ID];
                                    //let's log and see if the record is taken from cached data
                                    error_log("Record is taken from cached - post ID: $parent_post->ID");
                                } else {

                                    $feed = get_post_meta($logid, 'yoda_seo_process_feed', true);
                                    if (!$feed) {
                                        //fallback action for the previous log prior to this version
                                        //@since 1.6.10
                                        $feed = $this->pre_ai_process($parent_post->ID, $parent_post->post_title, 'meta');
                                        if (!$feed) {
                                            $log_status_failed = 'requires-attention';
                                        }
                                    }

                                    if ($feed) {
                                        $ai_content = $this->request_ai_content(
                                            [
                                                'type'        => 'Page: Title & Description',
                                                'value'       => $feed, //seems we only need the page title and let the AI will provide the description
                                                'previous'    => $parent_post->post_title
                                            ]
                                        );
                                    }
                                }

                                if (!$ai_content) {
                                    //save the data as failed
                                    $this->update_status($logid, $log_status_failed);

                                    if ($post_id > 0) {
                                        return json_encode(['status' => 'failed', 'message' => __('No relevant response from the server', 'yoda-tools')]);
                                    }

                                } else {
                                    $processed_data[$parent_post->ID] = $ai_content;

                                    //save previous text
                                    $this->save_previous_text(
                                        [
                                            'post_id'   => $parent_post->ID,
                                            'record'      => [
                                                'previous'  => $parent_post->post_title,
                                                'new'       => $ai_content
                                            ]
                                        ]
                                    );
                                }

                                //check for title
                                if ($log_details['attributes'] === 'title') {
                                    if (empty($installed_metas)) {
                                        //I think we don't have title for global
                                        update_post_meta($parent_post->ID, $this->seo_custom_title_meta_key, $ai_content[0]);
                                    } else {
                                        if (isset($log_details['source'])) {
                                            update_post_meta($parent_post->ID, $log_details['source'], $ai_content[0]);
                                        } else {
                                            foreach ($installed_metas as $meta) {
                                                update_post_meta($parent_post->ID, $meta['title'], $ai_content[0]);
                                            }
                                        }

                                    }
                                }

                                //check for description
                                if ($log_details['attributes'] === 'description') {
                                    if (empty($installed_metas)) {
                                        //save content for custom description
                                        update_post_meta($parent_post->ID, $this->seo_custom_meta_key, $ai_content[1]);
                                    } else {
                                        if (isset($log_details['source'])) {
                                            update_post_meta($parent_post->ID, $log_details['source'], $ai_content[1]);
                                        } else {
                                            foreach ($installed_metas as $meta) {
                                                update_post_meta($parent_post->ID, $meta['description'], $ai_content[1]);
                                            }
                                        }

                                    }

                                }

                                //update the log data status
                                $this->update_status($logid, 'completed');

                                //return json if individual
                                if ($post_id > 0) {
                                    return json_encode(['status' => 'success', 'message' => __('Completed', 'yoda-tools')]);
                                }

                            break;
                        case 'menu' :
                                $fallback_status_failed = "failed";

                                //preprocess procedure for accurate result
                                //we have to take the target page
                                $feed = get_post_meta($logid, 'yoda_seo_process_feed', true);
                                if (!$feed) {
                                    //fallback action for the previous log prior to this version
                                    //@since 1.6.10
                                    $feed = $this->pre_ai_process($parent_post->ID, $parent_post->post_title, 'menu');
                                    if (!$feed) {
                                        $fallback_status_failed = "requires-attention";
                                    }
                                }

                                if ($feed) {
                                    $ai_content = $this->request_ai_content(
                                        [
                                            'type'      => 'Menu: Title Text',
                                            'value'     =>  $feed,
                                            'previous'  => $log_details['nav_text']
                                        ]
                                    );
                                }


                                if (!empty($ai_content)) {
                                    $post_data = array(
                                        'ID'           => $parent_post->ID,
                                        'post_excerpt' => $ai_content,
                                    );
                                    wp_update_post( $post_data );

                                    //save previous text
                                    $this->save_previous_text(
                                        [
                                            'post_id'   => $parent_post->ID,
                                            'record'      => [
                                                'previous'  => $log_details['nav_text'],
                                                'new'       => $ai_content
                                            ]
                                        ]
                                    );

                                    //update the log data status
                                    $this->update_status($logid, 'completed');

                                    //return json if individual
                                    if ($post_id > 0) {
                                        return json_encode(['status' => 'success', 'message' => __('Completed', 'yoda-tools')]);
                                    }

                                } else {
                                    $this->update_status($logid, $fallback_status_failed);
                                    return json_encode(['status' => 'failed', 'message' => __('No relevant response from the server', 'yoda-tools')]);
                                }

                                return;

                            break;
                        case 'image':
                            //let's process the image alt text
	                        $attachments = get_attached_media('image', $parent_post->ID);

	                        if (!empty($attachments)) {
		                        foreach ($attachments as $attachment) {
			                        $img_url = wp_get_attachment_url($attachment->ID);
			                        $img_alt = get_post_meta($attachment->ID, '_wp_attachment_image_alt', true);
			                        $ai_alt_text = $this->request_ai_content(['type' => 'Image: Alt Text', 'value' => $img_url, 'previous' => $img_alt]);
			                        if ($ai_alt_text) {
				                        update_post_meta($attachment->ID, '_wp_attachment_image_alt', $ai_alt_text);
				                        wp_update_post([
					                        'ID' => $attachment->ID,
					                        'post_excerpt' => $ai_alt_text, // Update the title with the generated alt text if needed
				                        ]);
			                        }
		                        }

		                        $this->update_status($logid, 'completed');

		                        if ($post_id > 0) {
			                        return json_encode(['status' => 'success', 'message' => __('Completed', 'yoda-tools')]);
		                        }
	                        } else {
		                        $this->update_status($logid, 'failed');
		                        return json_encode(['status' => 'failed', 'message' => __('No images found in the post', 'yoda-tools')]);
	                        }

                            break;
                        default:
                            error_log('error cron 4: No object matched - '.$log_details['object']);
                            return json_encode(['status' => 'error', 'message' => __('No object matched', 'yoda-tools')]);
                            break;
                    }
                }
            } else {
                error_log('error cron 5: No meta data found');
                return json_encode(['status' => 'error', 'message' => __('No meta data found', 'yoda-tools')]);
            }

        }


        /**
         *
         *
         * render_custom_meta_tags
         * if the user did not use any 3rd party plugin let's render the title and description here base from current post title and content
         * @since 1.4.0
         *
         *
         */
        public function render_custom_meta_tags()
        {
            global $post;


            if (!isset($post))
                return;

            if ($this->has_supported) {
                return;
            }

            if (is_archive()) {
                $details = $this->archive_details();
                ?>
                <meta name="description" content="<?php echo esc_attr($details['description']); ?>" class="yoda-seo-tools" />
                <?php
                return;
            }

            if (is_single() || is_page()) {
                $post_type = get_post_type($post->ID);
                if (in_array($post_type, $this->supported_post_type)) {
                    $description = get_post_meta($post->ID, $this->seo_custom_meta_key, true);
                    $generated_title = get_post_meta($post->ID, $this->seo_custom_title_meta_key, true);

                    if ($generated_title):
                        ?>
                            <meta name="title" content="<?php echo esc_attr($generated_title); ?>" class="yoda-seo-tools" />
                        <?php
                    endif;
                    if ($description):
                        ?>
                            <meta name="description" content="<?php echo esc_attr($description); ?>" class="yoda-seo-tools" />
                        <?php
                    endif;
                }
            }




        }

        /**
         *
         *
         * render_custom_meta_title
         * render custom meta title by using wp_title hook
         * @since 1.4.0
         *
         *
         */
        public function render_custom_meta_title($data)
        {
            global $post;

            if ($this->has_supported) {
                return $data;
            }

            if (isset($post)) {
                $generated_title = get_post_meta($post->ID, $this->seo_custom_title_meta_key, true);
                if ($generated_title) {
                    $data['title'] =  $generated_title;
                }
            }

            if (is_archive()) {
                $details = $this->archive_details();
                $data['title'] = $details['title'];
            }

            return $data;
        }

        /**
         *
         *
         * sub_word_content
         * let's take the part content of the content by max length we set above
         * @since 1.4.0
         * @param $content - string
         * @return string
         *
         *
         */
        public function sub_word_content($content = '') : string
        {
            if (empty($content))
                return '';

            $words = str_word_count($content, 1);
            return implode(' ', array_slice($words, 0, $this->maximumt_meta_length));
        }

        /**
         *
         *
         * request_ai_content
         * Let's generate title and description or menu title
         * @since 1.4.0
         *
         *
         */
        public function request_ai_content($data)
        {
            if (!$this->seo_api) {
                error_log('No apikey is found');
                return false;
            }

            $this->seo_api->set_api_key($this->seo_api_key);
            $this->seo_api->set_body(
                [
                    'action'    => 'create',
                    'type'      => $data['type'],
                    'value'     => $data['value'],
                    'previous'  => $data['previous']
                ]
            );

            $reponse = $this->seo_api->execute_seo_curl();

            if (empty($reponse)) {
                return false;
            }

            $res = json_decode($reponse);

            if ($res->status === 'success') {
                switch ($data['type']) {
                    case 'Page: Title & Description':
                        if ($res->rows->response) {
                            $res_data = explode("SEO Meta Description:",$res->rows->response);
                            $res_clean_data = [];
                            foreach ($res_data as $rdata) {
                                $extracted_data = str_replace(['SEO Meta Title:', '\n'],['',''], $rdata);
                                //remove double quote
                                $res_clean_data[] = str_replace(["“","“",'"'],['','',''], $extracted_data);
                            }
                            return $res_clean_data;
                        }
                        break;
                    case 'Page: Title':
                    case 'Page: Description':
                    case 'Menu: Title Text':
                        //remove double quote
                        $ai_content_trimmed = str_replace(["“","“",'"'],['','',''], $res->rows->response);
                        return $ai_content_trimmed;
                        break;

                }
            } else {
                error_log(json_encode($res));
            }

            return false;
        }

        /**
         *
         *
         * check_if_has_supported_plugin_single
         * @since 1.4.0
         * @param $id - int
         * @return bool
         *
         *
         */
        public function check_if_has_supported_plugin_single($id = null) : bool
        {
            if (!$id) {
                return false;
            }

            foreach ($this->seo_plugin_metas as $metas) {
                if (get_post_meta($id, $metas['title'], true) || get_post_meta($id, $metas['description'], true)) {
                    return true;
                }
            }
            return false;
        }


        /**
         *
         *
         * limit_reached
         * we will check the credit remaining and if reached do not process what's insise the event schedule
         * @since 1.4.0
         * @param
         * @return bool
         *
         *
         */
        public function limit_reached() : bool
        {
            //get current credits
            $this->seo_api->set_api_key($this->seo_api_key);
            $current_credits = $this->seo_api->get_credits();

            if ($this->credits === 'all' || !$this->credits) {
                if ($current_credits <= 0) {
                    return true;
                } else {
                    return false;
                }
            } else {
                $stop_credits = json_decode($this->credits);
                if (is_array($stop_credits)) {
                    if (isset($stop_credits[1])) {
                        if ($current_credits <= $stop_credits[1]) {
                            return true;
                        }
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            }

        }

        /**
         *
         *
         * check_duplicated
         * let's check duplicated entries we don't want to have redundant data
         * @since 1.4.0
         * @param
         * @return array
         *
         *
         */
        public function check_duplicated() : array
        {
            //$get_our_statuses = array_keys($this->statuses);

            $ids = [];
            $args = [
                'posts_per_page' => -1,
                'post_type' => 'yoda-seo',
                'fields' => 'ids',
                'post_status' => ['pending', 'requires-attention']
            ];
            $all_yoda = get_posts($args);
            if (!empty($all_yoda)) {
                foreach ($all_yoda as $yoda_id) {
                    $post_id = get_post_meta($yoda_id, $this->seo_meta_key, true);
                    if ($post_id) {
                        $ids[] = $post_id['post_id'];
                    }
                }
                return $ids;
            }
            return $ids;
        }

        /**
         *
         *
         * update_status
         * let's update the status of the log acording the what the process does
         * @since 1.4.0
         * @param $post_id - int
         *        $status - string
         * @return void
         *
         *
         */
        public function update_status($post_id, $status) : void
        {
            $to_process = [
                'ID' => $post_id,
                'post_status' => $status,
            ];
            wp_update_post($to_process);
        }

        /**
         *
         *
         * dismiss_log
         * let's dismiss the log via ajax
         * @since 1.4.0
         *
         *
         */
        public function dismiss_log()
        {
            if (!wp_verify_nonce( $_POST['nonce'][0],  'yoda-seo-nonce' )) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __('Invalid nonce', 'yoda-tools')
                ]);
                exit();
            }

            $post_id = (isset($_POST['id']))? sanitize_text_field($_POST['id']) : null;

            if ($post_id) {
                $this->update_status($post_id, 'dismissed');
            }

            echo json_encode(
                [
                    'status' => 'success',
                    'message' => ''
                ]
            );

            exit();
        }


        /**
         *
         *
         * process_log
         * process the log via ajax
         * @since 1.4.0
         *
         *
         */
        public function process_log()
        {
            if (!wp_verify_nonce( $_POST['nonce'][0],  'yoda-seo-nonce' )) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __('Invalid nonce', 'yoda-tools')
                ]);
                exit();
            }

            $post_id = (isset($_POST['id']))? sanitize_text_field($_POST['id']) : null;

            if ($post_id) {
               echo $this->do_processing($post_id);
            }

            exit();
        }

        /**
         *
         *
         * event_schedule_next_call
         * @since 1.5.0
         *
         *
         */
        public function event_schedule_next_call()
        {

            $event = wp_get_scheduled_event( 'yoda_seo_smart_processor' );
            if (!$event) {
                return [];
            }

            $timestamp = $event->timestamp;
            $current_time = time();
            $minutes = abs(($timestamp - $current_time) / 60);
            $minutes = floor($minutes);
            $plural = ($minutes > 1)? 's' : '';
            $minutes = ($minutes > 0)? $minutes : 'less than a';

            $args = [
                'posts_per_page' => $this->max_schuduled_number,
                'post_type' => 'yoda-seo',
                'post_status' => ['pending','processing'],
                'orderby' => 'date',
                'order' => 'ASC'
            ];
            $scheduled = get_posts($args);

            $scheduled_ids = [];

            if (!empty($scheduled)) {
                foreach ($scheduled as $sched) {
                    $scheduled_ids[$sched->ID] = "Automatically process in {$minutes} minute{$plural}";
                }

            }

            return $scheduled_ids;

        }

        /**
         *
         *
         * custom_ui
         * add custom button to SEO suggestion screen to navigate back to SEO settings page
         * @since 1.5.0
         *
         *
         */
        public function custom_ui($which)
        {
            //global $post_type;
            $post_type = (isset($_GET['post_type']))? $_GET['post_type'] : null;
            if ($which === 'top' && $post_type === 'yoda-seo') {
                $url = admin_url('admin.php?page=yoda-tools-seo-tools');
                ?>
                    <a href="<?php echo esc_url($url); ?>" class="button button-primary"><?php esc_html_e('Go to SEO tools', 'yoda-seo'); ?></a>
                <?php
            }
        }


        /**
         *
         *
         * get_parent_menu
         * get parent menu ID to determine the edit URL of menu object
         * @since 1.5.2
         *
         *
         */
        function get_parent_menu($submenu_id) {
            $locations = get_nav_menu_locations();

            foreach ($locations as $location => $menu_id) {
                $menu_items = wp_get_nav_menu_items($menu_id);

                foreach ($menu_items as $menu_item) {
                    if ($submenu_id == $menu_item->object_id) {
                        return $menu_id;
                    }
                }
            }

            return null;
        }

        /**
         *
         *
         * scan_meta_category
         * scan the category and tag meta fields
         * @since 1.5.2
         *
         *
         */
        public function scan_meta_category_tag()
        {
            $installed_plugins = $this->check_supported_plugins();

            if (!empty($installed_plugins)) {

                foreach ($installed_plugins as $data) {

                    $titles = get_option('wpseo_titles');

                    if (!empty($titles)) { //means the supported plugin is WPSEO
                        $do_update = false;

                        if (!isset($data['tax_titles']['categories']['title']) || !isset($data['tax_titles']['categories']['description'])) {
                            continue;
                        }

                        if (empty($titles[$data['tax_titles']['categories']['title']])) {
                            $titles[$data['tax_titles']['categories']['title']] = $data['tax_titles']['categories']['yoda_tax_title_fix'];
                            $do_update = true;
                        }

                        if (empty($titles[$data['tax_titles']['categories']['description']])) {
                            $titles[$data['tax_titles']['categories']['description']] = $data['tax_titles']['categories']['yoda_tax_description_fix'];
                            $do_update = true;
                        }

                        if (empty($titles[$data['tax_titles']['tags']['title']])) {
                            $titles[$data['tax_titles']['tags']['title']] = $data['tax_titles']['tags']['yoda_tag_title_fix'];
                            $do_update = true;
                        }

                        if (empty($titles[$data['tax_titles']['tags']['description']])) {
                            $titles[$data['tax_titles']['tags']['description']] = $data['tax_titles']['tags']['yoda_tag_description_fix'];
                            $do_update = true;
                        }

                        if ($do_update) {
                            update_option('wpseo_titles', $titles);
                        }
                    }

                    //follow the other plugins here


                }

            }


        }


        /**
         *
         *
         *
         * archive_details
         * @since 1.5.2
         * @param
         * @return array
         *
         *
         */
        public function archive_details() : array
        {

            $archive = get_queried_object();
            if (is_tax()) {
                $archive_name = $archive->name;
            } elseif (is_post_type_archive()) {
                $archive_name = $archive->label;
            } else {
                $archive_name = get_the_archive_title();
            }

            $archive_name = sanitize_text_field($archive_name);
            $details = explode(':', $archive_name);
            $title = trim($details[1]);
            $sitename = get_bloginfo('name');
            return [
                'title' => "$title Archives | $sitename",
                'description' => "Are you looking for $title from $sitename? Then this is the link that you need to click!"
            ];

        }

        /**
         *
         * scan_duplications
         * check the duplicated title and descriptions. Rewrite if duplicated
         * @since 1.5.3
         *
         */
        public function scan_duplications()
        {
            $titles = $this->post_with_title_meta;
            $descriptions = $this->post_with_description_meta;

            $temp_titles = $titles;
            $temp_descriptions = $descriptions;

            //for title
            foreach ($titles as $id => $title) {
                foreach ($temp_titles as $temp_id => $temp_ti) {
                    if ($temp_ti['title'] === $title['title'] && $temp_id != $id) {
                        $this->to_log[] = ['obj' => 'duplicate', 'attr' => 'title', 'post_id' => $temp_id, 'src' => $temp_ti['src']];
                    }
                }
            }

            //for description
            foreach ($descriptions as $id => $desc) {
                foreach ($temp_descriptions as $temp_id => $temp_desc) {
                    if ($temp_desc['description'] === $desc['description'] && $temp_id != $id) {
                        $this->to_log[] = ['obj' => 'duplicate', 'attr' => 'description', 'post_id' => $temp_id, 'src' => $temp_desc['src']];
                    }
                }
            }


        }

        /**
         *
         *
         * get_seo_rendered_meta
         * render supported plugin SEO shortcodes i.e. %%title%% to actual title
         * @since 1.5.4
         *
         *
         */
        public function get_seo_rendered_meta($post_id = 0, $key = '')
        {
            if ($post_id == 0 || empty($key)) {
                return false;
            }

            switch ($key) {
                case '_yoast_wpseo_title':
                case '_yoast_wpseo_metadesc':
                    $content = get_post_meta($post_id, $key, true);
                    return wpseo_replace_vars($content, get_post($post_id));
                    break;
                case '_aioseo_title':
                case '_aioseo_description':
                    $content = get_post_meta($post_id, $key, true);
                    return aioseo()->tags->replaceTags( $content, $post_id );
                    break;
            }

            return false;
        }

        /**
         *
         *
         * aio_render_ai_title
         * @since 1.5.4
         *
         *
         */
        public function aio_render_ai_title($title)
        {
            global $post;
            if ( is_singular() ) {
                $meta = $this->seo_plugin_metas;
                return get_post_meta($post->ID, $meta[$this->aioseo_id]['title'], true);
            }
            return $title;
        }

        /**
         *
         *
         * aioseo_render_ai_description
         * @since 1.5.4
         *
         *
         */
        public function aioseo_render_ai_description($description)
        {
            global $post;
            if ( is_singular() ) {
                $meta = $this->seo_plugin_metas;
                return get_post_meta($post->ID, $meta[$this->aioseo_id]['description'], true);
            }
            return $description;
        }


        /**
         *
         *
         * create_site_details_cron
         * @since 1.5.6
         *
         *
         */
        public function create_site_details_cron()
        {
            //let's run the validator every day midnight
            if (!wp_next_scheduled('yoda_site_details_check')) {
                wp_schedule_event(strtotime('midnight'), 'daily', 'yoda_site_details_check');
            }
        }


        /**
         *
         *
         * send_site_details
         * @since 1.5.6
         *
         *
         */
        public function send_site_details()
        {
            $settings = new settings();
            $details = $settings->wp_overview();
            $details = json_encode($details);

            $this->seo_api->set_api_key($this->seo_api_key);

            $this->seo_api->set_body(
                [
                    'details' => $details,
                    'action'  => 'create'
                ]
            );

            $reponse = $this->seo_api->execute_seo_curl('site_details');

        }

        /**
         *
         * save_previous_text
         * save in DB the previous text before sending to API
         * @since 1.5.7
         * @param $data - array
         *
         *
         */
        public function save_previous_text($data = []) {

            if (!empty($data)) {

                if (isset($data['post_id'])) {
                    //save the data in meta
                    $saved_entry = get_post_meta($data['post_id'], 'yoda_advanced_seo_text_track', true);
                    if (!empty($saved_entry)) {
                        $saved_entry[] = json_encode($data['record']);
                        update_post_meta($data['post_id'], 'yoda_advanced_seo_text_track', $saved_entry);
                    } else {
                        update_post_meta($data['post_id'], 'yoda_advanced_seo_text_track', [json_encode($data['record'])]);
                    }
                }

            }

        }

        /**
         *
         *
         * process_bulk
         * process bulk action for YODA seo
         * @since 1.6.7
         *
         */
        public function process_bulk($redirect_to, $action, $post_ids)
        {
            switch ($action) {
                case 'yoda-bulk-process':

                    if (!empty($post_ids)) {
                        foreach ($post_ids as $id) {
                            $this->do_processing($id);
                        }
                    }

                    break;
                case 'yoda-bulk-dismiss':
                    if (!empty($post_ids)) {
                        foreach ($post_ids as $id) {
                            $this->update_status($id, 'dismissed');
                        }
                    }
                    break;
            }

            return $redirect_to;
        }


        /**
         *
         * process_yoda_bulk
         * process the bulk action from YODA admin page
         * @since 1.6.7
         *
         */
        public function process_yoda_bulk()
        {


            $page      = (isset($_GET['page']))? sanitize_text_field($_GET['page']) : null;
            $action    = (isset($_GET['action']))? sanitize_text_field($_GET['action']) : null;
            $logs      = (isset($_GET['yoda_bulk_action']))? $_GET['yoda_bulk_action'] : null;

            if ( $page === 'yoda-tools-seo-tools' && $action && $logs) {

                if (!$_GET['_wpnonce']) {
                    return;
                }

                if (!wp_verify_nonce( $_GET['_wpnonce'],  'yoda-seo-suggestions' )) {
                    error_log('SEO Suggestion bulk action: Invalid nonce');
                    return;
                }

                //clean the array
                $logs = array_map('sanitize_text_field', $logs);

                switch ($action) {
                    case 'yoda-bulk-process':
                        foreach ($logs as $id) {
                            $this->do_processing($id);
                        }
                        break;
                    case 'yoda-bulk-dismiss':
                        foreach ($logs as $id) {
                            $this->update_status($id,'dismissed');
                        }
                        break;
                    case 'trash':
                        //remove here come back later
                        foreach ($logs as $id) {
                            wp_delete_post($id, true);
                        }
                        break;
                }

                $post_status = (isset($_GET['post_status']))? '&post_status='.sanitize_text_field($_GET['post_status']) : null;

                $redirect = get_admin_url().'admin.php?page='.$page.$post_status;
                wp_redirect($redirect);

            }



        }

        /**
         *
         *
         * pre_ai_process
         * preprocess procedure to provide accurate result from API through checking if there's a summary in YODA settings or take the content from target page body
         * @since 1.6.7
         * @param   int - parent id
         *          string - original feed
         *
         */
        public function pre_ai_process($parent_id, $feed = '', $type = 'menu')
        {
            $feed = null;
            $util = new utilities();

            //step 1: check if there's summary from YODA settings
            if ($type === 'menu') {
                //let's take the feed from target post
                $get_obj_id = get_post_meta($parent_id,'_menu_item_object_id', true);
                if ($get_obj_id) {
                    $feed = $util->check_post_context($get_obj_id, $feed);
                }
            } else {
                $feed = $util->check_post_context($parent_id, $feed);
            }

            return $feed;
        }


        /**
         *
         *
         * required_attention
         * display WP dashboard widget
         * @since 1.6.10
         *
         *
         */
        public function required_attention()
        {
            wp_add_dashboard_widget(
                'yoda_tools_seo_dashboard_widget',
                'YODA SEO tools',
                [$this, 'load_seo_widget']
            );
        }

        /**
         *
         *
         * load_seo_widget
         * load the dahsboard content
         * @since 1.6.10
         *
         *
         */
        public function load_seo_widget()
        {
            global $wpdb;
            $status = [];
            $status_fig = [];
            foreach ($this->statuses as $slug => $name) {
                $status[$slug]      = "'".$slug."'";
                $status_fig[$slug]  = 0;
            }
            $status_clause = implode(",",$status);
            $data = $wpdb->get_results("SELECT post_status FROM {$wpdb->prefix}posts WHERE post_type = 'yoda-seo' AND post_status IN ({$status_clause})");

            if (!empty($data)) {
                foreach ($data as $row) {
                    $status_fig[$row->post_status] = $status_fig[$row->post_status] +=1;
                }
            }

            include YODA_TOOLS_PATH.'views/dashboard-widget.php';
        }


        /**
        *
        *
        * save_overview
        * save overview of the required attention log
        * @since 1.6.10
        *
        *
        */
        public function save_overview()
        {
            if ( ! wp_verify_nonce( $_POST['nonce'][0], 'yoda-seo-nonce' ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }

            $id             = (isset($_POST['id']))? sanitize_text_field($_POST['id']) : null;
            $overview       = (isset($_POST['overview']))? sanitize_text_field($_POST['overview']) : null;
            $generate       = (isset($_POST['generate']))? sanitize_text_field($_POST['generate']) : null;

            if ($overview && $id) {

                $log_meta = get_post_meta($id, 'yoda-seo-log-details', true);

                $post_id = $log_meta['post_id'];

                if ($log_meta['object'] === 'menu') {
                    $feed_title = $log_meta['nav_text'];
                } elseif ($log_meta['object'] === 'meta') {
                    $feed_title = get_the_title($post_id);
                }

                //build the feed for title & description type
                $feed = $feed_title.' for the content about '.$overview;
                update_post_meta($id, 'yoda_seo_process_feed', $feed);

                $to_process = [
                    'ID' => $id,
                    'post_status' => 'pending',
                ];
                wp_update_post($to_process);

                if ($generate === 'yes') {

                    //get other required fields
                    //do process
                    if ($id) {

                        $this->do_processing($id);
                        set_transient('yoda_seo_transaction_success', __('SEO is successfully generated', 'yoda-tools'));
                        echo json_encode([
                            'status'    => __('success', 'yoda-tools'),
                            'message'   => ''
                        ]);
                    }

                } else {
                    set_transient('yoda_seo_transaction_success', __('Overview is successfully saved', 'yoda-tools'));

                    echo json_encode([
                        'status'    => __('success', 'yoda-tools'),
                        'message'   => ''
                    ]);
                }
            } else {
                echo json_encode([
                    'status'    => __('failed', 'yoda-tools'),
                    'message'   => __('No overview or SEO suggestion found', 'yoda-tools')
                ]);
            }
            exit();

        }

        /**
         *
         *
         *
         * get_suggestion
         * get suggestion from API
         * @since 1.6.10
         *
         *
         */
        public function get_suggestion()
        {
            if ( ! wp_verify_nonce( $_POST['nonce'][0], 'yoda-seo-nonce' ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }

            $post_id = (isset($_POST['post_id']))? sanitize_text_field($_POST['post_id']) : null;

            if ($post_id) {
                $feed = get_post_meta($post_id, 'yoda_seo_process_feed', true);
                $meta = get_post_meta($post_id, 'yoda-seo-log-details', true);
                if (!$feed) {
                    $parent_title = get_the_title($meta['post_id']);
                    $feed = $this->pre_ai_process($meta['post_id'], $parent_title, $meta['object']);
                }
                if ($feed) {

                    switch ($meta['object']) {
                        case 'rewrite':
                        case 'meta':
                            switch($meta['attributes']) {
                                case 'title':
                                    $type = 'Page: Title';
                                    break;
                                case 'description':
                                    $type = 'Page: Description';
                                    break;
                            }
                            break;
                        case 'menu':
                            $type = 'Menu: Title Text';
                            break;
                    }

                    $ai_content = $this->request_ai_content(
                        [
                            'type'        => $type,
                            'value'       => $feed,
                            'previous'    => $parent_title
                        ]
                    );

                    echo json_encode([
                        'status'        => 'success',
                        'suggestion'    => $ai_content
                    ]);

                }

            }

            exit();



        }


        /**
         *
         *
         * apply_suggestion
         * Apply the generate suggestion to the parent post
         * @since 1.6.10
         *
         *
         */
        public function apply_suggestion()
        {
            if ( ! wp_verify_nonce( $_POST['nonce'][0], 'yoda-seo-nonce' ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }

            $post_id = (isset($_POST['post_id']))? sanitize_text_field($_POST['post_id']) : null;
            $suggestion = (isset($_POST['suggestion']))? sanitize_text_field($_POST['suggestion']) : null;

            if ($post_id && $suggestion) {

                $log_details = get_post_meta($post_id, 'yoda-seo-log-details', true);

                if (!empty($log_details['post_id'])) {

                    $installed_metas = $this->check_supported_plugins();

                    switch ($log_details['object']) {
                        case 'rewrite':
                        case 'meta':

                            $meta_key = $this->seo_custom_title_meta_key;

                            if ($log_details['attributes'] === 'description') {
                                $meta_key = $this->seo_custom_meta_key;
                            }

                            if (empty($installed_metas)) {
                                update_post_meta($log_details['post_id'], $meta_key, $suggestion);
                            } else {
                                if (isset($log_details['source'])) {
                                    update_post_meta($log_details['post_id'], $log_details['source'], $suggestion);
                                } else {
                                    foreach ($installed_metas as $meta) {
                                        update_post_meta($log_details['post_id'], $meta['title'], $suggestion);
                                    }
                                }

                                //remove the custom metas if supported plugins are installed
                                delete_post_meta($log_details['post_id'], $meta_key);

                            }

                            break;
                        case 'menu':
                            $post_data = array(
                                'ID'           => $log_details['post_id'],
                                'post_excerpt' => $suggestion,
                            );
                            wp_update_post( $post_data );

                            break;
                    }

                    $this->update_status($post_id, 'completed');

                    echo json_encode([
                        'status' => 'success'
                    ]);

                } else {

                    echo json_encode([
                        'status'    => 'failed',
                        'message'   => __('ERROR: No parent post ID is found, please contact developer', 'yoda-tools')
                    ]);

                }
            } else {
                echo json_encode([
                    'status'    => 'failed',
                    'message'   => __('ERROR: No suggestion is found', 'yoda-tools')
                ]);
            }


            exit();

        }

    }
?>