<?php 
    /**
     *
     *
     * seoSuggestionTableList
     * render table list using WP_List_table
     * @since 1.6.10
     *
     *
     */
    namespace yoda_tools\includes\core;
    use yoda_tools\includes\core\utilities;


    if ( !function_exists( 'add_action' ) ){
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    if ( !function_exists( 'add_filter' ) ){
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    class seoSuggestionTableList extends \WP_List_Table
    {
	    protected $auto_process;
        private string $yoda_meta_key = 'yoda-seo-log-details';

        public int $perPage = 20;

        private $all_count = 0;

        private array $all_data = [];


	    public function __construct($args = array()) {
		    parent::__construct(array(
			    'singular' => 'SEO Suggestion', // Singular label
			    'plural'   => 'SEO Suggestions', // Plural label
			    'ajax'     => false // Whether the table supports AJAX
		    ));

		    $this->auto_process = $args['auto_process'] ?? false; // Set default value if not provided
	    }

        /**
         * Prepare the items for the table to process
         *
         * @return Void
         */
        public function prepare_items()
        {

            //put the data here
            $data = $this->table_data();

            $columns = $this->get_columns();
            $hidden = $this->get_hidden_columns();
            $sortable = $this->get_sortable_columns();

            $totalItems = $data[1];

            $this->set_pagination_args( array(
                'total_items' => $totalItems,
                'per_page'    => $data[2]
            ) );

            $this->_column_headers = array($columns, $hidden, $sortable);
            $this->items = $data[0];
        }

        /**
         * Override the parent columns method. Defines the columns to use in your listing table
         *
         * @return Array
         */
        public function get_columns()
        {
            $columns = array(
                'cb'                    => '<input type="checkbox"/>',
                'title'                 => __('Post title', 'yoda-tools'),
                'object'                => __('Object', 'yoda-tools'),
                'attributes'            => __('Attributes', 'yoda-tools'),
                'post_type'             => __('Post type', 'yoda-tools'),
                'suggestions'            => __('Suggestions', 'yoda-tools'),
                'schedule'              => __('Schedule','yoda-tools'),
                'date'                  => __('Date','yoda-tools')
            );

            return $columns;
        }

        function extra_tablenav( $which ) {

            $admin_url = get_admin_url().'admin.php?page=yoda-tools-seo-tools';
            $post_statuses = get_post_stati(array('post_type' => 'yoda-seo'));

            if ( $which == 'top' ) {
                // Add your custom filter above the table.
                $default_status = (isset($_GET['post_status']))? null : "current";
                ?>
                <!-- hidden fields -->
                <input type="hidden" name="page" value="yoda-tools-seo-tools">
                <input type="hidden" name="_wpnonce" id="_wpnonce" value="<?php echo wp_create_nonce('yoda-seo-suggestions'); ?>">
                <input type="hidden" name="_wp_http_referer">
                <?php if (isset($_GET['post_status'])) {
                    $ss = sanitize_text_field($_GET['post_status']);
                ?>
                    <input type="hidden" name="post_status" value="<?php echo esc_attr($ss); ?>">
                <?php } ?>
                <div class="yoda_most_top_table">
                    <div class="yoda_grid_table yoda_status_filter">
                        <ul>
                            <li><a href="<?php echo $admin_url; ?>" class="<?php echo esc_attr($default_status); ?>"><?php esc_html_e('All', 'yoda-tools'); ?></a> (<?php echo count($this->all_data); ?>)</li>
                            <?php
                            if (!empty( $post_statuses )) {
                                $status_count = $this->status_counts();
                                foreach ($post_statuses as $status) {
                                    $current_status = null;
                                    if (isset($_GET['post_status'])) {
                                        if ($_GET['post_status'] === $status) {
                                            $current_status = 'current';
                                        }
                                    }
                                    $status_label = str_replace('-', ' ', $status);

                                    if ($status === 'requires-attention' && $status_count[$status] > 0) {
                                        $current_status .= ' yoda_required_attention';
                                    }
                            ?>
                                    <li><a href="<?php echo $admin_url; ?>&post_status=<?php echo $status; ?>" class="<?php echo esc_attr($current_status); ?>"><?php echo ucwords(esc_html($status_label)); ?></a> (<?php echo esc_html($status_count[$status]); ?>)</li>
                            <?php
                                }
                            } ?>
                        </ul>
                    </div>
                    <div class="yoda_grid_table">
                        <label class="screen-reader-text" for="post-search-input"><?php esc_html_e('Search SEO Suggestions:','yoda-tools'); ?></label>
                        <?php $s_value = (isset($_GET['s']))? sanitize_text_field($_GET['s']) : null;  ?>
                        <input type="search" id="post-search-input" name="s" value="<?php echo esc_attr($s_value); ?>">
                        <input type="submit" id="search-submit" class="button button-medium" value="<?php esc_attr_e('Search SEO Suggestions','yoda-tools'); ?>">
                    </div>
                </div>
                <div class="alignleft actions bulkactions">
                    <label for="bulk-action-selector-top" class="screen-reader-text"><?php esc_html_e('Select bulk action','yoda-tools'); ?></label>
                    <select name="action" id="bulk-action-selector-top" >
                        <option value="-1"><?php esc_html_e('Bulk actions','yoda-tools'); ?></option>
                        <option value="yoda-bulk-process"><?php esc_html_e('Process','yoda-tools'); ?></option>
                        <option value="yoda-bulk-dismiss"><?php esc_html_e('Dismiss','yoda-tools'); ?></option>
                        <option value="trash"><?php esc_html_e('Remove','yoda-tools'); ?></option>
                    </select>
                    <input type="submit" id="doaction" class="button action" value="<?php esc_html_e('Apply', 'yoda-tools') ?>">
                </div>

                <div class="alignleft actions bulkactions">
                    <?php
                        $from_date_filter = (!empty($_GET['yoda-seo-filter-from']))? $_GET['yoda-seo-filter-from'] : null;
                        $to_date_filter = (!empty($_GET['yoda-seo-filter-to']))? $_GET['yoda-seo-filter-to'] : null;
                    ?>
                    <label for="bulk-action-selector-top" class="screen-reader-text"><?php esc_html_e('Select bulk action','yoda-tools'); ?></label>
                    <input type="text" name="yoda-seo-filter-from" id="yoda-seo-filter-from" placeholder="Date from" value="<?php echo esc_attr($from_date_filter); ?>">&nbsp;<input type="text" name="yoda-seo-filter-to" id="yoda-seo-filter-to" placeholder="Date to" value="<?php echo esc_attr($to_date_filter); ?>">
                    <input type="submit" id="doaction" class="button action" value="<?php esc_html_e('Filter', 'yoda-tools') ?>">
                </div>
                <?php
            }

        }

        /**
        * @Method name  column_cb
        * @Params       $cal
        * @description  display check box for all log data value
        */
        function column_cb($data)
        {
                return '<input type="checkbox" name="yoda_bulk_action[]" value="'.$data['ID'].'" />';
        }

        /**
         * Define which columns are hidden
         *
         * @return Array
         */
        public function get_hidden_columns()
        {
            return array();
        }


        /**
         *
         * column_title
         * @param array
         * @return array
         */
        public function column_title($data)
        {

            $url = get_edit_post_link($data['ID']);
            $name = $data['post_title'];
            $status_text = ucwords(str_replace("-"," ",$data['post_status']));

            $status_class = "";
            if ($data['post_status'] === 'requires-attention') {
                $status_class = "yoda_required_attention_status";
            }

            $status = "<span class=\"post-state\"> <b class=\"{$status_class}\">— {$status_text}</b></span>";

            $action_nonce = wp_create_nonce('yoda-seo-report-action');

            //get log meta
            $log_meta = get_post_meta($data['ID'],'yoda-seo-log-details',true);

            switch ($data['post_status']) {
                case 'failed':
                case 'completed':
                    $action_links = '<div class="row-actions"><span class="yoda-seo-process"><a href="'.get_admin_url().'post.php?post='.$log_meta['post_id'].'&amp;action=edit" class="yoda-seo-action" target="_blank">View Post</a></span></div>';
                    break;
                case 'dismissed':
                    $action_links = '<div class="row-actions"><span class="yoda-seo-process"><a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-process" data-id="'.$data['ID'].'" data-nonce="'.$action_nonce.'">Process</a> | </span><span class="yoda-seo-view"><a href="'.get_admin_url().'post.php?post='.$log_meta['post_id'].'&amp;action=edit" class="yoda-seo-action" target="_blank">View Post</a></span></div>';
                    break;
                case 'requires-attention':
                    $action_links = '<div class="row-actions"><span class="yoda-seo-dismiss"><a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-dismiss" data-id="'.$data['ID'].'" data-nonce="'.$action_nonce.'">Dismiss</a> | </span><span class="yoda-seo-view"><a href="'.get_admin_url().'post.php?post='.$log_meta['post_id'].'&amp;action=edit" class="yoda-seo-action" target="_blank">View Post</a></span></div>';
                    break;
                default:
                    $action_links = '<div class="row-actions"><span class="yoda-seo-process"><a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-process" data-id="'.$data['ID'].'" data-nonce="'.$action_nonce.'">Process</a> | </span><span class="yoda-seo-dismiss"><a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-dismiss" data-id="'.$data['ID'].'" data-nonce="'.$action_nonce.'">Dismiss</a> | </span><span class="yoda-seo-view"><a href="'.get_admin_url().'post.php?post='.$log_meta['post_id'].'&amp;action=edit" class="yoda-seo-action" target="_blank">View Post</a></span></div>';
                    break;
            }

            if ($log_meta['object'] === 'menu') {
                $action_links = '<div class="row-actions"><span class="yoda-seo-dismiss"><a href="javascript:void(0);" class="yoda-seo-action yoda-seo-action-dismiss" data-id="'.$data['ID'].'" data-nonce="'.$action_nonce.'">Dismiss</a></div>';
            }

            return "<strong><a href=\"{$url}\" class=\"row-title\" target=\"_blank\">{$name}</a>{$status}</strong>".$action_links;
        }


        /**
         *
         * column_status
         * @since 1.6.10
         *
         */
        public function column_object($data)
        {
            $seo_details = get_post_meta($data['ID'], $this->yoda_meta_key, true);
            return $seo_details['object'];
        }


        /**
         *
         * column_attributes
         * @since 1.6.10
         *
         */
        public function column_attributes($data)
        {
            $seo_details = get_post_meta($data['ID'], $this->yoda_meta_key, true);
            return $seo_details['attributes'];
        }

        /**
         *
         * column_attributes
         * @since 1.6.10
         *
         */
        public function column_post_type($data)
        {
            $seo_details = get_post_meta($data['ID'], $this->yoda_meta_key, true);
            $post_type = get_post_type($seo_details['post_id']);
            $post_type = ($post_type === 'nav_menu_item')? 'Menu' : $post_type;
            return ucfirst($post_type);

        }

        /**
         *
         *
         * column_suggestions
         * @since 1.6.10
         *
         *
         */
        public function column_suggestions($data)
        {
            $no_parent = get_post_meta($data['ID'],'yoda_seo_parent_post_not_exist', true);

            if ($no_parent) {
                return __('No fix available, parent post has been removed.', 'yoda-tools');
                return;
            }

            switch ($data['post_status'] ) {
                case 'pending':
                    $settings = get_option('yoda-tools-credits');
                    $auto = 'N';
                    if (isset($settings['auto_process'])) {
                        $auto = $settings['auto_process'];
                    }
                    if ($auto === 'N') {
                        return "<span class=\"yoda_view_suggestion\" data-id=\"".$data['ID']."\">".__('View suggestion', 'yoda-tools')."</span>";
                    } else {
                        return '━';
                    }
                    break;
                case 'requires-attention':
                    return "<span class=\"yoda_views_need_attention\" data-id=\"".esc_attr($data['ID'])."\">".__('Fix the issue', 'yoda-tools')."</span>";
                    break;
                default:
                    return '━';
                    break;
            }

            return;

        }

        /**
         *
         * column_schedule
         * @since 1.6.10
         *
         */
        public function column_schedule($data)
        {
            $status = get_post_status($data['ID']);
            if ($status === 'pending') {
                if ($this->auto_process === 'Y') {
                    return (!empty($this->pending_schedules[$data['ID']]))? $this->pending_schedules[$data['ID']] : __('Preparing schedule...', 'yoda-tools');
                } else {
                    return __('Not scheduled','yoda-tools');
                }

            } else {
                return '━';
            }
        }

        /**
         *
         * column_schedule
         * @since 1.6.10
         *
         */
        public function column_date($data)
        {
            $date_text = __('Last Modified', 'yoda-tools');
            $date = date('Y/m/d', strtotime($data['post_modified_gmt']));
            $time = date('h:i A', strtotime($data['post_modified_gmt']));
            $util = new utilities();
            $converted_date = $util->to_timezone($date.' '.$time, "Y/m/d == h:i A");
            $converted_date = str_replace('==','at',$converted_date);
            return $date_text.'<br/>'.$converted_date;
        }




        /**
         * Get the table data
         *
         * @return Array
         */
        private function table_data()
        {
            global $wpdb;
            $data = [];

            $sort = $this->sort_data("p.post_modified_gmt", "DESC");

            try{

                //custom filter
                $filter = $this->prepare_filter();

                //limit goes here
                $limit = $this->prepare_limit($this->perPage);
                $query = $this->query_string($wpdb, $sort, $filter, null);

                $main_query = $query.' '.$limit;

                $data = $wpdb->get_results($main_query, ARRAY_A);

                $this->check_for_attention($data);

                $data_nolimt = $wpdb->get_results("SELECT post_status FROM {$wpdb->prefix}posts WHERE post_type = 'yoda-seo'");
                $this->all_data = $data_nolimt;

                //get count
                $query_count = $wpdb->get_var("SELECT COUNT(ID) as count FROM {$wpdb->prefix}posts p WHERE p.post_type = 'yoda-seo' {$filter}");
                $this->all_count = $query_count;

                return [$data, $query_count, $this->perPage];

            } catch (\Exception $e) {
                print_r($e);
            }

        }

        /**
         * Define what data to show on each column of the table
         *
         * @param  Array $item        Data
         * @param  String $column_name - Current column name
         *
         * @return Mixed
         */
        public function column_default( $item, $column_name )
        {

            switch( $column_name ) {
                case 'title':
                case 'object':
                case 'attributes':
                case 'post_type':
                case 'suggestions':
                case 'schedule':
                case 'date':
                    return $item[ $column_name ];

                default:
                    return print_r( $item, true ) ;
            }
        }

        /**
             * @Method name  get_sortable_columns
             * @description  implement sorting on elments included in $sortable_columns array
             */
        public function get_sortable_columns()
        {
                $sortable_columns = array(
                    'title' => array(
                        'title',
                        false
                    ),
                    'date' => array(
                        'date',
                        false
                    )
                );
                return $sortable_columns;
        }


        /**
         * Allows you to sort the data by the variables set in the $_GET
         *
         * @return Mixed
         */
        public function sort_data( $orderby, $order )
        {
            if (!empty($_GET['orderby'])) {
                $orderby = sanitize_text_field($_GET['orderby']);
                switch ($orderby) {
                    case 'title':
                        $orderby = 'p.post_title';
                        break;
                    case 'date':
                        $orderby = 'p.post_modified_gmt';
                        break;
                }
            }
            // If order is set use this as the order
            if (!empty($_GET['order'])) {
                $order = sanitize_text_field($_GET['order']);
            }
            $order = "ORDER BY ".$orderby." ".$order;

            return $order;
        }


        /**
         *
         * prepare_limit
         * @since 1.6.10
         *
         */
        public function prepare_limit($perPage)
        {
            $limit = "LIMIT 0,{$perPage}";
            if (isset($_GET['paged'])) {
                $paged_ = sanitize_text_field($_GET['paged']);
                $paged_ -= 1;
                $offset = $paged_ * $perPage;
                $limit = 'LIMIT '.$offset.','.$perPage;
            }
            return $limit;
        }

        /**
         *
         *
         * query_string
         * @since 1.6.10
         * @param  $db - object
         *         $order - string
         *         $filter - string
         *
         *
         */
        public function query_string($db, $sort = '', $filter = '', $limit = '')
        {
            return "SELECT 
                p.ID,
                p.post_title,
                p.post_content,
                p.post_status,
                p.post_modified,
                p.post_modified_gmt
            FROM 
                {$db->prefix}posts p
            WHERE
                p.post_type = 'yoda-seo' {$filter} {$sort}";

        }

        /**
         *
         * status_counts
         * @since 1.6.10
         *
         */

        public function status_counts()
        {

            $pending = 0;
            $processing = 0;
            $failed = 0;
            $dismissed = 0;
            $completed = 0;
            $required_attention = 0;

            if (!empty($this->all_data)) {
                foreach ($this->all_data as $data) {
                    switch ($data->post_status) {
                        case 'pending':
                            $pending++;
                            break;
                        case 'processing':
                            $processing++;
                            break;
                        case 'failed':
                            $failed++;
                            break;
                        case 'dismissed':
                            $dismissed++;
                            break;
                        case 'completed':
                            $completed++;
                            break;
                        case 'requires-attention':
                            $required_attention++;
                            break;
                    }
                }
            }

            return [
                'pending'       => $pending,
                'processing'    => $processing,
                'failed'        => $failed,
                'dismissed'     => $dismissed,
                'completed'     => $completed,
                'requires-attention'    => $required_attention
            ];

        }

        /**
         *
         *
         * prepare_filter
         * build the string query from url
         * @since 1.6.10
         *
         */
        public function prepare_filter()
        {

            $filter = '';
            $data = $_GET;


            if (!empty($data)) {

                foreach ($data as $index => $query) {
                    if (!is_array($query)) {
                        $data[$index] = sanitize_text_field($query);
                    } else {
                        $query = array_map('sanitize_text_field', $query);
                        $data[$index] = $query;
                    }
                }

                if (isset($data['post_status'])) {
                    $filter .= "AND p.post_status = '".$data['post_status']."'";
                }

                if (isset($data['s'])) {
                    $filter .= "AND p.post_title LIKE '%".$data['s']."%'";
                }

                if (!empty($data['yoda-seo-filter-from'])) {
                    $data['yoda-seo-filter-from'] = date("Y-m-d", strtotime($data['yoda-seo-filter-from']));
                }

                if (!empty($data['yoda-seo-filter-to'])) {
                    $data['yoda-seo-filter-to'] = date("Y-m-d", strtotime($data['yoda-seo-filter-to']));
                }

                if (!empty($data['yoda-seo-filter-from']) && empty($data['yoda-seo-filter-to'])) {
                    $filter .= "AND p.post_modified_gmt >='{$data['yoda-seo-filter-from']}' AND p.post_modified_gmt <= NOW()";
                }

                if (empty($data['yoda-seo-filter-from']) && !empty($data['yoda-seo-filter-to'])) {
                    $filter .= "AND p.post_modified_gmt <= '{$data['yoda-seo-filter-to']}'";
                }

                if (!empty($data['yoda-seo-filter-from']) && !empty($data['yoda-seo-filter-to'])) {
                    if ($data['yoda-seo-filter-from'] === $data['yoda-seo-filter-to']) {
                        $filter .= "AND DATE(p.post_modified_gmt) = '{$data['yoda-seo-filter-from']}'";
                    } else {
                        $filter .= "AND p.post_modified_gmt >= '{$data['yoda-seo-filter-from']}' AND p.post_modified_gmt <= '{$data['yoda-seo-filter-to']}'";
                    }
                }


            }

            return $filter;

        }

        /**
         *
         *
         * check_for_attention
         * check for missing context and change the status to requires-attention
         * @since 1.6.10
         *
         *
         */
        public function check_for_attention($data = [])
        {
            $util = new utilities();
            if (!empty($data)) {
                foreach ($data as $row) {
                    if ($row['post_status'] === 'pending' || $row['post_status'] === 'requires-attention') {

                        //get meta
                        $log_meta = get_post_meta($row['ID'],'yoda-seo-log-details',true);

                        $parent_id = $log_meta['post_id'];

                        if ($log_meta['object'] === 'menu') {
                            $parent_id = get_post_meta($parent_id,'_menu_item_object_id', true);
                        }

                        //let's check the parent post if exist
                        $parent = get_post($parent_id);
                        $feed = null;

                        if ($parent) {
                            $feed = $util->check_post_context($parent_id, '');
                            delete_post_meta($row['ID'], 'yoda_seo_parent_post_not_exist');
                        } else {
                            update_post_meta($row['ID'], 'yoda_seo_parent_post_not_exist', true);
                        }


                        $status = 'pending';
                        if (!$feed) {
                            $status = 'requires-attention';
                        }

                        if ($row['post_status'] != $status) {
                            $post_data = array(
                                'ID'          => $row['ID'],
                                'post_status' => $status,
                            );
                            wp_update_post($post_data);
                        }

                    }
                }
            }
        }


    }
?>