<?php
    namespace yoda_tools\includes\core;
    use yoda_tools\includes\core\register;
    use yoda_tools\includes\core\seoSmartValidator;

    /* Does User Belong Here */
    if (!defined('YODA_TOOLS_PATH')) {
        header('Status: 403 Forbidden');
        header('HTTP/1.1 403 Forbidden');
        exit();
    }

    class settings
    {
        public string $table_email_logs;
        
        public function __construct(){ 
            global $wpdb;
            $this->table_email_logs = $wpdb->prefix . 'yoda_tools_email_logs';
            add_filter( 'robots_txt', array($this, 'filter_robots'), 99, 2 );
            
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_script_update', array($this , 'ajax_update_script') );
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_dashboard_option', array($this , 'ajax_dashboard_option') );
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_theme_css_update', array($this , 'ajax_update_css_theme') );
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_do_authenticate', array($this , 'ajax_do_authenticate') );
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_do_authenticate_logout', array($this , 'ajax_do_logout') );
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_get_credits', array($this , 'ajax_get_credits') );
            add_action( 'wp_ajax_'.YODA_TOOLS_NAMESPACE.'_save_credits', array($this , 'ajax_save_credits') );
            
            
            //call preview ajax
            add_action('admin_footer', [$this, 'call_preview']);
            add_action('wp_ajax_check_tools_preview_version', [$this, 'ajax_get_preview_version']);
            add_action('wp_ajax_yoda_tools_reset_update_transient', [$this, 'ajax_reset_version_transient']);
            add_action('wp_ajax_yoda_tools_set_preview_flag', [$this, 'ajax_set_yoda_flag']);
    
            
        }

        public static function register(): void { }

        public static function getInstance() {
			static $inst = null;
			if ( $inst === null ) {
				$inst = new settings();
			}
			return $inst;
		}

        // Plugin Settings
        public static function register_settings($fnSettings, $fnSlug): void {
            register_setting($fnSlug, $fnSlug . '_defaults');
            register_setting($fnSlug . '_page', $fnSlug . '_settings');
            if (is_array($fnSettings)) {
                foreach ($fnSettings as $setting) {
                    if ($setting['settings_type'] === 'section') {
                        \add_settings_section(
                            $setting['id'],
                            $setting['title'],
                            $setting['callback'],
                            $setting['page']
                        );
                    }
                    if ($setting['settings_type'] === 'field') {
                        \add_settings_field(
                            $setting['id'],
                            $setting['title'],
                            $setting['callback'],
                            $setting['page'],
                            $setting['section'],
                            $setting['args']
                        );
                    }

                    if ((int)get_option($fnSlug.'_defaults', 0) === 0) {
                        //Run resets
                        if (isset($setting['args']) && is_array($setting['args']) && array_key_exists('initial', $setting['args'])) {
                            $saved_options[$setting['args']['id']] = $setting['args']['initial'];
                            update_option($fnSlug.'_settings', $saved_options);
                        }
                    }
                }

                if ((int)get_option($fnSlug.'_defaults', 0) === 0) {
                    update_option($fnSlug.'_defaults', 1);
                }
            }
        }

        //Description Section callback
        public static function description_section_callback( ): void {	}

        //Append a settings field to the fields section.
        public static function settings_field_render( array $options = array() ): void {
            $saved_options = get_option( 'yoda-tools_settings' );

            $atts = array(
                'id'          => $options['id'],
                'type'        => ( isset( $options['type'] ) ? $options['type'] : 'text' ),
                'disabled'     => ( get_option('yoda-tools-mode-tracking') == 'on' ? "on" : "off" ),
                'class'       => $options['class'],
                'name'        => 'yoda-tools_settings[' . $options['name'] . ']',
                'value'       => ( array_key_exists( 'default', $options ) ? $options['default'] : null ),
                'label_for'   => ( array_key_exists( 'label_for', $options ) ? $options['label_for'] : false ),
                'description' => ( array_key_exists( 'description', $options ) ? $options['description'] : false )
            );

            if ( isset( $options['id'] ) ) {
                if ( isset( $saved_options[$options['id']] ) AND ( $saved_options[$options['id']] != '') )  {
                    $val = $saved_options[$options['id']];
                } else {
                    $val = '';
                    if(array_key_exists( 'default', $options )){
                        $val = $options['default'];
                    }
                }
                $atts['value'] = $val;
            }
            if ( isset( $options['type'] ) && $options['type'] == 'checkbox' ) {
                if ( $atts['value'] ) {
                    $atts['checked'] = 'checked';
                }
                $atts['value'] = true;
            }

            //Input type Checkbox
            if ( $atts['type'] == 'checkbox' ) {
                $html = sprintf( '<input type="%1$s" class="%2$s" id="%3$s" name="%4$s" value="%5$s" %6$s %7$s />', $atts['type'], $atts['class'], $atts['id'], $atts['name'], $atts['value'], ( isset( $atts['checked'] ) ? "checked=".$atts['checked'] : ''), ( $atts['disabled'] == 'off' ? "disabled=".$atts['disabled'] : '') );
                if ( array_key_exists( 'description', $atts ) ){
                    $html .= sprintf( '<p class="description">%1$s</p>', $atts['description'] );
                }
                echo $html;
            }

            //Input type Text
            if ( $atts['type'] == 'text' ) {
                $html = sprintf( '<input type="%1$s" class="%2$s" id="%3$s" name="%4$s" value="%5$s"/>', $atts['type'], $atts['class'], $atts['id'], $atts['name'], $atts['value'] );
                if ( array_key_exists( 'description', $atts ) ){
                    $html .= sprintf( '<p class="description">%1$s</p>', $atts['description'] );
                }
                echo $html;
            }

            //Input type Textarea
            if ( $atts['type'] == 'textarea' ) {
                $html = sprintf( '<textarea cols="60" rows="5" class="%1$s" id="%2$s" name="%3$s">%4$s</textarea>', $atts['class'], $atts['id'], $atts['name'], $atts['value'] );
                if ( array_key_exists( 'description', $atts ) ){
                    $html .= sprintf( '<p class="description">%1$s</p>', $atts['description'] );
                }
                echo $html;
            }
        }

        //generic function for rendering table data
        public static function table_data_render( $items, $settings ): void {
            $default_settings = array(
                'columns'					=>	array(),
                'hover_actions'				=>	array(),
                'bulk_actions'				=>	array(),
                'hover_action_column_key'	=>	'',			// int of column to set value= in URL and rel tag= for using in JS.
                'action'					=>	'',
                'after_bulk'				=>	'',
                'css'						=>	'',
            );

            // Merge defaults.
            $settings = array_merge( $default_settings, $settings );

            ?>
            <div style="<?php echo $settings['css']; ?>" class="render-table-wrapper">
                <table class="widefat striped">
                    <thead>
                    <tr class="thead">
                        <?php
                        foreach ( $settings['columns'] as $column ) {
                            echo '<th>' . $column . '</th>';
                        }
                        ?>
                    </tr>
                    </thead>
                    <tfoot>
                    <tr class="thead">
                        <?php
                        foreach ( $settings['columns'] as $column ) {
                            echo '<th>' . $column . '</th>';
                        }
                        ?>
                    </tr>
                    </tfoot>
                    <tbody>
                    <?php
                    // LOOP THROUGH EACH ROW.
                    foreach ( (array)$items as $item_id => $item ) {
                        ?>
                        <tr class="entry-row" id="rowitem-<?php echo $item_id; ?>">
                            <td>
                                <?php
                                if ( is_array( $item['0'] ) ) {
                                    if ( $item['0'][1] == '' ) {
                                        echo '&nbsp;';
                                    } else {
                                        echo $item['0'][1];
                                    }
                                } else {
                                    if ( $item['0'] == '' ) {
                                        echo '&nbsp;';
                                    } else {
                                        echo $item['0'];
                                    }
                                }
                                ?>
                                <div class="row-actions" style="margin-top: 10px;">
                                    <?php
                                    $i = 0;
                                    foreach ( $settings['hover_actions'] as $action_slug => $action_title ) {
                                        // Display all hover actions.
                                        $i++;
                                        if ( $settings['hover_action_column_key'] != '' ) {
                                            if ( is_array( $item[$settings['hover_action_column_key']] ) ) {
                                                $hover_action_column_value = $item[$settings['hover_action_column_key']][0];
                                            } else {
                                                $hover_action_column_value = $item[$settings['hover_action_column_key']];
                                            }
                                        } else {
                                            $hover_action_column_value = '';
                                        }

                                        if ( strstr( $action_slug, '/' ) === false ) { // Word hover action slug.
                                            $hover_link= $action_slug . '=' . $item_id . '&value=' . $hover_action_column_value;
                                        } else {
                                            // URL hover action slug so just append value to URL.
                                            $hover_link = $action_slug . $hover_action_column_value;
                                        }

                                        // Some host's don't allow get params that end in .zip
                                        if ( '.zip' == substr( $hover_link, -4 ) ) {
                                            $hover_link .= '&bub_rand=' . rand( 100, 999 );
                                        }

                                        echo '<a href="' . $hover_link . '" class="hoveraction_' . $action_slug . '" rel="' . $hover_action_column_value . '">' . $action_title . '</a>';
                                        if ( $i < count( $settings['hover_actions'] ) ) {
                                            echo ' | ';
                                        }
                                    }
                                    ?>
                                </div>
                            </td>
                            <?php
                            $count = count( $item );

                            // LOOP THROUGH COLUMNS FOR THIS ROW.
                            for ( $i = 1; $i < $count; $i++ ) {
                                if ( ! isset( $item[$i] ) ) { continue; }
                                // This row does not have a corresponding index-based item.  It is probably a named key not for use in table?
                                ?>
                                <td>
                                    <?php
                                    if ( $item[$i] == '' ) {
                                        echo '&nbsp;';
                                    } else {
                                        echo $item[$i];
                                    }
                                    ?>
                                </td>
                                <?php
                            }
                            ?>
                        </tr>
                        <?php
                    }
                    ?>
                    </tbody>
                </table>
            </div>
            <?php
        }

        public function customPhp($phpContent, $other_details = []){
            $data_out = "";
            $data_tmp = $phpContent;
            $data_tmp = trim( $data_tmp );           // Cleaning
            $data_tmp = ltrim( $data_tmp, '<?php' ); // Cleaning
            $data_tmp = rtrim( $data_tmp, '?>' );    // Cleaning

            $data_out .= $data_tmp;

            try {
                return @eval($data_out);
            } catch (\Throwable $e) {
                // handle the error here
                $user_id = get_current_user_id();
                //display the error
                $trace = $e->getTrace();

                if ( $user_id ) {
                    if (current_user_can('administrator')) {
                        echo "<div style=\"padding: 15px; background: #f7c7c3; color: black;\">Yoda Error: " . $e->getMessage();
                        echo "<br/> Error traced to:<pre style=\"background: white;\">";
                        foreach ($trace as $trac) {
                            if (isset($trac['args'][1]) && count($trac['args'][1]) > 1) {
                                print_r($trac['args']);
                            }
                        }
                        echo "</pre>";
                        echo "</div>";
                        
                    }
                }
                
            }
        }

        function getOption($value){
            return get_option(YODA_TOOLS_SLUG.'-'.$value);
        }

        /**
         * 
         * 
         * getSettings
         * retrieve yoda settings
         * @since 1.3.2
         * @param $value - string {yoda_tools_events | yoda_tools_enhancedlinkatt | yoda_tools_displayfeatures | yoda_tools_userid}
         * @return mixed int | null
         * 
         * 
         */
        function getSettings($value = null)
        {
            //bail out if empty
            if (!$value)
                return; 

            $settings = get_option(YODA_TOOLS_SLUG.'_settings');
            return ( !empty($settings[$value]) )? $settings[$value] : null;
        }

        
        function ajax_update_script()
        {
            $errorLog = [];

            if(isset($_POST['id']) && !empty($_POST['id']) && isset($_POST['status']) && !empty($_POST['status'])){
                $id = htmlspecialchars($_POST['id']);
                $status = htmlspecialchars($_POST['status']);
                $type = htmlspecialchars($_POST['type']);

                $data = array(
                    'ID' => $id,
                    'post_type' => $type,
                    'post_status' => $status,
                );
                wp_update_post($data);
                if (!is_wp_error($id)) {
                    //the post is valid
                    $notifications = "Updated Successfully";
                }
            }

            if(empty($errorLog)){
                echo json_encode(["status" => "success","message"=>"saved"]);
            }else{
                echo json_encode(["status" => "success","message"=>"saved but with some errors"]);
            }

            die();

        }

        function ajax_dashboard_option(){
            $errorLog = [];

            if(isset($_POST['mode']) && !empty($_POST['mode']) && isset($_POST['status']) && !empty($_POST['status'])){
                $status = htmlspecialchars($_POST['status']);
                $mode = htmlspecialchars($_POST['mode']);
                
                if ($mode === 'seo-tools' && $status === 'off') {
                    //logout the SocialYODA if the user is logged in
                    if ($this->getOption('apikey')) {
                        $this->do_api_logout();
                    }
                }

                update_option(YODA_TOOLS_SLUG.'-mode-'.$mode, $status);
            }

            echo json_encode(["status" => "success", "message"=>"saved"]);

            die();
        }

        function ajax_update_css_theme(){
            if ( isset( $_POST['theme'] ) ) {
                $themeOption = utilities::clean_string('theme');
                update_option(YODA_TOOLS_SLUG.'-css-theme-option', $themeOption);

                echo json_encode(["status" => "success", "message"=>"saved"]);
            }
            die();
        }

        function getThemeOption(){
            $themeMode = $this->getOption('css-theme-option');

            return $themeMode;
        }

        function createTable(): void {
            global $wpdb;
            $charset_collate = $wpdb->get_charset_collate();
            $email_log = "CREATE TABLE {$this->table_email_logs} (
                log_id INT(11) NOT NULL AUTO_INCREMENT,
                date_time_sent DATETIME,
                email_subject TEXT,
                sent_to TEXT,
                email_status TEXT,
                PRIMARY KEY  (log_id)
            ) $charset_collate;";

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($email_log);
        }

        function checkTableLogs(): bool
        {
            global $wpdb;
            $table_email_logs = $this->table_email_logs;
            if($wpdb->get_var("SHOW TABLES LIKE '{$table_email_logs}'") != $table_email_logs) {
                $this->createTable();
                //update post type css
                $update_pt_css_data = "UPDATE wp_posts SET post_type = 'yoda-css' WHERE `post_type` = %s AND post_title = %s";
                $wpdb->query($wpdb->prepare($update_pt_css_data, array('yoda-script', 'yoda-css')));
                
                //update post type php
                $update_pt_php_data = "UPDATE wp_posts SET post_type ='yoda-php' WHERE `post_type` = %s AND post_title = %s";
                $wpdb->query($wpdb->prepare($update_pt_php_data, array('yoda-script', 'yoda-php')));
            }
            
            if($wpdb->get_var("SHOW TABLES LIKE '{$table_email_logs}'") == $table_email_logs) {
                $email_data = $wpdb->query("SHOW COLUMNS FROM `{$table_email_logs}` LIKE 'email_data'");
                if($email_data != 1){
                    $wpdb->query("ALTER TABLE `{$table_email_logs}` ADD `email_data` LONGTEXT");
                }

                $transcript = $wpdb->query("SHOW COLUMNS FROM `{$table_email_logs}` LIKE 'transcript'");
                if($transcript != 1){
                    $wpdb->query("ALTER TABLE `{$table_email_logs}` ADD `transcript` LONGTEXT");
                }
            }

            return false;
        }

        function filter_robots( $robots_txt, $public ): string
        {
		    $sitemap = $this->maybe_add_xml_sitemap($robots_txt);

            return trim( $robots_txt . PHP_EOL . $sitemap . PHP_EOL );
        }

        function maybe_add_xml_sitemap($output): string {

            $output = "";
            /**
             * Get site path.
             */
            $site_url = parse_url( site_url() );
            $path	  = ( ! empty( $site_url[ 'path' ] ) ) ? $site_url[ 'path' ] : '';
    
            /**
             * Remove line that allows robots to access AJAX interface.
             */
            $robots = preg_replace( '/Allow: [^\0\s]*\/wp-admin\/admin-ajax\.php\n/', '', $output );
    
            /**
             * If no error occurred, replace $output with modified value.
             */
            if ( null !== $robots ) {
                $output = $robots;
            }
            /**
             * Calculate and add a "Sitemap:" link.
             * Modify as needed.
             */
            $output .= "Sitemap: {$site_url[ 'scheme' ]}://{$site_url[ 'host' ]}/sitemap_index.xml\n";

            return $output;
        }

        /**
         * 
         * 
         * ajax_do_authenticate
         * @since 1.4.0
         * 
         * 
         */
        public function ajax_do_authenticate()
        {
            if (!wp_verify_nonce( $_POST['nonce'],  YODA_TOOLS_SLUG.'-save-authenticate-nonce' )) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __('Invalid nonce', 'yoda-tools') 
                ]);
                exit();
            }

            //login
            $fields = array_map('sanitize_text_field', $_POST);
            $register = new register();
            $register->set_username($fields['username']);
            $register->set_password($fields['password']);
            //set the deviceID in complete URL structure
            $site_url = get_site_url();
            $register->set_deviceID($site_url);

            try {
                $response = $register->authenticate();
            } catch(\Exception $e) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __($e->getMessage(), 'yoda-tools') 
                ]);
                exit();
            }
            
            if ($response['status'] === 'success') {
                update_option(YODA_TOOLS_SLUG.'-apikey', $response['apiKey']);
            }
            echo json_encode($response);
            exit();
            
        }

        /**
         * 
         * 
         * ajax_do_logout
         * @since 1.4.0
         * 
         * 
         */
        public function ajax_do_logout()
        {
            if (!wp_verify_nonce( $_POST['nonce'],  YODA_TOOLS_SLUG.'-tool-logout-nonce' )) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __('Invalid nonce', 'yoda-tools') 
                ]);
                exit();
            }
            $this->do_api_logout();
            echo json_encode(['status' => 'success']);
            exit();

        }

        /**
         * 
         * ajax_get_credits
         * @since 1.4.0
         * 
         */
        public function ajax_get_credits()
        {
            if (!wp_verify_nonce( $_POST['nonce'],  YODA_TOOLS_SLUG.'-tool-credits-nonce' )) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __('Invalid nonce', 'yoda-tools') 
                ]);
                exit();
            }

            $user_credit = new register();
            $user_credit->set_api_key($this->getOption('apikey'));
            $credits = $user_credit->get_credits();

            echo json_encode(
                [
                    'status' => 'success',
                    'message' => $credits
                ]
            );
            exit();

        }


        /**
         * 
         * 
         * ajax_save_credits
         * @since 1.4.0
         * 
         * 
         */
        public function ajax_save_credits()
        {
            if (!wp_verify_nonce( $_POST['nonce'],  YODA_TOOLS_SLUG.'-tool-credits-nonce' )) {
                echo json_encode([
                    'status' => __('failed','yoda-tools'),
                    'message' => __('Invalid nonce', 'yoda-tools') 
                ]);
                exit();
            }

            $option = sanitize_text_field($_POST['option']);
            $auto_process = sanitize_text_field($_POST['auto_process']);
            $data = ['credits' => $option, 'auto_process' => $auto_process];

            if ($data) {
                update_option('yoda-tools-credits', $data);
            }

            echo json_encode([
                'status' => __('success','yoda-tools'),
                'message' => ''
            ]);
            exit(); 

        }

        /**
         * 
         * do_api_logout
         * @since 1.5.0
         * 
         */
        public function do_api_logout()
        {
            $register = new register();
            $register->set_api_key($this->getOption('apikey'));
            $response = $register->logout();
            $response = json_decode($response);

            delete_option(YODA_TOOLS_SLUG.'-apikey');
            //remove scheduler
            wp_clear_scheduled_hook('yoda_seo_smart_validator');
            wp_clear_scheduled_hook('yoda_seo_smart_processor');
            wp_clear_scheduled_hook('yoda_site_details_check');
        }

        /**
         * 
         * 
         * get_themes
         * get all installed themes and their statuses
         * @param
         * @return array
         * @since 1.5.5
         * 
         */
        public function get_themes(): array {
            $get_themes = wp_get_themes();
            
            $arr = [];
            $wrap = [];

            foreach ($get_themes as $data) {

                $arr['Name'] = $data->get('Name');
                $arr['Version'] = $data->get('Version');

                if ($data->get('Name') == wp_get_theme()->get('Name')) {
                    $arr['Status'] = "Active";
                } else {
                    $arr['Status'] = "In-active";
                }

                if (!empty($data->get('Template'))) {
                    $arr['Parent'] = $data->get('Template');
                } else {
                    unset($arr['Parent']);
                }

                $wrap[] = $arr;
            }

            return $wrap;
        }

        /**
         * 
         * 
         * get_plugins
         * get all installed plugins and their statuses
         * @param
         * @return array
         * @since 1.5.5
         * 
         */
        public function get_plugins(): array {
            if ( ! function_exists( 'get_plugins' ) ) {
                require_once ABSPATH . 'wp-admin/includes/plugin.php';
            }

            $get_plugins = get_plugins();
            $arr = [];
            $wrap = [];

            foreach ($get_plugins as $key => $data) {
                $arr['Name'] = $data['Name'];
                $arr['Version'] = $data['Version'];

                if(is_plugin_active($key)){
                    $arr['Status'] = "Active";
                }else{
                   $arr['Status'] = "In-active";
                }

                $wrap[] = $arr;
            }

            return $wrap;

        }

        /**
         * 
         * 
         * get_post_count
         * get the number of posts
         * @return array
         * @since 1.5.5
         * 
         * 
         */
        public function get_post_count(): array {
            $count_posts = wp_count_posts();
            $individual_count = [];
            $total = 0;
            if (!empty($count_posts)) {
                foreach ($count_posts as $index => $count) {
                    $total = $total + $count;
                    if ($count > 0) {
                        $individual_count[$index] = $count;
                    }
                }
            }
            return [
                'total_count' => $total,
                'posts_count' => $individual_count
            ];
        }

        /**
         * 
         * 
         * get_page_count
         * get the number of pages
         * @return array
         * @since 1.5.5
         * 
         * 
         */
        public function get_page_count(): array {
            $count_pages = wp_count_posts('page');
            $total = 0;
            $individual_count = [];
            if (!empty($count_pages)) {
                foreach ($count_pages as $index => $count) {
                    $total = $total + $count;
                    if ($count > 0) {
                        $individual_count[$index] = $count;
                    }
                }
            }
            return [
                'total_count' => $total,
                'page_count' => $individual_count
            ];
        }

        /**
         * 
         * 
         * get_wp_version
         * get the version of wordpress
         * @return float
         * @since 1.5.5
         * 
         * 
         */
        public function get_wp_version(): float {
            global $wp_version;
            return $wp_version;
        }


        /**
         * 
         * 
         * wp_overview
         * get website overview details
         *          - List of themes and statuses
         *          - List of plugins and statuses
         *          - No. of posts
         *          - No. of pages
         *          - WordPress version
         * @param
         * @return array
         * @since 1.5.5
         * 
         */
        public function wp_overview(): array {
            return [
                'projectId'           => '00000000-0000-0000-0000-000000000000',
                'url'                 => get_site_url(),
                'version'             => $this->get_wp_version(),
                'themes'              => $this->get_themes(),
                'plugins'             => $this->get_plugins(),
                'Number_of_Posts'     => $this->get_post_count()['total_count'],
                'Number_of_Pages'     => $this->get_page_count()['total_count']
            ];
        }

        /**
         * 
         * call_preview
         * call preview ajax request
         * @since 1.6.3
         * 
         */
        public function call_preview()
        {
            if (isset($_GET['page'])) {
                if ($_GET['page'] === 'yoda-tools-settings') {
                    ?>
                    <script type="text/javascript">
                        yoda_tools_check_preview('<?php echo wp_create_nonce('yoda-preview-version') ?>');
                    </script>
                    <?php
                }
            }
        }

        /**
         * 
         * ajax_get_preview_version
         * @since 1.6.3
         * 
         */
        public function ajax_get_preview_version()
        {
            

            if ( ! wp_verify_nonce( $_POST['nonce'], 'yoda-preview-version' ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }
    
            $preview = wp_remote_get( 'https://plugin.maennche.com/wordpress/yoda-tools/preview' , array(
                'timeout' => 10,
                'headers' => array(
                    'Accept' => 'application/json'
                ) )
            );
            $details = json_decode($preview['body']);

            $has_new_version = null;

            $result = version_compare(YODA_TOOLS_VERSION, $details->new_version, '<');
            if ($result) {
                $has_new_version = $details->new_version;
            } else {
                delete_option('yoda_tools_get_preview_version');
            }
    
            include_once YODA_TOOLS_PATH.'/views/preview-version.php';
    
            exit();

        }

        /**
         * 
         * 
         * ajax_reset_version_transient
         * reset the upgrader transient version
         * @since 1.6.3
         * 
         * 
         */
        public function ajax_reset_version_transient()
        {
            if ( ! wp_verify_nonce( $_POST['nonce'], 'yoda-preview-version' ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }

            delete_transient('yoda_tools_plugin_update');

            echo json_encode(['status' => 'success']);

            exit();
        }

        /**
         * 
         * ajax_set_yoda_flag
         * set preview version flag to get the preview endpoint
         * @since 1.6.3
         * 
         */
        public function ajax_set_yoda_flag()
        {
            if ( ! wp_verify_nonce( $_POST['nonce'], 'yoda-preview-version' ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }

            update_option('yoda_tools_get_preview_version', 'Y');
            delete_transient('yoda_tools_plugin_update');
            $preview_version_prepared = true;
            include_once YODA_TOOLS_PATH.'/views/preview-version.php';

            exit();
        }

    }