<?php
    /**
     *
     *
     * spamGuard
     * class for basic anti spam bot functionality using honeypot method
     *
     * @since 1.3.2
     * @author - maennche
     *         - Juls Terobias
     *
     *
     */
    namespace yoda_tools\includes\core;
    use yoda_tools\includes\core\settings;
    use yoda_tools\includes\core\advancedSpamGuard;

    /* Does User Belong Here */
    if (!defined('YODA_TOOLS_PATH')) {
        header('Status: 403 Forbidden');
        header('HTTP/1.1 403 Forbidden');
        exit();
    }

    class spamGuard
    {


        /**
         *
         * This is the default field name if the user did not make any changes in the settings
         * @since 1.3.2
         *
         */
        private $spam_guard_field_name = "yodatoolsSG-field";

        /**
         *
         * This is the default message to bot if the user did not make any changes in the settings
         * @since 1.3.2
         *
         */
        private $return_message = "Yoda error 01: Bot submission is detected";

        /**
         *
         * this will take the settings object
         * @since 1.3.2
         *
         *
         */
        private $options;


        /**
         *
         *
         * advance_spamguard
         * instance of advanceSpamGuard class if null it means the user is not logged in meetYoda
         * @since 1.5.5
         *
         *
         */
        private $advance_spamguard = null;


        public function __construct(){

            $this->options = settings::getInstance();

            if ( $this->options->getOption('spam-guard-name')) {
                $this->spam_guard_field_name = $this->options->getOption('spam-guard-name');
            }

            if ( $this->options->getOption('spam-guard-message')) {
                $this->return_message =  $this->options->getOption('spam-guard-message');
            }

            //load the assets properly
            add_action('wp_enqueue_scripts', [$this,'load_spam_assets']);
            add_action('admin_enqueue_scripts', [$this,'load_admin_spam_assets']);

            //add jump start point
            add_action( 'login_form', [$this,'field_jumpstart']);
            add_action( 'woocommerce_login_form', [$this,'field_jumpstart']);
            add_action( 'register_form', [$this,'field_jumpstart']);


            //contact form 7
            add_filter( 'wpcf7_validate', [$this, 'wpcf7_check'], 10, 2 );

            //gravity form
            add_action( 'gform_validation', [$this, 'gform_check'], 10, 1);

            //wp comment forms
            add_filter( 'preprocess_comment', [$this, 'wp_comment_form_check'], 10, 1);

            //wp forms
            //not supporting for now
            //add_filter( 'wpforms_process_before', [$this, 'wp_forms_check'], 10, 2 );

            //wp_login
            //add_filter( 'authenticate', [$this, 'wp_login_check'],10,3);

            //registration
            add_filter( 'register_post', [$this, 'wp_registration_check'], 10, 3 );


            //save settings
            add_action('admin_init', [$this, 'save_settings']);

            //append widget to footer
            add_action('wp_footer', [$this,'append_widget']);


            if ($this->options->getOption('apikey') && $this->options->getOption('spam-advanced-spamguard')) {
                //instantiate advanced spam guard if user is logged in
                $this->advance_spamguard = new advancedSpamGuard($this->options->getOption('apikey'));
            }


            //check with API after submission
            add_action('gform_disable_notification',[$this, 'check_pre_submission'], 10, 4);

        }


        /**
         *
         *
         * load_spam_assets
         * @since 1.3.3
         *
         *
         */
        public function load_spam_assets()
        {
            wp_register_script( YODA_TOOLS_SLUG.'-spamGuard' , YODA_TOOLS_URL.'js/yodatoolSGuard.js', array('jquery') );
            wp_enqueue_script( YODA_TOOLS_SLUG.'-spamGuard' );
	        wp_add_inline_script(
		        YODA_TOOLS_SLUG.'-spamGuard',
		        'const yodaSGfield = \'<span class="yodaSGcontainer"><input type="text" name="' . $this->spam_guard_field_name . '"></span>\';'
	        );

	        wp_add_inline_script(
		        YODA_TOOLS_SLUG.'-spamGuard',
		        'const yodaSGDefaulMessage = "' . addslashes($this->return_message) . '";'
	        );
            wp_enqueue_style(YODA_TOOLS_SLUG.'-spamGuard', YODA_TOOLS_URL.'css/public_css.css');
        }


        /**
         *
         * load_admin_spam_assets
         * @since 1.3.3
         *
         */
        public function load_admin_spam_assets()
        {
            wp_enqueue_script( YODA_TOOLS_SLUG.'-admin-spamGuard' );
            wp_register_script( YODA_TOOLS_SLUG.'-admin-spamGuard' , YODA_TOOLS_URL.'js/yodatooladminSGuard.js', array('jquery') );
        }

        /**
         *
         *
         * save_settings
         * let's save the data from the settings page
         * @since 1.3.2
         *
         *
         */
        public function save_settings ()
        {

            if (isset($_POST['yoda_save_sg_settings'])) {
                 //check nonce
                if ( ! wp_verify_nonce( $_POST['yoda_sg_nonce'], 'yoda-spam-guard' ) ) {
                    //we will do nothing
                    return;
                }

                $option_fields = [
                    'yoda_sg_advanced_spamguard'    => '-spam-advanced-spamguard',
                    'yoda_sg_field_name'            => '-spam-guard-name',
                    'yoda_sg_error_message'         => '-spam-guard-message',
                    'yoda_sg_disable_test_widget'   => '-spam-guard-widget',
                    'yoda_sg_advanced_comments'     => '-spam-advanced-comments',
                    'yoda_sg_advanced_registration' => '-spam-advanced-registration',
                    'yoda_sg_advanced_single_form'  => '-spam-advanced-single-forms'
                ];

                foreach ($option_fields as $index => $feld) {
                    if (isset($_POST[$index])) {

                        if ($index === 'yoda_sg_advanced_single_form') {
                            //clean the array
                            $data = array_map('sanitize_text_field', $_POST[$index]);
                        } else {
                             //clean
                            $data = utilities::clean_string($index);
                            //save
                        }

                        update_option(YODA_TOOLS_SLUG.$feld, $data);
                    } else {
                        if (
                            $feld === '-spam-advanced-spamguard' ||
                            $feld === '-spam-advanced-comments' ||
                            $feld === '-spam-advanced-registration' ||
                            $feld === '-spam-advanced-single-forms'
                        ) {
                            delete_option(YODA_TOOLS_SLUG.$feld);
                        }
                    }
                }

                //redirect to avoid browser's reposting
                wp_redirect(get_admin_url().'admin.php?page=yoda-tools-spam-guard&success=1');
                exit();
            }
        }


        /**
         *
         *
         * field_jumpstart
         * this will add dummy html inside the form so that we have something to call when we add our honeypot field
         * @since 1.3.2
         *
         *
         */
        public function field_jumpstart()
        {
            echo '<input type="hidden" name="'.$this->spam_guard_field_name.'" class="yodafieldJumpStart" />';
        }

        /**
         *
         *
         * validate_submission
         * @since 1.3.2
         *
         * @param $data - array ($_POST)
         *
         * @return  bool
         *
         */
        private function validate_submission($data): bool
        {
            if (!empty($data[$this->spam_guard_field_name])) {
                //likely a bot submission
                $this->do_log();
                return true;
            } else {
                //likely a legit submission
                return false;
            }
        }


        /**
         *
         *
         * do_log
         * let's log the bot incident?
         * maybe in the future we will have a table for this
         * @since 1.3.2
         *
         *
         */
        private function do_log()
        {
            $get_count = get_option(YODA_TOOLS_SLUG.'-botLog');
            $get_count++;
            update_option(YODA_TOOLS_SLUG.'-botlog', $get_count);
        }


        /**
         *
         *
         * wpcf7_check
         *
         * Refer to contactform 7 documentation for more details
         *
         * @since 1.3.2
         * @param - $result (UNKNOWN)
         *         - $tags (UNKNOWN)
         *
         * @return $result (UNKNOWN)
         *
         */
        public function wpcf7_check($result, $tags)
        {

            if ($this->validate_submission($_POST)) {
                $result->invalidate('', $this->return_message);
            } else {
                //bot pass check using AI
                if ($this->advance_spamguard) {

                    $form_id = $_POST['_wpcf7'];
                    $get_settings_form = $this->options->getOption('spam-advanced-single-forms');

                    //@since 1.6.7
                    //check if single forms are enabled
                    if (!empty($get_settings_form)) {
                        //let's check if the form is enabled
                        if (!in_array($form_id, $get_settings_form)) {
                            //means the form is off do not call API
                            return $result;
                        }
                    }

                    $res = $this->advance_spamguard->check_contact_form_seven_ajax($_POST);
                    if ($res) {
                        $result->invalidate('', $this->return_message);
                    }


                }
            }
            return $result;
        }


        /**
         *
         *
         * gform_check
         * @since 1.3.2
         *
         * please refer to gravity form documentation for more details
         *
         * @param $validation_result (unknown)
         *
         * @return
         *
         *
         */
        public function gform_check($validation_result)
        {
            $form = $validation_result['form'];
            if ($this->validate_submission($_POST)){
                $validation_result['is_valid'] = false;
                foreach( $form['fields'] as &$field ) {
                    if ( (int) $field->id === 1 ) {
                        $field->failed_validation = true;
                        $field->validation_message = $this->return_message;
                        break;
                    }
                }
                $validation_result['form'] = $form;
            } else {
                //check using AI
                //bot pass check using AI
                //if honeypot passed let's check with api after submission
                //code below move to after submission
            }
            return $validation_result;
        }


        /**
         *
         *
         * check_after_submission
         * check after submission with api and mark spam if the response is true
         * $res: TRUE     = spam
         * $res: FALSE    = not spam
         * @since 1.6.5
         *
         *
         */
        public function check_pre_submission($is_disabled, $notification, $form, $entry)
        {
            if ($this->advance_spamguard) {

                $get_settings_form = $this->options->getOption('spam-advanced-single-forms');

                if (!empty($get_settings_form)) {
                    //now let's check
                    if (!in_array($form['id'], $get_settings_form)) {
                        //means the form is off do not call the API
                        return $is_disabled;
                    }
                }

                $res = $this->advance_spamguard->gravity_form_check($_POST, ['id' => $form['id'], 'title' => $form['title']]);
                if ($res) {
                    if (class_exists('GFAPI')) {
                        //create data manually and save as spam
                        $entry['status'] = 'spam';
                        \GFAPI::update_entry($entry);
                        //abort to avoid notification
                        return true;
                    }
                }
            }
            return $is_disabled;
        }


        /**
         *
         *
         * wp_comment_form_check
         * @since 1.3.2
         * @param $commentdata (array)
         * @return array
         *
         *
         */
        public function wp_comment_form_check( $commentdata ) {
            if ($this->validate_submission($_POST)){
                wp_die( $this->return_message );
            } else {
                //bot passed
                if ($this->advance_spamguard) {

                    //@since 1.6.7
                    //check if the advanced comment is enabled
                    $do_advanced_comment = $this->options->getOption('spam-advanced-comments');

                    if (!empty($do_advanced_comment) && $do_advanced_comment === 'Y') {
                        $res = $this->advance_spamguard->comment_check($commentdata);
                        if ($res) {
                            wp_die( $this->return_message );
                        }
                    }

                }

            }
            return $commentdata;
        }

        /**
         *
         *
         * not supporting for now
         * wp_forms_check
         * @since 1.3.2
         *
         *
         */
        public function wp_forms_check($entry, $form_data){
            if ($this->validate_submission($_POST)){
                wpforms()->process->errors[$form_data['id']]['0'] = $this->return_message;
            } else {
                //do this after testing
            }
        }

        /**
         *
         * wp_login_check
         *
         * @since 1.3.2
         *
         * @param $user (obj)
         *        $uername (string)
         *        $password (string)
         * @return
         *
         *
         */
        public function wp_login_check($user, $username, $password)
        {

            if (empty($username) || empty($password)) {
                return $user;
            }

            if ( !empty( $_POST ) ) {
                if ($this->validate_submission($_POST)){
                    wp_die( $this->return_message );
                } else {
                    //bot passed
                    if ($this->advance_spamguard) {
                        $res = $this->advance_spamguard->check_login($_POST);
                        if ($res) {
                            wp_die( $this->return_message );
                        }
                    }

                }
            }
            return $user;
        }

        /**
         *
         *
         * wp_registration_check
         * @since 1.3.2
         *
         *
         */
        public function wp_registration_check($user_login, $user_email, $errors )
        {

            if (empty($user_login) || empty($user_email)) {
                return $errors;
            }

            if ($this->validate_submission($_POST)){
                $errors->add( 'yoda_spam_guard_response', $this->return_message );
            } else {
                //bot passed
                if ($this->advance_spamguard) {

                    //@since 1.6.7
                    //check if the advanced registration is enabled
                    $has_advanced_registration = $this->options->getOption('spam-advanced-registration');

                    if (!empty($has_advanced_registration) && $has_advanced_registration === 'Y') {
                        $res = $this->advance_spamguard->user_registration_check($user_login, $user_email);
                        if ($res) {
                            $errors->add( 'yoda_spam_guard_response', $this->return_message );
                        }
                    }

                }

            }
            return $errors;
        }


        /**
         *
         * append_widget
         * @since 1.3.2
         *
         *
         */
        public function append_widget()
        {
            if (!current_user_can('administrator'))
                return;

            if ($this->options->getOption('spam-guard-widget') === 'no') {
            ?>
                <div class="yodaSpamGuardWidget"><img src="<?php echo YODA_TOOLS_URL; ?>images/maennche-horizontal-full-color-150.jpg" alt="Maennche Logo"> <?php esc_html_e('Spam Guard','yoda-tools'); ?>
                    <div class="text">
                        <h4><?php esc_html_e('This widget is only visible to Administrators', 'yoda-tools');  ?></h4>
                        <p class="yodaspamMessage"><?php esc_html_e('if you want to act as spam bot for testing', 'yoda-tools'); ?></p>
                        <button id="yodaImaBot"><?php esc_html_e('Click here','yoda-tools'); ?></button>
                    </div>
                </div>
            <?php
            }
        }

    }
?>