<?php
	namespace yoda_tools\includes\core;
	use stdClass;

	/* Does User Belong Here */
	if (!defined('YODA_TOOLS_PATH')) {
		header('Status: 403 Forbidden');
		header('HTTP/1.1 403 Forbidden');
		exit();
	}

	class upgrader{
		//Setup template to make life easier
		private array $yoda_tools_plugin_detail_array_template;
		private $transient;


		/**
		 * 
		 * yoda_upgrader_endpoint
		 * @since 1.5.5
		 * 
		 */
		private string $yoda_upgrader_endpoint = 'https://plugin.maennche.com/wordpress/yoda-tools/';



		/**
		 * 
		 * yoda_preview_upgrader_endpoint
		 * @since 1.6.3
		 * 
		 */
		private string $yoda_preview_upgrader_endpoint = 'https://plugin.maennche.com/wordpress/yoda-tools/preview';

		public function __construct(){

			$this->yoda_tools_plugin_detail_array_template = array(
				'slug' => YODA_TOOLS_SLUG,
				'plugin' => 'yoda-tools/yoda_tools.php',
				'new_version' => YODA_TOOLS_VERSION,
				'url' => '',
				'package' => '',
				'icons' => array(),
				'banners' => array(),
				'tested' => '',
				'requires_php' => '',
				'compatibility' => new stdClass(),
			);

			//plugins api
			add_filter('plugins_api', array( $this, 'get_plugin_info' ), 20, 3);

			//pre_set_site_transient_update_plugins
			add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'pre_set_site_transient_update_plugins' ) );

			//upgrader_process_complete
			add_action( 'upgrader_process_complete', array( $this, 'upgrader_process_complete' ) , 10, 2 );

		}

		//Used for Plugin API Filter
		public function get_plugin_info($res,$action,$args)
		{

			$fnTmpInfo = $this->get_plugin_info_transient();
			// do nothing if this is not about getting plugin information
			if( 'plugin_information' !== $action ) {
				return false;
			}

			//added to avoid conflicts with other plugin
			if(empty($args) || YODA_TOOLS_SLUG !== $args->slug ) {
				return $res;
			} else {
				if(isset($args->slug)) {

					if(!empty($fnTmpInfo) && isset($fnTmpInfo['body'])) {
						$res = new stdClass();
						$fnTmpInfoArray = json_decode($fnTmpInfo['body']);

						$res->name = 'YODA Tools';
						$res->slug = $fnTmpInfoArray->slug;
						$res->version = $fnTmpInfoArray->new_version;
						$res->tested = $fnTmpInfoArray->tested;
						$res->requires = $fnTmpInfoArray->requires;
						$res->author = $fnTmpInfoArray->author;
						$res->author_profile = $fnTmpInfoArray->author_profile;
						$res->download_link = $fnTmpInfoArray->package;
						$res->trunk = $fnTmpInfoArray->package;
						$res->requires_php = $fnTmpInfoArray->requires_php;
						$res->last_updated = $fnTmpInfoArray->last_updated;

						if(isset($fnTmpInfoArray->sections)) {
							$res->sections = (array)$fnTmpInfoArray->sections;
						}

						if(isset($fnTmpInfoArray->banners)) {
							$res->banners = (array)$fnTmpInfoArray->banners;
						}

						return $res;
					}
				}
			}



			return false;
		}

		//Used for pre_set_site_transient_update_plugins Filter
		public function pre_set_site_transient_update_plugins( $transient ) {
			$fnTmpInfo = $this->get_plugin_info_transient();
			if(!empty($fnTmpInfo) && is_array($fnTmpInfo) && isset($fnTmpInfo['body'])) {

				//Compare versions
				$fnTmpInfoDecoded = json_decode($fnTmpInfo['body']);
				if( isset($fnTmpInfoDecoded->new_version) && version_compare( YODA_TOOLS_VERSION, $fnTmpInfoDecoded->new_version, '<' ) && version_compare($fnTmpInfoDecoded->requires, get_bloginfo('version'), '<' ) ) {

					// Update is available.
					$this->yoda_tools_plugin_detail_array_template["new_version"] = $fnTmpInfoDecoded->new_version;
					$this->yoda_tools_plugin_detail_array_template["url"] = $fnTmpInfoDecoded->url;
					$this->yoda_tools_plugin_detail_array_template["package"] = $fnTmpInfoDecoded->package;
					$this->yoda_tools_plugin_detail_array_template["tested"] = $fnTmpInfoDecoded->tested;
					$this->yoda_tools_plugin_detail_array_template["icons"] = $fnTmpInfoDecoded->icons;

					if (isset($transient->response)) {
						$transient->response[$fnTmpInfoDecoded->plugin] = (object)$this->yoda_tools_plugin_detail_array_template;
					}
					

				} else {
					// No update is available.

					// Adding the "mock" item to the `no_update` property is required
					// for the enable/disable auto-updates links to correctly appear in UI.
					if (isset($transient->no_update)) {
						$transient->no_update['yoda-tools/yoda_tools.php'] = (object)$this->yoda_tools_plugin_detail_array_template;
					}
					
				}

			}

			return $transient;
		}

		public function upgrader_process_complete( $upgrader_object, $options ): void {
			if ( $options['action'] == 'update' && $options['type'] === 'plugin' )  {
				$settings = settings::getInstance();
            	$settings->checkTableLogs();

				// just clean the cache when new plugin version is installed
				delete_transient( 'yoda_tools_plugin_update' );
				$this->transient = false;

				//delete flag and repeat process
				foreach($options['plugins'] as $each_plugin) {
					if ( $each_plugin === 'yoda-tools/yoda_tools.php') {
						delete_option('yoda_tools_get_preview_version');
					}
				}

			}
		}

		public function get_plugin_info_transient(){
			$this->sync_plugin_info_transient();
			return $this->transient;
		}

		public function sync_plugin_info_transient(): void {
			$this->transient = get_transient( 'yoda_tools_plugin_update');
			if(!$this->transient) {

				$endpoint = $this->yoda_upgrader_endpoint.'?site='.get_site_url();

				//check if they want preview
				$for_preview = get_option('yoda_tools_get_preview_version');
				if ($for_preview === 'Y') {
					$endpoint = $this->yoda_preview_upgrader_endpoint.'?site='.get_site_url();
				}

				$this->transient = wp_remote_get( $this->yoda_upgrader_endpoint.'?site='.get_site_url(), array(
						'timeout' => 10,
						'headers' => array(
							'Accept' => 'application/json'
						) )
				);

				if ( !is_wp_error( $this->transient ) && isset( $this->transient['response']['code'] ) && $this->transient['response']['code'] == 200 && !empty( $this->transient['body'] ) ) {
					//cache results
					set_transient( 'yoda_tools_plugin_update', $this->transient, 43200 ); // 12 hours cache
				}
			}
		}
	}