<?php
    namespace yoda_tools\includes\core;

    /* Does User Belong Here */

    use WP_Error;

    if (!defined('YODA_TOOLS_PATH')) {
        header('Status: 403 Forbidden');
        header('HTTP/1.1 403 Forbidden');
        exit();
    }


    class utilities
    {

        public function __construct(){ 
            add_action("admin_post_view", array($this, "view_log_item"));
            add_action("admin_post_transcript", array($this, "view_transcript_log_item"));
            add_action("admin_post_trash", array($this, "trash"));

            if ( is_admin() ) {
                add_action( 'wp_ajax_yodatools_resend_mail', array(
                        $this,
                        'resendEmail',
                ) );

                add_action( 'wp_ajax_yodatools_send_test_mail', array(
                        $this,
                        'sendTestEmail',
                ) );
            }

            add_action("wp_ajax_yoda_tools_db_tools_replace_text", array($this, "db_tools_replace_text"));
            add_action("wp_ajax_yoda_tools_run_cron", array($this, "yoda_tools_db_tools_run_cron"));
            //get locations
            add_action('wp_ajax_yoda_tools_get_script_location', [$this, 'script_locations']);

            add_action('admin_footer', [$this, 'load_modal']);

        }

        public static function register(): void { }

        // Plugin Settings
        public static function clean_string($fnKey, $fnSanitize = true)
        {
            if (!empty($_POST[$fnKey])) {
                $fnResponse = stripslashes_deep($_POST[$fnKey]);
                if ($fnSanitize) {
                    $fnResponse = sanitize_text_field($fnResponse);
                }
                return $fnResponse;
            }

            return '';
        }

        static function getServerName() {
            if ( ! empty( $_SERVER ['SERVER_NAME'] ) ) {
                $serverName = sanitize_text_field($_SERVER ['SERVER_NAME']);
            } else if ( ! empty( $_SERVER ['HTTP_HOST'] ) ) {
                $serverName = sanitize_text_field($_SERVER ['HTTP_HOST']);
            } else {
                $serverName = 'localhost.localdomain';
            }
            return $serverName;
        }

        /**
         * Update TEXT Old with New Text - AJAX Model
         *
         */
        function db_tools_replace_text(): void
        {
            global $wpdb;

            $old_text    = $_POST['old_text'];
            $new_text    = $_POST['new_text'];
            $update_url_nonce  = $_POST['update_url_nonce'];

            if (!wp_verify_nonce($update_url_nonce, "yoda_tools-update-url-nonce")) {
                die('Security check, Something went wrong!');
            } else {
                $old_text = sanitize_text_field( $old_text );
                $new_text = sanitize_text_field( $new_text );

                $posttable = "UPDATE ".$wpdb->prefix."posts SET `post_content` = REPLACE(`post_content`, '$old_text', '$new_text')
                WHERE `post_content` LIKE '%$old_text%'";
                $postablequery = $wpdb->query($posttable);

                $optionstable = "UPDATE ".$wpdb->prefix."options SET `option_value` = REPLACE(`option_value`, '$old_text', '$new_text')
                WHERE `option_value` LIKE '%$old_text%'";
                $optiontablequery = $wpdb->query($optionstable);

                $commenttable = "UPDATE ".$wpdb->prefix."comments SET `comment_content` = REPLACE(`comment_content`, '$old_text', '$new_text')
                WHERE `comment_content` LIKE '%$old_text%'";
                $commenttablequery = $wpdb->query($commenttable);

                $usermetatable = "UPDATE ".$wpdb->prefix."usermeta SET `meta_value` = REPLACE(`meta_value`, '$old_text', '$new_text')
                WHERE `meta_value` LIKE '%$old_text%'";
                $usermeatatablequery = $wpdb->query($usermetatable);

                $postmetaatable = "UPDATE ".$wpdb->prefix."postmeta SET `meta_value` = REPLACE(`meta_value`, '$old_text', '$new_text')
                WHERE `meta_value` LIKE '%$old_text%'";
                $postmeatatablequery = $wpdb->query($postmetaatable);

                $termmetaatable = "UPDATE ".$wpdb->prefix."termmeta SET `meta_value` = REPLACE(`meta_value`, '$old_text', '$new_text')
                WHERE `meta_value` LIKE '%$old_text%'";
                $termmeatatablequery = $wpdb->query($termmetaatable);

                $termtextable = "UPDATE ".$wpdb->prefix."term_taxonomy SET `description` = REPLACE(`description`, '$old_text', '$new_text')
                WHERE `description` LIKE '%$old_text%'";
                $termtexttablequery = $wpdb->query($termtextable);

                
                if( $postablequery || $optiontablequery || $commenttablequery || $usermeatatablequery || $postmeatatablequery || $termmeatatablequery || $termtexttablequery){
                    $responses = [$postablequery, $optiontablequery, $commenttablequery, $usermeatatablequery, $postmeatatablequery, $termmeatatablequery,$termtexttablequery];
                    $tem_affected_total = 0;
                    foreach ($responses as $res) {
                        if (is_bool($res)) {
                            continue;
                        } elseif (is_numeric($res)) {
                            $tem_affected_total += $res;
                        }
                    }
                    echo json_encode(['status' => 'success', 'affected' => $tem_affected_total]);
                }else{
                    echo json_encode(['status' => 'false']);
                }
            }

            exit();
        }


        function view_log_item() {
            // only do this for administrators
            if ( is_admin() ) {
                $postid = absint( $_REQUEST ['email'] );
                $mail_data = $this->get_mail( $postid );
                
                $meta_values = json_decode($mail_data[0]->email_data);
    
                // https://css-tricks.com/examples/hrs/
            echo '<html>
                    <head>
                        <style>
                            body {font-family: monospace;} hr {
                                border: 0;
                                border-bottom: 1px dashed #ccc;
                                background: #bbb;
                            }

                            .yoda_log_table {
                                padding: 5px;
                                background: #f2f2f2;
                                width: 100%;
                            }
                            .yoda_log_table tr th{
                                width: 16%;
                            }

                            .yoda_log_table tr td{
                                width: 80%;
                            }
                            .yoda_log_table tr th,
                            .yoda_log_table tr td{
                                text-align: left;
                                padding: 5px 10px;
                            }

                            .yoda_email_content {
                                padding: 15px;
                            }
                        </style>
                    </head>
                <body>
                <table class="yoda_log_table">';
                if ( ! empty( $meta_values->from_headers ) ) {
                echo '<tr>
                        <th>From:</th>
                        <td>'.esc_html( $meta_values->from_headers.' <'.$meta_values->from.'>').'</td>
                    </tr>';
                }
                // show the To header (it's optional)
                if ( ! empty( $meta_values->to ) ) {
                echo '<tr>
                        <th>To:</th>
                        <td>'.esc_html( $meta_values->to ).'</td>
                    </tr>';
                }
                // show the Cc header (it's optional)
                if ( ! empty( $meta_values->cc[0] ) ) {
                echo '<tr>
                        <th>Cc:</th>
                        <td>'.esc_html( $meta_values->cc[0] ).'</td>
                    </tr>';
                }
                // show the Bcc header (it's optional)
                if ( ! empty( $meta_values->bcc[0] ) ) {
                echo '<tr>
                        <th>Bcc:</th>
                        <td>'.esc_html( $meta_values->bcc[0] ).'</td>
                    </tr>';
                }
                
                // show the Reply-To header (it's optional)
                if ( ! empty( $meta_values->ReplyTo ) ) {

                    $reply_to_email = '';
                    foreach ($meta_values->ReplyTo as $reply) {
                        $reply_to_email = $reply[1].' <'.$reply[0].'>';
                    }
                echo '<tr>
                        <th>Reply-To:</th>
                        <td>'.esc_html( $reply_to_email ).'</td>
                    </tr>';
                }
                echo '<tr>
                        <th>Date:</th>
                        <td>'.$mail_data[0]->date_time_sent.'</td>
                    </tr>';
                echo '<tr>
                        <th>Subject:</th>
                        <td>'.esc_html( $meta_values->subject ).'</td>
                    </tr>';
                // The Transport UI is always there, in more recent versions that is
                if ( ! empty( $meta_values->delivery_uri ) ) {
                echo '<tr>
                        <th>Delivery-URI:</th>
                        <td>'.esc_html( $meta_values->delivery_uri ).'</td>
                    </tr>';
                }
                echo '</table>';
                echo '<hr/>';
                echo '<div class="yoda_email_content">';
                echo html_entity_decode($this->sanitize_message( $meta_values->message ));
                echo '</div>';
                echo '</body>';
            echo '</html>';
                
                die();
            }
        }

        function sanitize_message( $message ) {
            $allowed_tags = wp_kses_allowed_html( 'post' );
            $allowed_tags['style'] = array();

            return wp_kses( $message, $allowed_tags );
        }


        /**
         */
        function view_transcript_log_item() {
            // only do this for administrators
            if ( is_admin() ) {
                $postid = absint( $_REQUEST ['email'] );
                $mail_data = $this->get_mail( $postid );
                
                $meta_transcript = json_decode($mail_data[0]->transcript);
                $meta_values = json_decode($mail_data[0]->email_data);
                // https://css-tricks.com/examples/hrs/
            echo '<html>
                    <head>
                    <style>
                        body {font-family: monospace;} hr {
                            border: 0;
                            border-bottom: 1px dashed #ccc;
                            background: #bbb;
                        }

                        .yoda_log_notice {
                            padding: 20px; background: yellow;
                        }
                        .yoda_log_container {
                            padding: 20px;
                        }
                        .yoda_log_heading {
                            border-bottom: 1px dashed #000; border-top: 1px dashed #000; background: #f2f2f2; padding: 10px;
                        }
                        .yoda_log_failed {
                            color: red;
                        }
                    </style>
                    </head>
                <body>';
                echo '<div class="yoda_log_notice">'.__( 'This is the conversation between Yoda tools and the mail server. It can be useful for diagnosing problems. <b>DO NOT</b> post it on-line, it may contain your account password.', YODA_TOOLS_SLUG ).'</div>';
                echo '<hr/>';
                echo '<div class="yoda_log_container">';

                if ( ! empty( $meta_values ) ) {

                    if ($mail_data[0]->email_status != 'Email successfully delivered!') {
                        
                        //display error log
                        echo '<div class="yoda_log_heading"><b>EMAIL STATUS</b></div><p><div>';
                        echo '<h3><span class="yoda_log_failed">'.$meta_transcript->report->error.'</span></h3>';
                        if (isset($meta_transcript->report->human_message)) {
                            echo '<p>'.$meta_transcript->report->human_message.'</p>';
                        }
                        echo '<h4>Error Code: '.$meta_transcript->report->error_code.'</h4>';

                        foreach ($meta_transcript->report->details as $detail) {
                            echo '<div><h4>Source: '.$detail->function.'</h4></div>';
                            if (!is_array($detail->header) && !is_object($detail->header)) {
                                echo '<div class="yoda_log_heading"><b>HEADER</b></div><p><div>'.str_replace("\r\n",'<span style="height: 15px; display: block;"></span>',htmlspecialchars($detail->header)).'</p></div>';
                            }
                            echo '<div class="yoda_log_heading"><b>EMAIL CONTENT</b></div><p><div>';
                            if (!is_array($detail->content) && !is_object($detail->content)) {
                                echo $detail->content;
                            }
                        }

                        echo '<div class="yoda_log_heading"><b>RAW DATA</b></div><p><div>';
                        
                        if(!empty($meta_values->transcript)){
                            print_r($meta_values->transcript);
                            echo '<p>'.__( '--Raw message follows--', YODA_TOOLS_SLUG ).'</p>';
                        }
    
                        $meta_data["html"] = esc_html($meta_values->message);
                        echo '<p>'.__( '--Header--', YODA_TOOLS_SLUG ).'</p>';
                        print_r($meta_transcript);
                        echo '<p>'.__( '--Body message--', YODA_TOOLS_SLUG ).'</p>';
                        print_r($meta_data);

                    } else {
                        //display success log
                        $email_data = json_decode($mail_data[0]->email_data);
                        //print_r($email_data);
                        echo '<div class="yoda_log_heading"><b>EMAIL STATUS</b></div><p><div>';
                        echo '<h3><span style="color: green;">'.$mail_data[0]->email_status.'</span></h3>';
                        echo '<div><h4>Status: Success</h4></div>';
                        echo '<div class="yoda_log_heading"><b>HEADER</b></div><p><div></p></div>';
                        
                        echo '<p>Date: '.$mail_data[0]->date_time_sent.'</p>';
                        echo '<p>to: '.$email_data->to.'</p>';
                        $attachments_ = [];
                        if (!empty($email_data->attachments)) {
                            foreach ($email_data->attachments as $atts) {
                                if (is_array($atts)) {
                                    if (isset($atts[0])) {
                                        $attachments_[] = $atts[0];
                                    }
                                } else {
                                    $attachments_[] = $atts;
                                }
                            }   
                            echo '<p>Attachments: '.implode(',',$attachments_).'</p>';
                        }
                        echo '<p>From Headers: '.$email_data->from_headers.'</p>';
                        echo '<p>From: '.$email_data->from.'</p>';
                        echo '<p>cc: '.implode(',',$email_data->cc).'</p>';
                        echo '<p>bcc: '.implode(',',$email_data->bcc).'</p>';
                        $repto = '';
                        foreach($email_data->ReplyTo as $index => $replyto){
                            if (!empty($replyto)) {
                                $repto =  $replyto[0].' <'.$replyto[1].'>';
                            }
                        }
                        echo '<p>ReplyTo: '.$repto.'</p>';
                        echo '<div class="yoda_log_heading"><b>EMAIL CONTENT</b></div><p><div>';
                        echo $email_data->message;
                        echo '<div class="yoda_log_heading"><b>RAW DATA</b></div><p><div>';
                        
                        if(!empty($meta_values->transcript)){
                            print_r($meta_values->transcript);
                            echo '<p>'.__( '--Raw message follows--', YODA_TOOLS_SLUG ).'</p>';
                        }
    
                        $meta_data["html"] = esc_html($meta_values->message);
                        echo '<p>'.__( '--Header--', YODA_TOOLS_SLUG ).'</p>';
                        print_r($meta_transcript);
                        echo '<p>'.__( '--Body message--', YODA_TOOLS_SLUG ).'</p>';
                        print_r($meta_data);
                        
                    }
           
                } else {
                    echo __( 'n/a', YODA_TOOLS_SLUG );
                }
                echo '</div>';
                echo '</body></html>';
                die();
            }
        }

        /**
         */
        public function resendEmail() {
            check_admin_referer( 'yoda-tools-resend', 'security' );

            // get the email address of the recipient from the HTTP Request
            $postid = absint( $_POST['email'] );
			$mail_data = $this->get_mail( $postid );
        
            
            if ( ! empty( $postid ) ) {
                $meta_values = json_decode($mail_data[0]->email_data);

                if ( isset( $_POST['mail_to'] ) && ! empty( $_POST['mail_to'] ) ) {
                    $emails = explode( ',', $_POST['mail_to'] );
                    $to = array_map( 'sanitize_email', $emails );
                } else {
                    $to = $meta_values->to;
                }

                $headers = $this->get_test_headers();
				
				add_filter('wp_mail_content_type', function( $content_type ) {
					return 'text/html';
				});

                $success = wp_mail( $to, $meta_values->subject, $meta_values->message, $headers );

                // post-handling
                if ( $success ) {
                    // the message was sent successfully, generate an appropriate message for the user
                    $statusMessage = __( 'Email was successfully re-sent :)', YODA_TOOLS_SLUG );

                    // compose the JSON response for the caller
                    $response = array(
                        'message' => $statusMessage
                    );

                    // send the JSON response
                    wp_send_json_success( $response );
                } else {
                    $statusMessage = __('Email was not successfully re-sent - see email logs for more info.', YODA_TOOLS_SLUG);
                    $response = array(
                        'message' => $statusMessage
                    );

                    // send the JSON response
                    wp_send_json_error( $response );
                }
            } else {
                // compose the JSON response for the caller
                $response = array();
                // send the JSON response
                wp_send_json_error( $response );
            }
        }


        function sendTestEmail(){
            
            check_admin_referer( 'yoda-tools-test-email', 'security' );

            $email = sanitize_email($_POST['email']);

            $serverName = $this->getServerName();

            $subject = sprintf( _x( 'YODA tools SMTP Test (%s)', 'Test Email Subject', YODA_TOOLS_SLUG ), $serverName );

            $header = $this->get_test_headers();

            // create the message content
            $message = $this->createMessageContent();
            
            add_filter('wp_mail_content_type', function( $content_type ) {
                return 'text/html';
            });
            // send the message
            $success = wp_mail( $email, $subject, $message, $header );

            if ( $success ) {
                // the message was sent successfully, generate an appropriate message for the user
                $statusMessage = __( 'Test Email was successfully sent :)', YODA_TOOLS_SLUG );

                // compose the JSON response for the caller
                $response = array(
                    'message' => $statusMessage
                );

                // send the JSON response
                wp_send_json_success( $response );
            } else {
                $statusMessage = __('Test Email was not successfully sent - see email logs for more info.', YODA_TOOLS_SLUG);
                $response = array(
                    'message' => $statusMessage
                );

                // send the JSON response
                wp_send_json_error( $response );
            }
        }

        function createMessageContent(){
            ob_start();
            include(YODA_TOOLS_PATH.'views/test-email.php');
            return ob_get_clean();
        }

        function trash()
        {
            if(isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'yoda-tools-trash-nonce' ))
            {
                $id = sanitize_text_field($_REQUEST['id']);

                $msg = 'delete';
                global $wpdb;
                $log_table = $wpdb->prefix."yoda_tools_email_logs";
                $wpdb->query("DELETE FROM {$log_table} WHERE log_id = '$id'");

                $msg = 'Email log data removed successfully..';

                $redirect_url = admin_url("admin.php?page=yoda-tools-smtp&msg={$msg}");
                wp_redirect($redirect_url);
            }
        }
        
        /**
         * Run Cron - AJAX Model
         *
         */
        function yoda_tools_db_tools_run_cron(): void {
            $fnJob = null;

            if ( isset( $_POST['job'] ) ) {
                $fnJob = $this->clean_string('job', false);
            }
            $this->yoda_tools_force_schedule_single_event($fnJob);
            spawn_cron();

            echo "true";
        }

        /**
         * schedules a single job
         * @param string $hook Action hook to execute when the event is run.
         * @param array $args Optional. Array containing each separate argument to pass to the hook's callback function.
         * @return true|WP_Error True if event successfully scheduled. WP_Error on failure.
         */
        function yoda_tools_force_schedule_single_event(string $hook, array $args = array() )
        {
            $event = (object) array(
                'hook'      => $hook,
                'timestamp' => 1,
                'schedule'  => false,
                'args'      => $args,
            );
            $crons = (array) _get_cron_array();
            $key   = md5( serialize( $event->args ) );

            $crons[ $event->timestamp ][ $event->hook ][ $key ] = array(
                'schedule' => $event->schedule,
                'args'     => $event->args,
            );
            uksort( $crons, 'strnatcasecmp' );

            $result = _set_cron_array( $crons );

            // Not using the WP_Error from `_set_cron_array()` here so we can provide a more specific error message.
            if ( false === $result ) {
                return new WP_Error(
                    'could_not_add',
                    sprintf(
                    /* translators: 1: The name of the cron event. */
                        __( 'Failed to schedule the cron event %s.', 'yoda-tools' ),
                        $hook
                    )
                );
            }

            return true;
        }

        function get_mail($mailid){
            global $wpdb;
            $log_table = $wpdb->prefix."yoda_tools_email_logs";

            $query = "SELECT * FROM {$log_table} WHERE log_id = %d";

            $result = $wpdb->get_results($wpdb->prepare(
                    $query,
                    array(
                        $mailid
                    )
                )
            );

            return $result;
        }


        /**
         * 
         * 
         * ssl_encrypt
         * @since 1.3.2
         * @param array  'action' = string { encrypt | decrypt }
         *               'data' = string 
         * @return string
         *  
         * 
         */
        public static function ssl_encrypt($data = [])
        {
            if (empty($data)) {
                return;
            }
            $output = false;
            $encrypt_method = "AES-256-CBC";
            $sec = 'x5aCNlop+ebHPC+VmvX88GuQir+mSKK2etSNOX+inbs=';
            $secret_key = $sec;
            $sec_iv = 'I8NgYVqjHRvJWzvX3UvBfQ==';
            $secret_iv = $sec_iv;
            $key = hash('sha256', $secret_key);    
            $iv = substr(hash('sha256', $secret_iv), 0, 16);

            if ( $data['action'] == 'encrypt' ) {

                $output = openssl_encrypt($data['data'], $encrypt_method, $key, 0, $iv);

            } else if( $data['action'] == 'decrypt' ) {

                $output = openssl_decrypt($data['data'], $encrypt_method, $key, 0, $iv);
                
            }

            return $output;
        }


        /**
         * 
         * 
         * get_sender_email
         * this function will check if the FROM email is empty and making sure it's returning something valid
         * @since 1.3.2
         * 
         * 
         */
        public function get_sender_email( $from = null )
        {
            if (!$from || empty($from) || $from == '') {
                
                $settings = settings::getInstance();
                $alternative = $settings->getOption('envelope_sender');
                if ($alternative) {
                    $sender_name = $settings->getOption('envelope_sender_name');
                    if ($sender_name) {
                        return $sender_name.' <'.$alternative.'>';
                    } else {
                        return $alternative;
                    }
                   
                } else {
                    return 'noreply <no@noreply.com>';
                }
                
            } else {
                return $from;
            }
        }

        /**
         * 
         * get_test_headers
         * @since 1.3.2
         * this function will provide headers for test emails
         * 
         */
        public function get_test_headers()
        {
            return array(
                'Content-Type: multipart/alternative; charset=UTF-8',
                'MIME-Version: 1.0',
                'From: '.$this->get_sender_email()
            );
        }

        /**
         * 
         * 
         * check_php_script
         * @since 1.3.2
         * @since $content - string
         * @return bool
         * 
         * 
         */
        public static function check_php_script($content = null, $id = 0)
        {
            if (!$content || $content == '')
                return;

            
            //from field
            $from_field = (new self)->extract_unique_function($content);
            $count_unique = count(array_unique($from_field));
            $count = count($from_field);
            if ($count_unique != $count) {
                //return if the within the field has duplicate function
                return true;
            } else {
                //if no duplicate function inside field let's check from saved post
                $args = [
                    'posts_per_page' => -1,
                    'post_type' => 'yoda-php',
                    'post_status' => ['publish','draft']
                ];
                $phps = get_posts($args);
                
                if (empty($phps))
                    return;
                    
                //extract the functions from saved post
                foreach ($phps as $ph) {
                    $from_post[$ph->ID] = (new self)->extract_unique_function($ph->post_content);
                }

                //loop the extracted functions inside saved post
                foreach ($from_post as $pid => $fp) {
                    foreach($fp as $fn) {
                        //check the saved post extracted functions to the functions inside the field
                        if (in_array($fn, $from_field)) {
                            //if not current edit means it has duplicate functions from other saved post
                            if ($id != $pid)
                                return true;
                        }
                    }
                }

                return;
            }
            
        }

        /**
         * 
         * 
         * extract_unique_function
         * @since 1.3.2
         * 
         * 
         */
        public function extract_unique_function($content)
        {
            $content = esc_html($content);
            $data = explode('function', $content);
            $fncs = [];
            foreach ($data as $row) {
                if (strpos($row, "{") !== false) {
                    $x = explode("{", $row); 
                    $y = explode('(', $x[0]);
                    $fncs[] = trim($y[0]);
                }
            }

            return $fncs;
        }

        /**
         * 
         * 
         * prepare_locations
         * build and structure the script show to location
         * @since 1.6.5
         * 
         * 
         */
        public static function prepare_locations ()
        {
            //wordpress default post types
            $locations = [
                'general' => [
                    'entire_site'       => __('Entire Site', 'yoda-tools'),
                    'all_singular'      => __('All Singular', 'yoda-tools'),
                    'all_archives'      => __('All Archives', 'yoda-tools'),
                    'author_archives'   => __('Author Archives', 'yoda-tools'),
                    'date_archives'     => __('Date Archives', 'yoda-tools'),
                    'search_results'    => __('Search Results', 'yoda-tools'),
                    '404_page'          => __('404 Page', 'yoda-tools')
                ],
                'page'  => [
                    'page'  => __('Page', 'yoda-tools')
                ],
                'attachment' => [
                    'attachment'        => __('Media attachment', 'yoda-tools')
                ]
            ];
            
            //let's get the custom post types
            //NOTE: only public post types
            $custom_post_types = get_post_types(['public' => true, '_builtin' => false]);
            array_unshift($custom_post_types, 'post');

            
            if (!empty($custom_post_types)) {
                foreach ($custom_post_types as $post_type) {

                        //get post details                      
                        $taxonomies = get_taxonomies(['object_type' => [$post_type], 'public' => true], 'objects');
                        $post_type_object = get_post_type_object($post_type);

                        $post_type_locations = [];

                        if (!empty($taxonomies)) {
                            
                            $post_type_locations[$post_type] = $post_type_object->labels->name;
                            foreach ($taxonomies as $taxonomy) {
                                //exclude the post_format
                                if ($taxonomy->name === 'post_format') {
                                    continue;
                                }
                                $post_type_locations[$taxonomy->name.'_YODA-taxonomy'] = $post_type_object->labels->name.' '.$taxonomy->label;
                            }

                        } else {
                            $post_type_locations = [$post_type => $post_type_object->labels->name];
                        }

                        //let's add for archive
                        $locations[$post_type] = $post_type_locations;

                        $post_type_locations_archive = [];

                        if (!empty($taxonomies)) {

                            $post_type_locations_archive[$post_type.'_YODA-archive'] = $post_type_object->labels->name. ' Archive';
                            foreach ($taxonomies as $taxonomy) {
                                //exclude the post_format
                                if ($taxonomy->name === 'post_format') {
                                    continue;
                                }
                                $post_type_locations_archive[$taxonomy->name.'_YODA-archive-taxonomy'] = $taxonomy->label.' Archive';
                            }
                            
                        } else {
                            $post_type_locations_archive = [$post_type.'_YODA-archive' => $post_type_object->labels->name.' Archive'];
                        }

                        $locations[$post_type.'_YODA-archive'] = $post_type_locations_archive;

                        
                   
                } 
            }

            return $locations;

        }

        /**
         * 
         * 
         * script_locations
         * @since 1.6.5
         * 
         */
        public function script_locations()
        {
            $source = sanitize_text_field($_POST['source']);
            $nonce_key = 'yoda_script_extended_nonces';

            if ( ! wp_verify_nonce( $_POST['nonce'], $nonce_key ) ) {
                echo json_encode([
                    'status'    => __('failed','yoda-tools'),
                    'message'   => __('Invalid nonce, please contact developer', 'yoda-tools')
                ]);
                exit();
            }

            //sanitize the input data
            $location = (isset($_POST['location']))? sanitize_text_field($_POST['location']) : null; 

            $location_list = $this->get_locations($location);

            if ($location_list) {
                echo json_encode(
                    [
                        'status'    => 'success',
                        'response'  => $location_list
                    ]
                );
            }

            exit();

        }

        /**
         * 
         * 
         * get_location
         * @since 1.5.6
         * 
         * 
         */
        public function get_locations($location = null, $selected_value = null)
        {

            if (!$location)
                return;

        
            $locations = $this->prepare_locations();
                
            //check if not post_types
            foreach ($locations['general'] as $index => $general) {
                if ($index === $location) {
                    //not a post type
                    echo json_encode(
                        [
                            'status' => 'success',
                            'response'  => 'general'
                        ]
                    );
                }
            }
                
            //get get all taxonomies of post type
            //but first let's check if the post type is "post" the category and tag is already given.
            if (strpos($location, '_YODA-taxonomy') !== false || strpos($location, '_YODA-archive-taxonomy') !== false) {
                        
                //this solves the problem of unknown registered taxonomies and doesn't have to string search the keywords "category" and "tag" just to get the data
                //@since 1.5.7
                $delimeter = (strpos($location, '_YODA-archive-taxonomy') !== false)? '_YODA-archive-taxonomy' : '_YODA-taxonomy';
                
                $post_loc = explode($delimeter, $location);
                $tax_details = get_taxonomy($post_loc[0]);
                if ($tax_details) {
                    $post_terms = get_terms( ['taxonomy' => $post_loc[0], 'hide_empty' => false] );
                    return $this->term_options($post_terms, $tax_details->label, $selected_value);
                }

            }
            

            if (strpos($location, '_YODA-archive') !== false ) {
              
                //ayt is category archive
                echo json_encode(
                    [
                        'status' => 'success',
                        'response'  => 'general'
                    ]
                );
                exit();
                
            }

            //ayt the location is post
            $args = [
                'posts_per_page'    => -1,
                'post_type'         => $location,
                'post_status'       => 'publish',
                'order'             => 'ASC',
                'orderby'           => 'post_title'
            ];

            //get all post statuses
            $post_statuses = get_post_stati();
            if (!empty($post_statuses)) {
                $statiss = [];
                foreach ($post_statuses as $post_status) {
                    $statiss[] = $post_status;
                }
                $args['post_status'] = $statiss;
            }

            if ($location === 'attachment') {
                $args = [
                    'post_type'      => 'attachment',
                    'post_mime_type' => 'image',
                    'posts_per_page' => -1
                ];
            }

            $location_target = get_posts($args);

            if (!empty($location_target)) {
                //prepare the data
                //let's construct the html
                    
                $location_list = "<option value=\"all\">All {$locations[$location][$location]}</option>";
                foreach ($location_target as $loc) {
                    $selected_attr = ((int)$selected_value === $loc->ID)? "selected" : null;
                    $location_list .= "<option value=\"{$loc->ID}\" {$selected_attr}>{$loc->post_title}</option>";
                }

                return $location_list;
            } 
         
            return;
        }

        /**
         * 
         * 
         * term_options
         * @since 1.5.6
         * 
         */
        public function term_options ($post_terms, $all_label, $selected_value = null)
        {
            if (!empty($post_terms)) {  
                $location_list = "<option value=\"all\">All {$all_label}</option>";
                foreach ($post_terms as $p_term) {
                    $selected = ((int)$selected_value === $p_term->term_id)? "selected" : null;
                    $location_list .= "<option value=\"{$p_term->term_id}\" {$selected}>{$p_term->name}</option>";
                }
                return $location_list;
            }
            return;
        }
        
        
        /**
         * 
         * load_modal
         * @since 1.5.6
         * 
         * 
         */
        public function load_modal()
        {
            if (isset($_GET['page'])) {
                if ($_GET['page'] === 'yoda-tools-scripts') {
                    ?>
                    <div class="yoda-tools-modal">
                        <div class="yoda-tools-content"></div>
                    </div>
                    <?php
                } 
            }
        }

        /**
         * 
         * 
         * is_load_script
         * @since 1.5.6
         * 
         * 
         */
        public function is_load_script( $post_id, $current_post )
        {

            $locations = get_post_meta($post_id, 'yoda-tools-script-expanded-location', true);

            if (!$locations) {
                return true;
            }

            $do_load = 0;

            foreach ($locations as $loc) {
                if (is_array($loc) && count($loc) === 1) {

                    //general settings
                    switch ($loc[1]) {
                        case 'entire_site':
                                $do_load++;
                            break;
                        case 'all_singular':
                            if (is_single()) {
                                $do_load++;
                            }
                            break;
                        case 'all_archives':
                            if (is_archive()) {
                                $do_load++;
                            }
                            break;
                        case 'author_archives':
                            if (is_author()) {
                                $do_load++;
                            }
                            break;
                        case 'date_archives':
                            if (is_date()) {
                                $do_load++;
                            }
                            break;
                        case 'search_results':
                            if (is_search()) {
                                $do_load++;
                            }
                            break;
                        case '404_page':
                            if (is_404()) {
                                $do_load++;
                            }
                            break;
                    }
    
                    
                    if (strpos($loc[1], '_YODA-archive') !== false) {
                        //it's tag archive
                        $loc_name = str_replace('_YODA-archive','',$loc[1]);
                        if (is_archive() && !is_single() && $current_post->post_type === $loc_name) {

                            $queried_object = get_queried_object();
                            if ($queried_object instanceof \WP_Term) {
                                //do nothing
                            } else {
                                $do_load++;
                            }
                            
                        }
                        
                    }
                    
                    
                } elseif ( is_array($loc) && count($loc) > 1 ) {
                    //let's process the post_targets
                    $type = (isset($loc[1]))? $loc[1] : null;
                    $location = (isset($loc[2]))? $loc[2] : null;

                    if ($type && $location) {
                        //get current post ID

                        //let's check the type if post_type or categories or tag
                        //check if wp post types and taxonomies
                        switch ($type) {
                            case 'post':
                            case 'page':
                                $location_ID = ($location !== 'all')? (int)$location : $location;

                                if (!empty($current_post)) {
                                    if ( $current_post->ID === $location_ID) {
                                        $do_load++;
                                    }
                                }

                                if (isset($current_post->post_type)) {
                                    if ($location === 'all' && $current_post->post_type === $type) {
                                        $do_load++;
                                    }
                                }
                                
                               
                            break;
                        }
                 
                        //new solution for unknown registered taxonomies and doesn't have to string search if the slug has "category" or "tag"
                        if (strpos($type, '_YODA-taxonomy') !== false) {
                            $taxonomy = str_replace('_YODA-taxonomy','',$type);

                            if ($location === 'all') {

                                $terms = wp_get_post_terms($current_post->ID, $taxonomy);
                                $post_type = $current_post->post_type;
                                $taxonomy = get_taxonomy($taxonomy);

                                if ($terms && in_array($post_type, $taxonomy->object_type) && !is_home()) {
                                    $do_load++;
                                }

                            } else {
                                $has = $this->get_location_terms($taxonomy, $current_post->ID, $location);
                                if ($has && !is_archive()) {
                                    $do_load++;
                                }

                            }

                        } elseif (strpos($type, '_YODA-archive-taxonomy') !== false) {
                            //it's category archive
                            $term_id = get_queried_object_id();
                            if ($term_id === (int)$location) {
                                $do_load++;
                            }

                            if ($location === 'all') {
                                $get_slug = str_replace('_YODA-archive-taxonomy', '', $type);
                                $term = get_term($term_id, $get_slug);
                                if (is_archive() && $current_post->post_type === 'product' && $term) {
                                    $do_load++;
                                }
                            }
                            
                            
                        } else {    

                            $location_ID = ($location !== 'all')? (int)$location : $location;

                            if (!empty($current_post)) {
                                if ($current_post->ID === $location_ID) {
                                    $do_load++;  
                                }
                                if ($current_post->post_type === $type && $location === 'all') {
                                    $do_load++;
                                }
                            }

                           
                        }
                      

                    } else {
                        //fallback return true
                      
                        $do_load++;
                    }

                } 
            }

 

            if ($do_load > 0) {
                return true;
            } else {
                return false;
            }
            
            

        }


        /**
         * 
         * get_location_terms
         * @since 1.5.6
         * 
         */
        public function get_location_terms($taxonomy, $post_id, $location)
        {   
            $terms = wp_get_post_terms($post_id, $taxonomy);
            if (!is_wp_error($terms)) {
                foreach ($terms as $term) {
                    if ($term->term_id === (int)$location) {
                        return true;
                    }
                }
            }

            return;
        }

        /**
         * 
         * 
         * check_post_context
         * check individual post context
         * @since 1.6.7
         * @param int $post_id
         *        string $feed
         * 
         * @return string
         * 
         * 
         */
        public function check_post_context($post_id, $feed)
        {
            $obj_post = get_post($post_id);
            $content = strip_tags($obj_post->post_content);
            
            $context_found = false; //we need only one description from the terms

            if (strlen($content) <= 5) {
                //the content is not enough to produce relevant content let's check the website name and tag line.
                //let's check the categories descriptions
                //get all the terms available
                $taxonomies = get_object_taxonomies($obj_post->post_type);
                

                if (!empty($taxonomies)) {
                    foreach ($taxonomies as $taxonomy) {
                        $terms = wp_get_post_terms($post_id, $taxonomy);
                        if (!empty($terms) && !is_wp_error($terms)) {
                            foreach ($terms as $term) {
                                if ($term->description) {
                                    $feed = $feed." for the content about ".$term->name.' | '.$term->description;
                                    $context_found = true;
                                    break; //we need only one description from the terms
                                }
                            }
                        }
                        if ($context_found) {
                            break;
                        }
                    }
                }

                //no text found in categories
                //let's check the website's details
                if (!$context_found) {

                    $site_name  = get_bloginfo('name');
                    $tagline    = get_bloginfo('description');
                    
                    if ($tagline) {
                        $context_found = true;
                        $feed = $feed." for the content about ".$site_name.'|'.$tagline;
                    } else {
                        //if no description found then let's check the summary field
                        //seems we don't need anymore
                        $sumnmary = get_option('yoda-tools-ai-help');
                        if ($sumnmary) {
                            $context_found = true;
                            $feed = $feed." for the content about ".$sumnmary;
                        }  
                    }

                }
                
                
            } else {
                $context_found = true;
                $feed = $feed." for the content about ".$content;
            }
            
            if (!$context_found) {
                return;
            } else {
                return $feed;
            }
            
        }

	    /**
	     *
	     *
	     * to_timezone
	     * covert UCT/GMT time to set timezone in the settings
	     * @throws \Exception
	     * @since 1.6.10
	     *
	     *
	     */
        public function to_timezone($date = '', $format = 'Y/m/d h:i a', $nonGMT = true) {

            if ($date === '') {
                return '';
            }

            $timestamp = strtotime($date);

            $datetime = new \DateTime('@' . $timestamp);

            // Set to original UTC timezone
            $originalTimezone = new \DateTimeZone('UTC');
            $datetime->setTimezone($originalTimezone);

            // Get the site's timezone using wp_timezone()
            $targetTimezone = wp_timezone();

            $datetime->setTimezone($targetTimezone);

            return $datetime->format($format);
        }


        /**
         * 
         * 
         * load_part_script_locations
         * load script location UI
         * @since 1.6.12
         * 
         * 
         * 
         */
        public function load_part_script_locations($id = 0)
        {

            //varialbe seems unused but it is called inside the part file
            $loc_options = (new self)->prepare_locations(); 

            //reset data
            $yoda_saved_locations_set = [];
            
            $yoda_saved_locations = get_post_meta($id, 'yoda-tools-script-expanded-location', true);
            if (!$yoda_saved_locations) {
                //prepare the default
                $yoda_saved_locations_set[] = [
                    'type'      => 'post',
                    'target'    => $this->get_locations('post')
                ];
            } else {
                //prepare the options from saved data
                if (!empty($yoda_saved_locations)) {
                    foreach ($yoda_saved_locations as $saved_locations) {
                        if (count($saved_locations) > 1) {
                            //means has target
                            $yoda_saved_locations_set[] = [
                                'type'   => $saved_locations[1],
                                'target' => $this->get_locations($saved_locations[1], $saved_locations[2])
                            ];
                        } else {
                            //means has no target
                            $yoda_saved_locations_set[] = [
                                'type'      => $saved_locations[1],
                                'target'    => null
                            ];
                        }
                    }
                } else {
                    $yoda_saved_locations_set[] = $this->get_locations('post');
                }
                
            }

            //varialbe seems unused but it is called inside the part file
            $has_location_selected = null;
            $has_location_selected_toggle = null;
            if ($yoda_saved_locations) {
                $has_location_selected = "checked";
                $has_location_selected_toggle = "style=\"display: block;\"";
            }
            include YODA_TOOLS_PATH.'views/scripts-part-location.php';

        }


    }