<?php
    namespace yoda_tools\includes;
    class cron_tracking{
        public array $cron_array;
        public function __construct(){
            $this->cron_array = array();
            $this->prepare_cron_for_display();
        }

        public function get_last_cron_run(): string
        {
            if ( is_numeric( get_option( '_transient_doing_cron' ) ) && ( get_option( '_transient_doing_cron' ) > 0 ) ) {
                $last_cron_run = get_option( '_transient_doing_cron' ) . ' ago (' . round( get_option( '_transient_doing_cron' ) ) . ')';
            } else {
                $last_cron_run = 'Not running (completed)';
            }

            return $last_cron_run;
        }

        //Prepares Cron Info for display
        private function prepare_cron_for_display(): void {
            $cron = get_option( 'cron' );
            if ( ! isset( $cron_warnings ) ) {
                $cron_warnings = array();
            }

            $yoda_current_url = "//".$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];

            // Run cron if cron button trigger.
            if( isset($_REQUEST['run_cron']) && $_REQUEST['run_cron'] !== "" ){
                wp_schedule_event(time(), 'hourly', $_REQUEST['run_cron']);
                spawn_cron();
            }

            // Loop through each cron time to fill $this->cron_array for displaying later.
            foreach ( (array) $cron as $time => $cron_item ) {
                if (is_numeric($time)) {
                    // Loop through each schedule for this time.

                    foreach ((array)$cron_item as $hook_name => $event) {
                        foreach ( (array) $event as $item_name => $item ) {
                            $yoda_query_string = null;

                            if ( ! empty( $item['schedule'] ) ) {
                                // Recurring schedule.
                                $period = '';
                                $cron_interval = $this->prepare_cron_interval_for_display( $item['interval'] );
                                if ( false !== $cron_interval ) {
                                    $period .= '<span title="Interval tag: `' . $cron_interval[0] . '`.">' . $cron_interval[1] . '</span>';
                                } else {
                                    $period .= '<span title="Interval tag: `' . $item['schedule'] . '`.">' . $item['schedule'] . '</span>';
                                }
                            } else {
                                // One-time only cron.
                                $period = "";
                            }

                            // Determine interval.
                            if ( ! empty( $item['interval'] ) ) {
                                $interval = $item['interval'] . ' seconds';
                            } else {
                                $interval = 0;
                            }

                            // Determine arguments.
                            if ( ! empty( $item['args'] ) ) {
                                $arguments = '';
                                foreach ( $item['args'] as $args ) {
                                    $arguments_inner = array();
                                    $is_array        = false;
                                    if ( !is_array( $args ) ) {
                                        $arguments_inner[] = $args;
                                    } else {
                                        $is_array = true;
                                        foreach ( $args as $arg ) {
                                            if ( is_array( $arg ) ) {
                                                $arguments_inner[] = print_r( $arg, true );
                                            } else {
                                                $arguments_inner[] = $arg;
                                            }
                                        }
                                    }
                                    if ( true === $is_array ) {
                                        $arguments_inner = 'Array( ' . implode( ', ', $arguments_inner ) . ' )';
                                    } else {
                                        $arguments_inner = implode( ', ', $arguments_inner );
                                    }
                                    //$arguments .= '<textarea wrap="off">' . $arguments_inner . '</textarea>';
                                    $arguments = '<div class="yoda_inline">'.$arguments_inner.'</div>';
                                }
                            } else {

                                if( isset($_REQUEST['run_cron']) && $_REQUEST['run_cron'] !== "" ){
                                    $query_get = $_GET;
                                    $query_get['run_cron'] = $hook_name;
                                    $query_result = http_build_query($query_get);
                                    $yoda_query_string = $yoda_current_url.$query_result;
                                }else{
                                    $yoda_query_string = $yoda_current_url."&run_cron=".$hook_name;
                                }

                                $arguments = "none";
                            }

                            // If run time is in the past, note this.
                            $past_time = '';
                            if ( $time < time() ) {
                                $warning = 'WARNING: Next run time has passed. It should have run on ' . date('M j, Y g:i:s a', $time)  . '. Cron problem?';
                                $msg     = 'Something may be wrong with your WordPress cron such as a malfunctioning caching plugin or webhost problems.';

                                $tip = '(' . $msg . ')';

                                $past_time       = '<br><span style="color: red;"> ** ' . $warning . ' ** ' . $tip . '</span>';
                                $cron_warnings[] = $warning;
                            }

                            // Populate crons array for displaying later.
                            $this->cron_array[ $time . '|' . $hook_name . '|' . $item_name ] = array(
                                '<span title=\'Key: ' . $item_name . '\'>' . $hook_name . '</span>',
                                date('M j, Y g:i:s a', $time) . '<br><span class="description"> Timestamp: ' . $time . '</span>' . $past_time,
                                $period,
                                $interval,
                                $arguments,
                                '<div><a href="'.$yoda_query_string.'" class="yoda-tools-run-cron" data-id="'.$hook_name.'"><button class="button button-primary yoda-run-cron"> Run</button></a></div>'
                            );

                        }

                        unset( $item );
                        unset( $item_name );
                    }

                    unset( $event );
                    unset( $hook_name );
                }
            }

            unset( $cron_item );
            unset( $time );
        }

        //Prepares Cron Intervals Info for display
        private function prepare_cron_interval_for_display($interval)
        {
            $schedule_intervals = wp_get_schedules();
            foreach ( $schedule_intervals as $interval_tag => $schedule_interval ) {
                if ( $interval == $schedule_interval['interval'] ) {
                    return array( $interval_tag, $schedule_interval['display'] );
                }
            }

            return false;
        }
    }