/*
 * CodeMirror editor settings
 *
 * @package     My Custom Functions
 * @author      Arthur Gareginyan
 * @link        https://www.spacexchimp.com
 * @copyright   Copyright (c) 2016-2021 Space X-Chimp. All Rights Reserved.
 */


jQuery(document).ready(function($) {

    "use strict";

    var waiting;
    // Find textareas on page and replace them with the CodeMirror editor
    if(document.getElementById("textarea_script")){
        var editor_script = CodeMirror.fromTextArea(document.getElementById("textarea_script"), {
            lineNumbers: true,
            firstLineNumber: 1,
            matchBrackets: true,
            indentUnit: 4,
            mode: 'text/html',
            autoRefresh: true,
            styleActiveLine: true,
        });
        
        var slides = document.getElementsByClassName("script-code-textarea");
        for (var i = 0; i < slides.length; i++) {
            var item = slides.item(i);
        
            var editor_script_loop = CodeMirror.fromTextArea(item, {
                lineNumbers: true,
                firstLineNumber: 1,
                matchBrackets: true,
                indentUnit: 4,
                mode: 'text/html',
                autoRefresh: true,
                styleActiveLine: true,
            });
        }

    }else if( document.getElementById("code_php") ){
        var editor = CodeMirror.fromTextArea(document.getElementById("code_php"), {
            lineNumbers: true,
            firstLineNumber: 1,
            matchBrackets: true,
            indentUnit: 4,
            mode: 'text/x-php',
            autoRefresh: true,
            gutters: ["CodeMirror-lint-markers"],
            styleActiveLine: true,
            lint: {highlightLines: true},
        });
        
        editor.on("change", function() {
            clearTimeout(waiting);
            waiting = setTimeout(updateHints, 300);
        });

        setTimeout(updateHints, 100);

        function updateHints() {
            editor.operation(function(){
                for (var i = 0; i < widgets.length; ++i)
                editor.removeLineWidget(widgets[i]);
                widgets.length = 0;

                JSHINT(editor.getValue());
                
                if(JSHINT.errors.length > 0){
                    $(".php-code-snippets").attr("disabled","disabled");
                    $(".php-code-snippets").attr("type","button");

                    for (var i = 0; i < JSHINT.errors.length; ++i) {
                        var err = JSHINT.errors[i];
    
                        var msg = document.createElement("div");
                        var icon = msg.appendChild(document.createElement("span"));
                        icon.innerHTML = "❌ ";
                        icon.className = "lint-error-icon";
                        msg.appendChild(document.createTextNode(err.reason));
                        msg.className = "lint-error";
    
                        // editor.doc.setGutterMarker(err.line, "CodeMirror-lint-markers", makeMarker(msg));
                        widgets.push(editor.addLineWidget(err.line - 1, msg, {coverGutter: false, noHScroll: false}));
                    }
                }else{
                    $(".php-code-snippets").removeAttr("disabled","disabled");
                    $(".php-code-snippets").attr("type","submit");
                }

            });
            var info = editor.getScrollInfo();
            var after = editor.charCoords({line: editor.getCursor().line + 1, ch: 0}, "local").top;
            if (info.top + info.clientHeight < after)
                editor.scrollTo(null, after - info.clientHeight + 3);
        }

        var phploop = document.getElementsByClassName("php-snippets-code-textarea");
        
        for (var i = 0; i < phploop.length; i++) {
            var item = phploop.item(i);
            
            CodeMirror.fromTextArea(document.getElementById("code_php_"+i), {
                lineNumbers: true,
                firstLineNumber: 1,
                matchBrackets: true,
                indentUnit: 4,
                mode: 'text/x-php',
                autoRefresh: true,
                gutters: ["CodeMirror-lint-markers"],
                styleActiveLine: true,
                lint: {highlightLines: true}
            }).on("change", function(e) {
                var parentcode = $(e.getTextArea()).parent().parent();
                clearTimeout(waiting);
                waiting = setTimeout(updateHintsLoop(e, parentcode), 300);
            });
        }

        var widgets = []
        function updateHintsLoop(loop, parentcode) {
            loop.operation(function(){
                for (var i = 0; i < widgets.length; ++i)
                loop.removeLineWidget(widgets[i]);
                widgets.length = 0;

                JSHINT(loop.getValue());
                
                if(JSHINT.errors.length > 0){
                    parentcode.find('.yoda-field-group input[type="submit"]').attr("disabled","disabled");
                    parentcode.find('.yoda-field-group input[type="submit"]').attr("type","button");
                }else{
                    parentcode.find('.yoda-field-group input.button-primary').removeAttr("disabled","disabled");
                    parentcode.find('.yoda-field-group input.button-primary').attr("type","submit");
                }

                for (var i = 0; i < JSHINT.errors.length; ++i) {
                    var err = JSHINT.errors[i];

                    if (!err) continue;
                    var msg = document.createElement("div");
                    var icon = msg.appendChild(document.createElement("span"));
                    icon.innerHTML = "❌ ";
                    icon.className = "lint-error-icon";
                    msg.appendChild(document.createTextNode(err.reason));
                    msg.className = "lint-error";

                    // loop.doc.setGutterMarker(err.line, "CodeMirror-lint-markers", makeMarker(msg));
                    widgets.push(loop.addLineWidget(err.line - 1, msg, {coverGutter: false, noHScroll: false}));
                }
            });
            var info = loop.getScrollInfo();
            var after = loop.charCoords({line: loop.getCursor().line + 1, ch: 0}, "local").top;
            if (info.top + info.clientHeight < after)
                loop.scrollTo(null, after - info.clientHeight + 3);
        }

        function makeMarker(msg) {
            var marker = document.createElement("div");
            marker.innerHTML = `<div>❌</div>`;
            marker.setAttribute("title", msg.innerText);
            return marker;
        }

    } else if ( document.getElementById("textarea_css") ) {
        var editor_css = CodeMirror.fromTextArea(document.getElementById("textarea_css"), {
            lineNumbers: true,
            firstLineNumber: 1,
            matchBrackets: true,
            indentUnit: 4,
            mode: 'text/css',
            autoRefresh: true,
            gutters: ["CodeMirror-lint-markers"],
            styleActiveLine: true
        });

        var slides = document.getElementsByClassName("yoda-tools-custom-css-code");
        for (var i = 0; i < slides.length; i++) {
            var item = slides.item(i);
        
            var editor_css_loop = CodeMirror.fromTextArea(item, {
                lineNumbers: true,
                firstLineNumber: 1,
                matchBrackets: true,
                indentUnit: 4,
                mode: 'text/css',
                autoRefresh: true,
                styleActiveLine: true,
            });
        }

    }else if ( document.getElementById("textarea_meta") ) {
        CodeMirror.fromTextArea(document.getElementById("textarea_meta"), {
            lineNumbers: true,
            firstLineNumber: 1,
            matchBrackets: true,
            indentUnit: 4,
            mode: 'text/html',
            autoRefresh: true,
            gutters: ["CodeMirror-lint-markers"],
            styleActiveLine: true
        });

        var slides = document.getElementsByClassName("yoda-tools-meta-code");
        for (var i = 0; i < slides.length; i++) {
            var item = slides.item(i);
        
            CodeMirror.fromTextArea(item, {
                lineNumbers: true,
                firstLineNumber: 1,
                matchBrackets: true,
                indentUnit: 4,
                mode: 'text/html',
                autoRefresh: true,
                styleActiveLine: true,
            });
        }
    }
});
