<?php
    use yoda_tools\includes\core\settings;
    /* Does User Belong Here */

    if ( ! defined( 'YODA_TOOLS_VERSION' ) ) {
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    if(!current_user_can('administrator') ){
        die( "Insufficient Permissions" );
    }

    $this->options = settings::getInstance();
    $mode_scripts = $this->options->getOption('mode-scripts');
    $mode_server = $this->options->getOption('mode-server-tools');
    $mode_smtp_tools = $this->options->getOption('mode-smtp-tools');
    $mode_seo_tools = $this->options->getOption('mode-seo-tools');
    $mode_spamguard_tools = $this->options->getOption('mode-spam-guard');
    $mode_admin_notices = $this->options->getOption('mode-admin-notices');

    $tools = array(
        'Scripts' => array(
            'type' => 'scripts',
            'default' => ($mode_scripts) ? $mode_scripts : 'on',
            'description' => '<span>Maennche Tools allows you to easily add custom CSS, Javascript, and PHP code snippets without the need for FTP access</span> <br/> 
                <a href="?page=yoda-tools-scripts&tab=custom-css">CSS</a> | 
                <a href="?page=yoda-tools-scripts&tab=javascript">JavaScript</a> | 
                <a href="?page=yoda-tools-scripts&tab=meta">Meta</a> | 
                <a href="?page=yoda-tools-scripts&tab=php-snippets">PHP</a>'
        ),
        'Server Tools' => array(
            'type' => 'server-tools',
            'default' => ($mode_server) ? $mode_server : 'off',
            'description' => '<span>Need to know your website server settings, database details, or current cron schedules? Toggle this tool on and stay in the know</span> <br/>
                <a href="?page=yoda-tools-server-tools">Server</a> | 
                <a href="?page=yoda-tools-server-tools&tab=database">Database</a> | 
                <a href="?page=yoda-tools-server-tools&tab=cron">WordPress Cron Schedules</a>'
        ),
        'SMTP' => array(
            'type' => 'smtp-tools',
            'default' => ($mode_smtp_tools) ? $mode_smtp_tools : 'off',
            'description' => '<span>Monitor every email sent from your website with Maennche SMTP Tools</span> <br/>
                <a href="?page=yoda-tools-smtp">Logs</a> |
                <a href="?page=yoda-tools-smtp&tab=smtp_options">Configurations</a>'
        ),
        'SEO Tools' => array(
            'type' => 'seo-tools',
            'default' => ($mode_seo_tools) ? $mode_seo_tools : 'on',
            'description' => '<span>Monitor and Auto Generate On-Site SEO data (like Post Meta Description, Post Meta Titles, Post Excerpts, and more), Create/Edit your Robots.txt file and send advanced analytics tracking details to Google Analytics.</span> <br/>
            <a href="?page=yoda-tools-seo-tools">SEO Suggestions</a> | <a href="?page=yoda-tools-seo-tools&tab=tracking">Tracking</a> | 
            <a href="?page=yoda-tools-seo-tools&tab=robots">Robots.txt</a>'
        ),
        'Spam Guard' => array(
            'type' => 'spam-guard',
            'default' => ($mode_spamguard_tools) ? $mode_spamguard_tools : 'off',
            'description' => '<span>Block spam submission no Captcha or extra verification. Automatic anti spam protection.</span> <br/>
            <a href="?page=yoda-tools-spam-guard">Settings</a>'
        ),
        'Admin Notices' => array(
            'type' => 'admin-notices',
            'default' => ($mode_admin_notices) ? $mode_admin_notices : 'off',
            'description' => '<span>Manage admin notices and put them in one place.</span> <br/>
            <a href="?page=yoda-tools-admin-notices">Settings</a>'
        ),
        'Settings' => array(
            'type' => 'settings',
            'default' => 'on',
            'description' => '<span>Clear version transient and preview in-coming release.</span> <br/>
            <a href="?page=yoda-tools-settings">Maennche.com</a> | <a href="?page=yoda-tools-settings&tab=prev-version">Settings</a>'
        )
    );

    
?>

<h2 class="yoda-nav-title">Dashboard</h2>
<p>This options will allows you to enable functionalities listed below.</p>
<div class="yoda-tab-content">
    <?php
    foreach($tools as $data=>$value){
    ?>
        <div class="yoda-dashboard">
            <div class="yoda-dashboard-header">
                <div class="heading-wrap">
                    <h3><?php echo $data; ?></h3>
                    <p>
                        <?php
                            echo $value['description'];
                        ?>
                    </p>
                </div>
                <div class="yoda-dflex">
                    <?php if ($data !== 'Settings') { ?>
                    <label class="yoda-switch">
                        <input type="checkbox" class="yoda-tools-mode-status" data-mode="<?php echo $value['type']?>" name="yoda-tools-mode-status" <?php echo (isset($value['default']) && $value['default'] == 'on') ? "checked=checked" : "" ?>>
                        <span class="slider"></span>
                    </label>
                    <?php } ?>
                </div>
            </div>
        </div>
    <?php
    }
    ?>
</div><!-- end .yoda-tab-content -->
