<?php
    use yoda_tools\includes\core\utilities;

    /* Does User Belong Here */
    if ( ! defined( 'YODA_TOOLS_VERSION' ) ) {
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    if(!current_user_can('administrator') ){
        die( "Insufficient Permissions" );
    }
    
?>
<h2 class="yoda-nav-title">Email Logs</h2>
    <?php
    class EmailLogTable extends WP_List_Table
    {
        function __construct()
        {
            parent::__construct();
        }
        /**
         * @Method name  column_default
         * @Params       $cal,$column_name
         * @description  display static column name and corrosponding value
         */
        function column_default($cal, $column_name)
        {
            global $wpdb, $user_ID;
    
                switch ($column_name)
                {
                    case 'email_subject':
                        echo $cal['email_subject'];
                        break;
                    case 'sent_to':
                        echo $cal['sent_to'];
                        break;
                    case 'email_status':
                        echo $cal['email_status'];
                        break;
                    case 'date_time_sent':
                        echo $cal['date_time_sent'];
                        break;
                    default:
                        return $cal->$column_name;
                }
        }
        /**
         * @Method name  column_name
         * @Params       $cal
         * @description  display static column name and corrosponding value
         */
        function column_title($cal)
        {
            
            $content = "";
            $actions = array(
                'delete' => '<a onclick="javascript: return confirm(\'Please confirm you wish to delete this logs. This cannot be undone.\')" href="' .admin_url('admin.php?page=cal_id='.$cal['log_id']).'">'.esc_html( __('Delete', 'yoda-tools') ).'</a>'
            );
            $content .= $cal['email_subject'].$this->row_actions($actions);
            
            return $content;
        }
    
        /**
         * @Method name  column_cb
         * @Params       $cal
         * @description  display check box for all log data value
         */
        function column_cb($cal)
        {
            return '<input type="checkbox" name="check[]" value="'.$cal['log_id'].'" />';
        }
    
        /**
         * @Method name  get_columns
         * @description  display head tr for table
         */
        function get_columns()
        {
            $columns = array(
                'cb' => '<input type="checkbox"/>',
                'email_subject' => esc_html( __('Subject', 'yoda-tools') ),
                'sent_to' =>esc_html( __('Sent to', 'yoda-tools') ),
                'email_status' =>esc_html( __('Email Status', 'yoda-tools') ),
                'date_time_sent' =>esc_html( __('Date & time sent', 'yoda-tools') )
            );
            return $columns;
        }
    
        function process_bulk_action()
        {
            if(isset($_REQUEST['check']))
            {
                $check = esc_sql($_REQUEST['check']);
                if( 'trash' === $this->current_action() )
                {
                    $msg = 'delete';
                    global $wpdb;
                    $log_table = $wpdb->prefix."yoda_tools_email_logs";
                    foreach($check as $id){
                        $wpdb->query("DELETE FROM {$log_table} WHERE log_id = '$id'");
                    }

                    echo '<div class="notice notice-success is-dismissible"><p>Email log data removed successfully..</p></div>';
                }
            }
        }
    
        /**
         * @Method name  get_sortable_columns
         * @description  implement sorting on elments included in $sortable_columns array
         */
        function get_sortable_columns()
        {
            $sortable_columns = array(
                'email_subject' => array(
                    'email_subject',
                    false
                ),
                'date_time_sent' => array(
                    'date_time_sent',
                    false
                )
            );
            return $sortable_columns;
        }
        /**
         * @Method name  get_bulk_actions
         * @description  implement bulk action included in $actions array
         */
        function get_bulk_actions()
        {
            $actions = array(
                'trash' => esc_html( __('Trash', 'yoda-tools') )
            );
            return $actions;
        }
    
        /**
         * @Method name  prepare_items
         * @description  ready data to display
         */
        function prepare_items()
        {
            
            $this->process_bulk_action();

            global $wpdb, $user_ID;
            $log_table = $wpdb->prefix."yoda_tools_email_logs";
            $cal_per_page   = 10;
            $getsort = $this->sort_data("email_subject", "desc");
            
            $cal_query = "SELECT * FROM {$log_table} order by {$getsort}";
            $cal_data = $wpdb->get_results($cal_query, ARRAY_A);
            $columns   = $this->get_columns();
            $sortable  = $this->get_sortable_columns();
            $this->_column_headers = array(
                $columns,
                array(),
                $sortable
            );
    
            //pagging code starts from here
            $current_page = $this->get_pagenum();
            $total_cal = count((array) $cal_data);
            $cal_data = array_slice(
                $cal_data,(
                    ($current_page - 1) * $cal_per_page
                ),$cal_per_page
            );
            $this->items = $cal_data;
    
            $this->set_pagination_args(
                array(
                    'total_items'=>$total_cal,
                    'per_page'=> $cal_per_page,
                    'total_pages'=>ceil($total_cal/$cal_per_page)
                )
            );
            //pagging code ends from here
        }
    
        /**
         * @Method name  sort_data
         * @params $a $b
         * @description  sort product member data
         */
        public function sort_data($column, $order)
        {
            // Set defaults
            // If orderby is set, use this as the sort column
            if (!empty($_GET['orderby']))
            {
                $column = $_GET['orderby'];
            }
            // If order is set use this as the order
            if (!empty($_GET['order']))
            {
                $order = $_GET['order'];
            }
            
            $array = array($column, $order);
            $list = implode(" ", $array);
            
            return $list;
        }
    }
    ?>
    <form id="my-logs" name="my-logs" method="post" action="">
        <?php
            $EmailLogTable = new EmailLogTable();
            $EmailLogTable->prepare_items();
            $EmailLogTable->display();
            $EmailLogTable->process_bulk_action();
        ?>
    </form>