<?php
    use yoda_tools\includes\core\settings;

    $home_path = get_home_path();

    if ( ! is_writable( $home_path ) && ! empty( $_SERVER['DOCUMENT_ROOT'] ) ) {
        $home_path = $_SERVER['DOCUMENT_ROOT'] . DIRECTORY_SEPARATOR;
    }

    $robots_file    = $home_path . 'robots.txt';
    $action_url = network_admin_url( 'admin.php?page=yoda-tools-seo-tools&tab=robots' );

    if ( isset( $_POST['create_robots'] ) ) {
        if ( ! current_user_can( 'manage_options' ) ) {
            $die_msg = sprintf(
                /* translators: %s expands to robots.txt. */
                __( 'You cannot create a %s file.', 'yoda-tools' ),
                'robots.txt'
            );
            die( esc_html( $die_msg ) );
        }

        check_admin_referer( 'yoda_tools_create_robots' );

        ob_start();
        error_reporting( 0 );
        do_robots();
        $robots_content = ob_get_clean();

        $f = fopen( $robots_file, 'x' );
        fwrite( $f, $robots_content );

        //returns warning error
        //wp_safe_redirect( $action_url );
    }

    if ( isset( $_POST['submitrobots'] ) ) {

        if ( ! current_user_can( 'manage_options' ) ) {
            $die_msg = sprintf(
                /* translators: %s expands to robots.txt. */
                __( 'You cannot edit the %s file.', 'yoda-tools' ),
                'robots.txt'
            );
            die( esc_html( $die_msg ) );
        }

        check_admin_referer( 'yoda-tools-robotstxt' );

        if ( isset( $_POST['robotsnew'] ) && file_exists( $robots_file ) ) {
            $robotsnew = sanitize_textarea_field( wp_unslash( $_POST['robotsnew'] ) );
            if ( is_writable( $robots_file ) ) {
                $f = fopen( $robots_file, 'w+' );
                fwrite( $f, $robotsnew );
                fclose( $f );
                /* translators: %s expands to robots.txt. */
                $msg = sprintf(

                    __( 'Updated %s', 'yoda-tools' ),
                    'robots.txt'
                );
            }
        }

        //returns warning error
        //wp_safe_redirect( $action_url );
    }

    if ( isset( $msg ) && ! empty( $msg ) ) {
        echo '<div id="message" class="notice notice-success"><p>', esc_html( $msg ), '</p></div>';
    }
?>
<h2 class="yoda-nav-title">Robots.txt</h2>

<div class="yoda-server-tools no-border">
	<?php
        if ( ! file_exists( $robots_file ) ) {
            if ( is_writable( $home_path ) ) {
                echo '<form action="" method="post" id="createrobots_form">';
                wp_nonce_field( 'yoda_tools_create_robots', '_wpnonce', true, true );
                echo '<p>';
                printf(
                    /* translators: %s expands to robots.txt. */
                    esc_html__( 'You don\'t have a %s file, create one here:', 'yoda-tools' ),
                    'robots.txt'
                );
                echo '</p>';
                if ( current_user_can( 'manage_options' ) ) {
                    printf(
                        '<input type="submit" class="button" name="create_robots" value="%s">',
                        sprintf(
                            /* translators: %s expands to robots.txt. */
                            esc_attr__( 'Create %s file', 'yoda-tools' ),
                            'robots.txt'
                        )
                    );
                }else{
                    echo '<p>'.esc_attr__( 'You don\'t have permission to create robots.txt file. Please contact your administrator.', 'yoda-tools' ).'</p>';
                }
                echo '</form>';
            } else {
                echo '<p>';
                printf(
                    /* translators: %s expands to robots.txt. */
                    esc_html__( 'If you had a %s file and it was editable, you could edit it from here.', 'yoda-tools' ),
                    'robots.txt'
                );
                echo '</p>';
            }
        } else {
            $f = fopen( $robots_file, 'r' );

            $content = '';
            if ( filesize( $robots_file ) > 0 ) {
                $content = fread( $f, filesize( $robots_file ) );
            }

            if ( ! is_writable( $robots_file ) ) {
                echo '<p><em>';
                printf(
                    /* translators: %s expands to robots.txt. */
                    esc_html__( 'If your %s were writable, you could edit it from here.', 'yoda-tools' ),
                    'robots.txt'
                );
                echo '</em></p>';
                echo '<textarea class="large-text code" disabled="disabled" rows="15" name="robotsnew">', esc_textarea( $content ), '</textarea><br/>';
            } else {
                echo '<form action="" method="post" id="robotstxtform">';
                wp_nonce_field( 'yoda-tools-robotstxt', '_wpnonce', true, true );
                echo '<label for="robotsnew" class="yoast-inline-label">';
                printf(
                    /* translators: %s expands to robots.txt. */
                    esc_html__( 'Edit the content of your %s:', 'yoda-tools' ),
                    'robots.txt'
                );
                echo '</label>';
                echo '<textarea class="large-text code" rows="15" name="robotsnew" id="robotsnew">', esc_textarea( $content ), '</textarea><br/>';

                if ( current_user_can( 'manage_options' ) ) {
                    printf(
                        '<div class="submit"><input class="button" type="submit" name="submitrobots" value="%s" /></div>',
                        sprintf(
                            /* translators: %s expands to robots.txt. */
                            esc_attr__( 'Save changes to %s', 'yoda-tools' ),
                            'robots.txt'
                        )
                    );
                }else{
                    echo '<p>'.esc_attr__( 'You don\'t have permission to create robots.txt file. Please contact your administrator.', 'yoda-tools' ).'</p>';
                }

                echo '</form>';
            }
        }
	?>
</div>