<?php
	use yoda_tools\includes\core\utilities;
    use yoda_tools\includes\core\settings;

    /* Does User Belong Here */
	if ( ! defined( 'YODA_TOOLS_VERSION' ) ) {
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    $notifications = null;

    if(!current_user_can('administrator') ){
        die( "Insufficient Permissions" );
    }

    $this->options = settings::getInstance();
    $mode_scripts = $this->options->getOption('mode-scripts');

    //load the location UI
    $util = new utilities();

    //Custom CSS
    if (isset( $_POST['yoda-tools-custom-css-nonce'] ) && wp_verify_nonce( $_POST['yoda-tools-custom-css-nonce'], 'yoda-tools-custom-css-nonce' ) ) {
        $fnId = null;
        $fnTitle = null;
        $fnCode = null;
        $fnStep = null;
        $fnStatus = "publish"; //draft - I was drafted by default

        if ( isset( $_POST['yoda-tools-custom-css-id'] ) ) {
            $fnId = utilities::clean_string('yoda-tools-custom-css-id');
        }
        if ( isset( $_POST['yoda-tools-custom-css-title'] ) ) {
            $fnTitle = utilities::clean_string('yoda-tools-custom-css-title');
        }
        if ( isset( $_POST['yoda-tools-custom-css-code'] ) ) {
            $fnCode = utilities::clean_string('yoda-tools-custom-css-code',false);
        }
        if ( isset( $_POST['yoda-tools-custom-css-status'] ) ) {
            $fnStatus = utilities::clean_string('yoda-tools-custom-css-status',false);
        }
        if ( isset( $_POST['yoda-tools-custom-css-step'] ) ) {
            $fnStep = utilities::clean_string('yoda-tools-custom-css-step',false);
        }
        if ( isset( $_POST['yoda-tools-css-script-location'] ) ) {
            $fnLocation = utilities::clean_string('yoda-tools-css-script-location',false);
        }

        //clean data from locations
        $fnlocStatus = null;
        if ( isset( $_POST['yoda-load-specific-location-checkop'] ) ) {
            $fnlocStatus = utilities::clean_string('yoda-load-specific-location-checkop',false);
        }

        $fnLocSelected = null;

        if ($fnlocStatus === 'Y') {
            //clean_string didn't support multidimension array let's clean it in different way
            if (!empty($_POST['yoda-script-location-selected'])) {
                foreach ($_POST['yoda-script-location-selected'] as $index => $location_selected) {
                    foreach ($location_selected as $cindex => $selected) {
                        if (!empty($selected)) {
                            $_POST['yoda-script-location-selected'][$index][$cindex] = sanitize_text_field($selected);
                        }
                    }
                }
                //cleaned
                $fnLocSelected = $_POST['yoda-script-location-selected'];
            }
            
        } 

        
        if ( !empty($fnTitle) && !empty($fnCode)){
            GLOBAL $wpdb;
            switch($fnStep) {
                case '1';
                    if (!empty($fnId) && $fnId > 0) {
                        //Update
                        $data = array(
                            'ID' => $fnId,
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode,
                            'meta_input' => array(
                                'yoda-tools-script-location' => $fnLocation
                            )
                        );

                        if ($fnLocSelected && $fnlocStatus === 'Y') {
                            $data['meta_input']['yoda-tools-script-expanded-location'] = $fnLocSelected;
                        } else {
                            //remove
                            delete_post_meta($fnId, 'yoda-tools-script-expanded-location');
                        }
                        wp_update_post($data);
                        if (!is_wp_error($fnId)) {
                            //the post is valid
                            $notifications = "Updated Successfully";
                        }
                    } else {
                        //Insert
                        $post_id = wp_insert_post(array(
                            'post_type' => 'yoda-css',
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode,
                            'post_status' => $fnStatus
                        ));

                        if (!is_wp_error($post_id)) {
                            //the post is valid
                            update_post_meta($post_id, 'yoda-tools-script-location', $fnLocation);

                            if ($fnLocSelected && $fnlocStatus === 'Y') {
                                //update
                                update_post_meta($post_id, 'yoda-tools-script-expanded-location',  $fnLocSelected);
                            }
                            
                            $notifications = "Saved Successfully";
                        }
                    }

                    break;
                case '2':
                    //Delete Script
                    if (!empty($fnId) && $fnId > 0) {
                        wp_delete_post($fnId, true);
                        $notifications = "Deleted Successfully";
                    }
                    break;
            }
        }
    }

    //PHP Snippets
    if (isset( $_POST['yoda-tools-php-snippets-nonce'] ) && wp_verify_nonce( $_POST['yoda-tools-php-snippets-nonce'], 'yoda-tools-php-snippets-nonce' ) ) {
        $fnId = null;
        $fnTitle = null;
        $fnCode = null;
        $fnStep = null;
        $fnStatus = "publish";

        if ( isset( $_POST['yoda-tools-php-snippets-id'] ) ) {
            $fnId = utilities::clean_string('yoda-tools-php-snippets-id');
        }
        if ( isset( $_POST['yoda-tools-php-snippets-title'] ) ) {
            $fnTitle = utilities::clean_string('yoda-tools-php-snippets-title');
        }
        if ( isset( $_POST['yoda-tools-php-snippets-code'] ) ) {
            $fnCode = utilities::clean_string('yoda-tools-php-snippets-code',false);
        }
        if ( isset( $_POST['yoda-tools-php-snippets-status'] ) ) {
            $fnStatus = utilities::clean_string('yoda-tools-php-snippets-status',false);
        }
        if ( isset( $_POST['yoda-tools-php-snippets-step'] ) ) {
            $fnStep = utilities::clean_string('yoda-tools-php-snippets-step',false);
        }
        if ( !empty($fnTitle) && !empty($fnCode)){
            GLOBAL $wpdb;
            switch($fnStep) {
                case '1';

                    //let's check for function duplication
                    $has_function_duplicate = utilities::check_php_script($fnCode, $fnId);
                    if ($has_function_duplicate) {
                        $notifications = "Yoda tools detected duplicate function, please double check your script and uniquely name your functions.";
                    } else {

                        if (!empty($fnId) && $fnId > 0) {
                            //Update
                            $data = array(
                                'ID' => $fnId,
                                'post_title' => $fnTitle,
                                'post_content' => $fnCode
                            );
                            wp_update_post($data);
                            if (!is_wp_error($fnId)) {
                                //the post is valid
                                $notifications = "Updated Successfully";
                            }
                        } else {
                            //Insert
                            $post_id = wp_insert_post(array(
                                'post_type' => 'yoda-php',
                                'post_title' => $fnTitle,
                                'post_content' => $fnCode,
                                'post_status' => $fnStatus
                            ));
    
                            if (!is_wp_error($post_id)) {
                                //the post is valid
                                $notifications = "Saved Successfully";
                            }
                        }
                        

                    }

                    
                    break;
                case '2':
                    //Delete Script
                    if (!empty($fnId) && $fnId > 0) {
                        wp_delete_post($fnId, true);
                        $notifications = "Deleted Successfully";
                    }
                    break;
            }
        }
    }

    //Scripts
    if (isset( $_POST['yoda-tools-script-nonce'] ) && wp_verify_nonce( $_POST['yoda-tools-script-nonce'], 'yoda-tools-script-nonce' ) ) {
        $fnId = null;
        $fnTitle = null;
        $fnLocation = null;
        $fnCode = null;
        $fnStep = null;
        $fnStatus = "publish";

        if ( isset( $_POST['yoda-tools-script-id'] ) ) {
            $fnId = utilities::clean_string('yoda-tools-script-id');
        }
        if ( isset( $_POST['yoda-tools-script-title'] ) ) {
            $fnTitle = utilities::clean_string('yoda-tools-script-title');
        }
        if ( isset( $_POST['yoda-tools-script-location'] ) ) {
            $fnLocation = utilities::clean_string('yoda-tools-script-location');
        }
        if ( isset( $_POST['yoda-tools-script-code'] ) ) {
            $fnCode = utilities::clean_string('yoda-tools-script-code',false);
        }
        if ( isset( $_POST['yoda-tools-script-status'] ) ) {
            $fnStatus = utilities::clean_string('yoda-tools-script-status',false);
        }
        if ( isset( $_POST['yoda-tools-script-step'] ) ) {
            $fnStep = utilities::clean_string('yoda-tools-script-step',false);
        }

        //clean data from locations
        $fnlocStatus = null;
        if ( isset( $_POST['yoda-load-specific-location-checkop'] ) ) {
            $fnlocStatus = utilities::clean_string('yoda-load-specific-location-checkop',false);
        }

        $fnLocSelected = null;

        if ($fnlocStatus === 'Y') {
            //clean_string didn't support multidimension array let's clean it in different way
            if (!empty($_POST['yoda-script-location-selected'])) {
                foreach ($_POST['yoda-script-location-selected'] as $index => $location_selected) {
                    foreach ($location_selected as $cindex => $selected) {
                        if (!empty($selected)) {
                            $_POST['yoda-script-location-selected'][$index][$cindex] = sanitize_text_field($selected);
                        }
                    }
                }
                //cleaned
                $fnLocSelected = $_POST['yoda-script-location-selected'];
            }
            
        } 

        if ( !empty($fnTitle) && !empty($fnLocation) && !empty($fnCode)){
            GLOBAL $wpdb;
            switch($fnStep) {
                case '1';
                    if (!empty($fnId) && $fnId > 0) {
                        //Update
                        $data = array(
                            'ID' => $fnId,
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode,
                            'meta_input' => array(
                                'yoda-tools-script-location' => $fnLocation
                            )
                        );

                        if ($fnLocSelected && $fnlocStatus === 'Y') {
                            $data['meta_input']['yoda-tools-script-expanded-location'] = $fnLocSelected;
                        } else {
                            //remove
                            delete_post_meta($fnId, 'yoda-tools-script-expanded-location');
                        }

                        wp_update_post($data);
                        if (!is_wp_error($fnId)) {
                            //the post is valid
                            $notifications = "Updated Successfully";
                        }
                    } else {
                        //Insert
                        $post_id = wp_insert_post(array(
                            'post_type' => 'yoda-script',
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode,
                            'post_status' => $fnStatus
                        ));

                        if (!is_wp_error($post_id)) {
                            //the post is valid
                            update_post_meta($post_id, 'yoda-tools-script-location', $fnLocation);

                            if ($fnLocSelected && $fnlocStatus === 'Y') {
                                //update
                                update_post_meta($post_id, 'yoda-tools-script-expanded-location',  $fnLocSelected);
                            }

                            $notifications = "Saved Successfully";
                        }
                    }
                    break;
                case '2':
                    //Delete Script
                    if (!empty($fnId) && $fnId > 0) {
                        wp_delete_post($fnId, true);
                        $notifications = "Deleted Successfully";
                    }
                    break;
            }
        }
    }
    
    //Meta
    if (isset( $_POST['yoda-tools-meta-nonce'] ) && wp_verify_nonce( $_POST['yoda-tools-meta-nonce'], 'yoda-tools-meta-nonce' ) ) {
        $fnId = null;
        $fnTitle = null;
        $fnCode = null;
        $fnStep = null;
        $fnStatus = "publish";

        if ( isset( $_POST['yoda-tools-meta-id'] ) ) {
            $fnId = utilities::clean_string('yoda-tools-meta-id');
        }
        if ( isset( $_POST['yoda-tools-meta-title'] ) ) {
            $fnTitle = utilities::clean_string('yoda-tools-meta-title');
        }
        if ( isset( $_POST['yoda-tools-meta-code'] ) ) {
            $fnCode = utilities::clean_string('yoda-tools-meta-code',false);
        }
        if ( isset( $_POST['yoda-tools-meta-status'] ) ) {
            $fnStatus = utilities::clean_string('yoda-tools-meta-status',false);
        }
        if ( isset( $_POST['yoda-tools-meta-step'] ) ) {
            $fnStep = utilities::clean_string('yoda-tools-meta-step',false);
        }
        if ( !empty($fnTitle) && !empty($fnCode)){
            GLOBAL $wpdb;
            switch($fnStep) {
                case '1';
                    if (!empty($fnId) && $fnId > 0) {
                        //Update
                        $data = array(
                            'ID' => $fnId,
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode
                        );
                        wp_update_post($data);
                        if (!is_wp_error($fnId)) {
                            //the post is valid
                            $notifications = "Updated Successfully";
                        }
                    } else {
                        //Insert
                        $post_id = wp_insert_post(array(
                            'post_type' => 'yoda-meta',
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode,
                            'post_status' => $fnStatus
                        ));

                        if (!is_wp_error($post_id)) {
                            //the post is valid
                            $notifications = "Saved Successfully";
                        }
                    }
                    break;
                case '2':
                    //Delete Script
                    if (!empty($fnId) && $fnId > 0) {
                        wp_delete_post($fnId, true);
                        $notifications = "Deleted Successfully";
                    }
                    break;
            }
        }
    }

    //Get the active tab from the $_GET param
    $tab = isset($_GET['tab']) ? $_GET['tab'] : '';

?>

<div class="yoda-notice">
	<?php if(!empty($notifications)): 
            $success_messages = ['Saved Successfully','Deleted Successfully','Updated Successfully'];
            $class_yoda_noti = (in_array($notifications, $success_messages))? 'notice-success' : 'notice-error';
    ?>
	    <div class="notice <?php echo $class_yoda_noti; ?>"> 
			<p><strong><?php echo $notifications; ?></strong></p>
		</div>
	<?php endif; ?>
</div>

<h2 class="yoda-nav-title">Scripts</h2>

<?php
if($mode_scripts == 'on'){
?>
    <ul class="yoda-tab yoda-dflex">
        <li><a href="?page=yoda-tools-scripts&tab=custom-css" class="<?php echo ($tab == 'custom-css' || $tab == '' ? 'active' : ''); ?>">CSS</a></li>
        <li><a href="?page=yoda-tools-scripts&tab=javascript" class="<?php echo ($tab == 'javascript' ? 'active' : ''); ?>">Javascript</a></li>
        <li><a href="?page=yoda-tools-scripts&tab=meta" class="<?php echo ($tab == 'meta' ? 'active' : ''); ?>">Meta</a></li>
        <li><a href="?page=yoda-tools-scripts&tab=php-snippets" class="<?php echo ($tab == 'php-snippets' ? 'active' : ''); ?>">PHP</a></li>
    </ul>
<?php
}
?>

<div class="yoda-tab-content">
    <input type="hidden" id="yoda_tools_extend_location_nonce" value="<?php echo wp_create_nonce('yoda_script_extended_nonces'); ?>">
    <?php
        switch($tab):
            case 'javascript':
                ?>
                <?php
                if($mode_scripts == 'on'){
                ?>
            	<div class="yoda-scripts">

                    <div class="yoda-script-header">
                        <div class="yoda-script-header-text"><h2><?php _e( 'JavaScript', 'yoda' ); ?></h2></div>
                        <div class="yoda-script-header-button"><button class="button button-primary yoda-add-script-action" data-target="yoda-script-js"><span class="dashicons dashicons-plus"></span> New</button></div>
                    </div>

                    <div class="yoda-script-js-new-container yoda-script-toggle-container">
                        <div class="yoda-accordion">
                            <div class="yoda-accordion-header">
                                <h3>Add New Script</h3>
                                <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                            </div>
                            <div class="yoda-accordion-content">
                                <form method="post">
                                    <input type="hidden" name="yoda-tools-script-step" class="step" value="1"/>
                                    <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                    <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                    <div class="yoda-field-group">
                                        <input type="text" name="yoda-tools-script-title" value="<?php echo esc_attr( get_option('yoda-tools-script-title') ); ?>" placeholder="Title" class="yoda-full-width" required/>
                                    </div>
                                    <div class="yoda-field-group">
                                        <select name="yoda-tools-script-location" class="yoda-full-width">
                                            <option>Select Location</option>
                                            <option value="1">Header</option>
                                            <option value="2">Footer</option>
                                        </select>
                                    </div>

                                    <div class="yoda-field-group">
                                        <?php echo $util->load_part_script_locations(0); ?>                                                    
                                    </div>

                                    <div class="yoda-field-group">
                                        <textarea id="textarea_script" rows="5" cols="50" name="yoda-tools-script-code" data-type="text/javascript" placeholder="i.e. <script> alert('ok'); </script>"><?php echo esc_attr( get_option('yoda-tools-script-code') ); ?></textarea>
                                    </div>
                                    <div class="yoda-field-group">
                                        <?php wp_nonce_field( 'yoda-tools-script-nonce','yoda-tools-script-nonce' ); ?>
                                        <input type="submit" name="submit" class="button button-primary" value="Add New Script" /> <input type="button" class="yoda-script-cancel-new button button-secondary button-medium" data-target="yoda-script-js" value="Cancel">
                                    </div>
                                </form>
                            </div>
                        </div>
                    </div>

                    <div class="yoda-script-js-added-container yoda-script-toggle-container active">
            		<?php
                        $query = new WP_Query(array(
                            'post_type' => 'yoda-script',
                            'posts_per_page' => -1,
                            'post_status' => array('draft','publish')
                        ));
                        if ($query->have_posts() ){
                            while($query->have_posts()){
                                $query->the_post();
                                $fnStatus = get_post_status(get_the_ID());
                                $fnChecked = ($fnStatus === 'publish') ? 'checked' : '';
                                $yoda_tools_script_location = get_post_meta(get_the_ID(), 'yoda-tools-script-location', true);
                                ?>
                                    <div class="yoda-accordion">
                                        <div class="yoda-accordion-header">
                                            <h3>
                                                <?php echo ucwords(get_the_title()); ?> 
                                                <span class="type-label">
                                                    <?php
                                                    if ( (int)$yoda_tools_script_location === 2 ){
                                                        echo 'Footer';
                                                    }else{
                                                        echo 'Header';
                                                    }
                                                    ?>
                                                </span>
                                            </h3>
                                            <div class="yoda-dflex">
                                                <label class="yoda-switch">
                                                    <input type="checkbox" class="yoda-tools-on-status" data-type="script" name="yoda-tools-script-status" data-id="<?php echo get_the_ID(); ?>" <?php echo $fnChecked; ?>>
                                                    <span class="slider"></span>
                                                </label>
                                                <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                                            </div>
                                        </div>
                                        <div class="yoda-accordion-content yoda-dnone">
                                            <form method="post">
                                                <input type="hidden" name="yoda-tools-script-step" class="step" value="1"/>
                                                <input type="hidden" name="yoda-tools-script-id" value="<?php echo get_the_ID(); ?>"/>
                                                <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                                <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                                <div class="yoda-field-group">
                                                    <input type="text" name="yoda-tools-script-title" value="<?php echo get_the_title(); ?>" placeholder="Title" class="yoda-full-width" required/>
                                                </div>
                                                <div class="yoda-field-group">
                                                    <?php $yoda_tools_script_location = get_post_meta(get_the_ID(),'yoda-tools-script-location',true); ?>
                                                    <select name="yoda-tools-script-location" class="yoda-full-width">
                                                        <option>Select Location</option>
                                                        <option  <?php if ( (int)$yoda_tools_script_location === 1 ) echo 'selected' ; ?> value="1">Header</option>
                                                        <option  <?php if ( (int)$yoda_tools_script_location === 2 ) echo 'selected' ; ?> value="2">Footer</option>
                                                    </select>
                                                </div>
                                                
                                                <div class="yoda-field-group">
                                                    <?php echo $util->load_part_script_locations(get_the_ID()); ?>                                                    
                                                </div>


                                                <div class="yoda-field-group">
                                                    <textarea class="script-code-textarea" rows="5" cols="50" name="yoda-tools-script-code" data-type="text/javascript" placeholder="Script" required><?php echo get_the_content(); ?></textarea>
                                                </div>
                                                <div class="yoda-field-group">
                                                    <?php wp_nonce_field( 'yoda-tools-script-nonce','yoda-tools-script-nonce' ); ?>
                                                    <input type="submit" name="submit" class="button button-primary" value="Update Script">
                                                    <button type="button" class="button button-delete">Delete</button>
                                                </div>
                                            </form>
                                        </div>
                                    </div>
                                <?php
                            }
                            wp_reset_postdata();
                        } else {
                    ?>
                        <div>No added scripts</div>
                    <?php
                        }
                    ?>
                    </div>

                </div>
                <?php
                }else{
                ?>
                    <p>Make sure you enable option in <a href="?page=yoda-tools-dashboard">Dashboard</a> page</p>
                <?php
                }
                break;
                case 'meta':
                ?>
                <?php
                if($mode_scripts == 'on'){
                ?>
                <div class="yoda-scripts">

                    <div class="yoda-script-header">
                        <div class="yoda-script-header-text"><h2><?php _e( 'Meta', 'yoda' ); ?></h2></div>
                        <div class="yoda-script-header-button"><button class="button button-primary yoda-add-script-action" data-target="yoda-script-meta"><span class="dashicons dashicons-plus"></span> New</button></div>
                    </div>

                    <div class="yoda-script-meta-new-container yoda-script-toggle-container">
                        <div class="yoda-accordion">
                        <div class="yoda-accordion-header">
                            <h3>Add New Meta</h3>
                            <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                        </div>
                        <div class="yoda-accordion-content">
                            <form method="post">
                                <input type="hidden" name="yoda-tools-meta-step" class="step" value="1"/>
                                <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                <div class="yoda-field-group">
                                    <input type="text" name="yoda-tools-meta-title" value="<?php echo esc_attr( get_option('yoda-tools-meta-title') ); ?>" placeholder="Title" class="yoda-full-width" required/>
                                </div>
                                <div class="yoda-field-group">
                                    <textarea id="textarea_meta" rows="5" cols="50" name="yoda-tools-meta-code" data-type="text/javascript" placeholder="i.e. <meta name='robots' content='noindex, nofollow' />"></textarea>
                                </div>
                                <div class="yoda-field-group">
                                    <?php wp_nonce_field( 'yoda-tools-meta-nonce','yoda-tools-meta-nonce' ); ?>
                                    <input type="submit" name="submit" class="button button-primary" value="Add New Meta" /> <input type="button" class="yoda-script-cancel-new button button-secondary button-medium" data-target="yoda-script-meta" value="Cancel">
                                </div>
                            </form>
                        </div>
                        </div>
                    </div>

                    <div class="yoda-script-meta-added-container yoda-script-toggle-container active">
                    <?php
                        $query = new WP_Query(array(
                            'post_type' => 'yoda-meta',
                            'posts_per_page' => -1,
                            'post_status' => array('draft','publish')
                        ));
                        if ($query->have_posts() ){
                            while($query->have_posts()){
                                $query->the_post();
                                $fnStatus = get_post_status(get_the_ID());
                                $fnChecked = ($fnStatus === 'publish') ? 'checked' : '';
                                ?>
                                    <div class="yoda-accordion">
                                        <div class="yoda-accordion-header">
                                            <h3>
                                                <?php echo ucwords(get_the_title()); ?> 
                                            </h3>
                                            <div class="yoda-dflex">
                                                <label class="yoda-switch">
                                                        <input type="checkbox" class="yoda-tools-on-status" data-type="meta" name="yoda-tools-meta-status" data-id="<?php echo get_the_ID(); ?>" <?php echo $fnChecked; ?>>
                                                        <span class="slider"></span>
                                                </label>
                                                <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                                            </div>
                                        </div>
                                        <div class="yoda-accordion-content yoda-dnone">
                                            <form method="post">
                                                <input type="hidden" name="yoda-tools-meta-step" class="step" value="1"/>
                                                <input type="hidden" name="yoda-tools-meta-id" value="<?php echo get_the_ID(); ?>"/>
                                                <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                                <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                                <div class="yoda-field-group">
                                                    <input type="text" name="yoda-tools-meta-title" value="<?php echo get_the_title(); ?>" placeholder="Title" class="yoda-full-width" required/>
                                                </div>
                                                <div class="yoda-field-group">
                                                    <textarea class="yoda-tools-meta-code" rows="5" cols="50" name="yoda-tools-meta-code" data-type="text/javascript" placeholder="meta" required><?php echo get_the_content(); ?></textarea>
                                                </div>
                                                <div class="yoda-field-group">
                                                    <?php wp_nonce_field( 'yoda-tools-meta-nonce','yoda-tools-meta-nonce' ); ?>
                                                    <input type="submit" name="submit" class="button button-primary" value="Update Meta">
                                                    <button type="button" class="button button-delete">Delete</button>
                                                </div>
                                            </form>
                                        </div>
                                    </div>
                                <?php
                            }
                            wp_reset_postdata();
                        } else {
                    ?>
                        <div>No added scripts</div>
                    <?php
                        }
                    ?>
                    </div>

                </div>

                <?php
                }else{
                ?>
                    <p>Make sure you enable option in <a href="?page=yoda-tools-dashboard">Dashboard</a> page</p>
                <?php
                }
                break;
            case 'php-snippets':
                ?>
                <?php
                if($mode_scripts == 'on'){
                ?>
                
                <div class="yoda-scripts">

                <div class="yoda-script-header">
                        <div class="yoda-script-header-text"><h2><?php _e( 'Functions (PHP code)', 'yoda' ); ?></h2></div>
                        <div class="yoda-script-header-button"><button class="button button-primary yoda-add-script-action" data-target="yoda-script-php"><span class="dashicons dashicons-plus"></span> New</button></div>
                </div>
                
                <div class="yoda-script-php-new-container yoda-script-toggle-container">
                    <div class="yoda-accordion">
                        <div class="yoda-accordion-header">
                            <h3>Add New PHP Script</h3>
                            <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                        </div>
                        <div class="yoda-accordion-content">
                            <form method="post" enctype="multipart/form-data">
                                <input type="hidden" name="yoda-tools-php-snippets-step" class="step" value="1"/>
                                <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                <div class="yoda-field-group">
                                    <input type="text" name="yoda-tools-php-snippets-title" value="<?php echo esc_attr( get_option('yoda-tools-php-snippets-title') ); ?>" placeholder="Title" class="yoda-full-width" required/>
                                </div>
                                <?php 
                                $code_here = "i.e.

    add_shortcode('subscribe', 'subscribe_link');
    function subscribe_link(){
            return ".esc_attr("'Follow us on <a rel=\"nofollow\" href=\"https://twitter.com/MyVCMO\">Twitter</a>'").";
    }";
                                ?>
                                <div class="postbox">
                                    <div class="inside">
                                        <textarea id="code_php"
                                            class="code_php"
                                            name="yoda-tools-php-snippets-code" 
                                            id="yoda-tools-php-snippets" 
                                            placeholder="<?php echo $code_here; ?>" ></textarea>
                                    </div>
                                </div>

                                <!-- SUBMIT -->
                                <?php wp_nonce_field( 'yoda-tools-php-snippets-nonce','yoda-tools-php-snippets-nonce' ); ?>
                                <input type="submit" name="submit" id="submit" class="button button-primary php-code-snippets" value="Add new PHP" > <input type="button" class="yoda-script-cancel-new button button-secondary button-medium" data-target="yoda-script-php" value="Cancel">
                                <!-- END SUBMIT -->

                            </form>
                        </div>
                    </div>

                </div>

                <div class="yoda-script-php-added-container yoda-script-toggle-container active">
                    <?php
                        $query = new WP_Query(array(
                            'post_type' => 'yoda-php',
                            'posts_per_page' => -1,
                            'post_status' => array('draft','publish')
                        ));
                        if ($query->have_posts() ){
                            $cnt = 0;
                            while($query->have_posts()){
                                $query->the_post();
                                $fnStatus = get_post_status(get_the_ID());
                                $fnChecked = ($fnStatus === 'publish') ? 'checked' : '';
                                ?>
                                    <div class="yoda-accordion">
                                        <div class="yoda-accordion-header">
                                            <h3>
                                                <?php echo ucwords(get_the_title()); ?> 
                                            </h3>
                                            <div class="yoda-dflex">
                                                <label class="yoda-switch">
                                                        <input type="checkbox" class="yoda-tools-on-status" data-type="php" name="yoda-tools-php-snippets-status" data-id="<?php echo get_the_ID(); ?>" <?php echo $fnChecked; ?>>
                                                        <span class="slider"></span>
                                                </label>
                                                <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                                            </div>
                                        </div>
                                        <div class="yoda-accordion-content yoda-dnone">
                                            <form method="post">
                                                <input type="hidden" name="yoda-tools-php-snippets-step" class="step" value="1"/>
                                                <input type="hidden" name="yoda-tools-php-snippets-id" value="<?php echo get_the_ID(); ?>"/>
                                                <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                                <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                                <div class="yoda-field-group">
                                                    <input type="text" name="yoda-tools-php-snippets-title" value="<?php echo get_the_title(); ?>" placeholder="Title" class="yoda-full-width" required/>
                                                </div>
                                                <div class="yoda-field-group">
                                                    <textarea class="php-snippets-code-textarea" id="code_php_<?php echo $cnt++;?>" rows="5" cols="50" name="yoda-tools-php-snippets-code" placeholder="<?php echo $code_here; ?>" required><?php echo get_the_content(); ?></textarea>
                                                </div>
                                                <div class="yoda-field-group">
                                                    <?php wp_nonce_field( 'yoda-tools-php-snippets-nonce','yoda-tools-php-snippets-nonce' ); ?>
                                                    <input type="submit" name="submit" class="button button-primary" value="Update code">
                                                    <button type="button" class="button button-delete">Delete</button>
                                                </div>
                                            </form>
                                        </div>
                                    </div>
                                <?php
                            }
                            wp_reset_postdata();
                        } else {
                    ?>
                        <div>No added scripts</div>
                    <?php
                        }
                    ?>
                </div>
                </div>
                <?php
                }else{
                ?>
                    <p>Make sure you enable option in <a href="?page=yoda-tools-dashboard">Dashboard</a> page</p>
                <?php
                }
            break;
            default:
            	?>
                <div class="yoda-scripts">

                    <div class="yoda-script-header">
                        <div class="yoda-script-header-text"><h2><?php _e( 'Custom CSS', 'yoda' ); ?></h2></div>
                        <div class="yoda-script-header-button"><button class="button button-primary yoda-add-script-action" data-target="yoda-script-css"><span class="dashicons dashicons-plus"></span> New</button></div>
                    </div>
                    
                    <div class="yoda-script-css-new-container yoda-script-toggle-container">
                    <div class="yoda-accordion">
                    <div class="yoda-accordion-header">
                        <h3>Add New CSS</h3>
                        <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                    </div>
                    <div class="yoda-accordion-content">
                        <form method="post" class="yoda-full-width">
                            <input type="hidden" name="yoda-tools-custom-css-step" class="step" value="1"/>
                            <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                            <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                            <div class="yoda-field-group">
                                <?php
                                if($mode_scripts == 'on'){
                                ?>
                                    <input type="text" name="yoda-tools-custom-css-title" value="<?php echo esc_attr( get_option('yoda-tools-custom-css-title') ); ?>" placeholder="Title" class="yoda-full-width" required/>
                                <?php 
                                }else{
                                ?>
                                    <input type="text" name="yoda-tools-custom-css-title" value="" placeholder="Title" class="yoda-full-width" readonly/>
                                <?php
                                }
                                ?>

                            </div>
                            <div class="yoda-field-group">
	            					<?php $yoda_tools_css_script_location = get_option('yoda-tools-script-location'); ?>
	                                <select name="yoda-tools-css-script-location" class="yoda-full-width">
	                                    <option>Select Location</option>
	                                    <option  <?php if ( $yoda_tools_css_script_location == 1 ) echo 'selected' ; ?> value="1">Header</option>
	                                    <option  <?php if ( $yoda_tools_css_script_location == 2 ) echo 'selected' ; ?> value="2">Footer</option>
	                                </select>
	            			</div>
                            
                            
                            <?php
                                //pass the saved ID of locations leave empty or 0 for new script
                                $util->load_part_script_locations(0);
                            ?>

                                
                            <div class="yoda-field-group">
                                <?php
                                if($mode_scripts == 'on'){
                                ?>
                                    <textarea rows="20" id="textarea_css" name="yoda-tools-custom-css-code" placeholder="i.e. body{ background-color:red; }" onkeydown="if(event.keyCode===9){var v=this.value,s=this.selectionStart,e=this.selectionEnd;this.value=v.substring(0, s)+'\t'+v.substring(e);this.selectionStart=this.selectionEnd=s+1;return false;}" ></textarea>
                                <?php 
                                }else{
                                ?>
                                    <textarea rows="20" id="yoda-tools-custom-css-code" readonly placeholder="i.e. body{ background-color:red; }" onkeydown="if(event.keyCode===9){var v=this.value,s=this.selectionStart,e=this.selectionEnd;this.value=v.substring(0, s)+'\t'+v.substring(e);this.selectionStart=this.selectionEnd=s+1;return false;}" ></textarea>
                                <?php
                                }
                                ?>
                            </div>
                            <?php
                            if($mode_scripts == 'on'){
                            ?>
                            <div class="yoda-field-group">
                                <?php wp_nonce_field( 'yoda-tools-custom-css-nonce','yoda-tools-custom-css-nonce' ); ?>
                                <input type="submit" name="submit" class="button button-primary" value="Add new CSS" > <input type="button" class="yoda-script-cancel-new button button-secondary button-medium" data-target="yoda-script-css" value="Cancel">
                            </div>
                            <?php } ?>
                        </form>
                    </div>
                    </div>
                    </div>
                
                    <div class="yoda-script-css-added-container yoda-script-toggle-container active">
            <?php
                if($mode_scripts == 'on'){
                    $query = new WP_Query(array(
                        'post_type' => 'yoda-css',
                        'posts_per_page' => -1,
                        'post_status' => array('draft','publish')
                    ));
                    if ($query->have_posts() ){
                        while($query->have_posts()){
                            $query->the_post();
                            $fnStatus = get_post_status(get_the_ID());
                            $fnChecked = ($fnStatus === 'publish') ? 'checked' : '';
                            ?>
                                <div class="yoda-accordion">
                                    <div class="yoda-accordion-header">
                                        <h3>
                                            <?php echo ucwords(get_the_title()); ?> 
                                        </h3>
                                        <div class="yoda-dflex">
                                            <label class="yoda-switch">
                                                    <input type="checkbox" class="yoda-tools-on-status" data-type="css" name="yoda-tools-custom-css-status" data-id="<?php echo get_the_ID(); ?>" <?php echo $fnChecked; ?>>
                                                    <span class="slider"></span>
                                            </label>
                                            <span class="dashicons dashicons-arrow-down-alt2" aria-hidden="true"></span>
                                        </div>
                                    </div>
                                    <div class="yoda-accordion-content yoda-dnone">
                                        <form method="post">
                                            <input type="hidden" name="yoda-tools-custom-css-step" class="step" value="1"/>
                                            <input type="hidden" name="yoda-tools-custom-css-id" value="<?php echo get_the_ID(); ?>"/>
                                            <?php settings_fields( 'yoda-tools-settings-group' ); ?>
                                            <?php do_settings_sections( 'yoda-tools-settings-group' ); ?>
                                            <div class="yoda-field-group">
                                                <input type="text" name="yoda-tools-custom-css-title" value="<?php echo get_the_title(); ?>" placeholder="Title" class="yoda-full-width" required/>
                                            </div>
                                            <div class="yoda-field-group">
                                                    <?php $yoda_tools_css_script_location = get_post_meta(get_the_ID(),'yoda-tools-script-location',true); ?>
                                                    <select name="yoda-tools-css-script-location" class="yoda-full-width">
                                                        <option>Select Location</option>
                                                        <option  <?php if ( $yoda_tools_css_script_location == 1 ) echo 'selected' ; ?> value="1">Header</option>
                                                        <option  <?php if ( $yoda_tools_css_script_location == 2 ) echo 'selected' ; ?> value="2">Footer</option>
                                                    </select>
                                                </div>


                                                <?php
                                                    //pass the saved ID of locations leave empty or 0 for new script
                                                    $util->load_part_script_locations(get_the_ID());
                                                ?>


                                            <div class="yoda-field-group">
                                                <textarea class="yoda-tools-custom-css-code" rows="5" cols="50" name="yoda-tools-custom-css-code" data-type="text/javascript" placeholder="css" required><?php echo get_the_content(); ?></textarea>
                                            </div>
                                            <div class="yoda-field-group">
                                                <?php wp_nonce_field( 'yoda-tools-custom-css-nonce','yoda-tools-custom-css-nonce' ); ?>
                                                <input type="submit" name="submit" class="button button-primary" value="Update code">
                                                <button type="button" class="button button-delete">Delete</button>
                                            </div>
                                        </form>
                                    </div>
                                </div>
                            <?php
                        }
                        wp_reset_postdata();
                    } else {
                ?>
                    <div>No added scripts</div>
                <?php
                    }
                }

                ?>
                    </div>
                </div>
                <?php
            break;
                
        endswitch;
    ?>
    
</div><!-- end .yoda-tab-content -->