<?php
    use yoda_tools\includes\core\utilities;
    use yoda_tools\includes\core\settings;
    use yoda_tools\includes\core\register;
    use yoda_tools\includes\core\seoSuggestionTableList;

    /* Does User Belong Here */
    if ( ! defined( 'YODA_TOOLS_VERSION' ) ) {
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    if(!current_user_can('administrator') ){
        die( "Insufficient Permissions" );
    }
    
    $this->options = settings::getInstance();
    $mode_seo_tools = $this->options->getOption('mode-seo-tools');


    //Scripts
    if (isset( $_POST['yoda-tools-script-nonce'] ) && wp_verify_nonce( $_POST['yoda-tools-script-nonce'], 'yoda-tools-script-nonce' ) ) {
        $fnId = null;
        $fnTitle = null;
        $fnLocation = null;
        $fnCode = null;
        $fnStep = null;

        if ( isset( $_POST['yoda-tools-script-id'] ) ) {
            $fnId = utilities::clean_string('yoda-tools-script-id');
        }
        if ( isset( $_POST['yoda-tools-script-title'] ) ) {
            $fnTitle = utilities::clean_string('yoda-tools-script-title');
        }
        if ( isset( $_POST['yoda-tools-script-location'] ) ) {
            $fnLocation = utilities::clean_string('yoda-tools-script-location');
        }
        if ( isset( $_POST['yoda-tools-script-code'] ) ) {
            $fnCode = utilities::clean_string('yoda-tools-script-code',false);
        }
        if ( isset( $_POST['yoda-tools-script-step'] ) ) {
            $fnStep = utilities::clean_string('yoda-tools-script-step',false);
        }

        if ( !empty($fnTitle) && !empty($fnLocation) && !empty($fnCode)){
            GLOBAL $wpdb;
            switch($fnStep) {
                case '1';
                    if (!empty($fnId) && $fnId > 0) {
                        //Update
                        $data = array(
                            'ID' => $fnId,
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode,
                            'meta_input' => array(
                                'yoda-tools-script-location' => $fnLocation
                            )
                        );
                        wp_update_post($data);
                        if (!is_wp_error($fnId)) {
                            //the post is valid
                            $notifications = "Updated Successfully";
                        }
                    } else {
                        //Insert
                        $post_id = wp_insert_post(array(
                            'post_status' => 'publish',
                            'post_type' => 'yoda-script',
                            'post_title' => $fnTitle,
                            'post_content' => $fnCode
                        ));

                        if (!is_wp_error($post_id)) {
                            //the post is valid
                            update_post_meta($post_id, 'yoda-tools-script-location', $fnLocation);
                            $notifications = "Saved Successfully";
                        }
                    }
                    break;
                case '2':
                    //Delete Script
                    if (!empty($fnId) && $fnId > 0) {
                        wp_delete_post($fnId, true);
                        $notifications = "Deleted Successfully";
                    }
                    break;
            }
        }
    }

    //Get the active tab from the $_GET param
    //get the API
    $connected = $this->options->getOption('apikey');
    $tab = isset($_GET['tab']) ? $_GET['tab'] : 'default';
?>
<h2 class="yoda-nav-title">SEO Tools</h2>

<ul class="yoda-tab yoda-dflex">
    <?php
    if($mode_seo_tools == 'on'){
    ?>

        <li><a href="admin.php?page=yoda-tools-seo-tools" class="<?php echo ($tab == 'default' ? 'active' : ''); ?>">SEO Suggestions</a></li>
        <li><a href="?page=yoda-tools-seo-tools&tab=tracking" class="<?php echo ($tab == 'tracking' ? 'active' : ''); ?>">Tracking</a></li>
        <li><a href="?page=yoda-tools-seo-tools&tab=robots" class="<?php echo ($tab == 'robots' ? 'active' : ''); ?>">Robots.txt</a></li>
    <?php
    }
    ?>
</ul>

<div class="yoda-notice">
    <?php settings_errors(); ?>
    <?php if (get_transient('yoda_seo_transaction_success')) { ?>
        <div class="notice notice-success"><p><?php echo get_transient('yoda_seo_transaction_success'); ?></p></div>
    <?php delete_transient('yoda_seo_transaction_success'); } ?>
</div>

<div class="yoda-tab-content yoda-settings">
    <?php
        switch($tab):
            case 'default':
                //placeholder if the user is not connected to Maennche.com
                ?>


                <?php if (!$connected) {  ?>
                    <div class="yoda_seo_suggest_connect">
                        <table class="yoda_auth_details" width="80%">
                            <tr>
                                <td><img src="<?php echo YODA_TOOLS_URL ?>images/yoda-cartoon-welcome-03.png" width="350" alt=""></td>
                                <td>
                                    <h1><?php esc_html_e('WELCOME!', 'yoda-tools'); ?></h1>
                                    <h3><?php esc_html_e('Unlock the Power of AI for Your SEO Content', 'yoda-tools'); ?></h3>
                                    <p><?php esc_html_e('Connect now to embark on this exciting adventure. Let\'s create content that captivates, converts, and conquers the digital realm together.','yoda-tools'); ?></p>
                                    <p><a href="admin.php?page=yoda-tools-settings" class="button button-primary button-large"><?php esc_html_e('Connect to Maennche.com', 'yoda-tools'); ?></a></p>
                                </td>
                            </tr>
                        </table>
                    </div>    
                <?php } else { ?>

                    <div>   
                        <?php 
                            $table = new seoSuggestionTableList();
                            $table->prepare_items();  
                        ?>
                        <div class="yoda_seo_suggestions">
                            <form id="posts-filter" method="get">
                                <?php $table->display(); ?>
                            </form>
                        </div>
                    </div>

                    <!-- SEO suggestion -->
                    <div class="yoda_tools_seo_global_modal" id="yoda_open_suggestion">
                        <div class="yoda_tools_seo_global_modal_content">
                            <div class="header_content">
                                <div><img src="<?php echo YODA_TOOLS_URL; ?>images/favicon.png" alt=""> <?php esc_html_e('Maennche SEO', 'yoda-tools'); ?></div>
                                <div><span class="dashicons dashicons-no"></span></div>
                            </div>
                            <div class="content_content" id="yoda_seo_suggestion_area">
                                <div class="yoda_seo_suggestion_area_wrap">
                                    <h3><?php esc_html_e('YODA\'s Suggestion:', 'yoda-tools'); ?></h3>
                                    <div class="yoda_seo_suggestion_result"></div>
                                </div>
                                <div class="yoda_seo_asking_sug_loading"><span class="dashicons dashicons-update"></span> <span><?php esc_html_e('Asking for suggestion...','yoda-tools'); ?></span></div>
                            </div>
                            <div class="footer_content">
                                <button class="yoda_seo_modal_button yoda_seo_modal_button_secondary yoda_seo_save_no_generate" id="yoda_seo_use_generated_suggestion"><span class="dashicons dashicons-saved"></span> <?php esc_attr_e('Use Generated Suggestion','yoda-tools'); ?></button>
                            </div>
                        </div>
                    </div>


                    <div class="yoda_tools_seo_global_modal" id="yoda_seo_open_attention">
                        <div class="yoda_tools_seo_global_modal_content">
                            <div class="header_content">
                                <div><img src="<?php echo YODA_TOOLS_URL; ?>images/favicon.png" alt=""> <?php esc_html_e('Maennche SEO | Requires Attention', 'yoda-tools'); ?></div>
                                <div><span class="dashicons dashicons-no"></span></div>
                            </div>
                            <div class="content_content">
                                <p><?php esc_html_e('YODA tools require some context to effectively generate accurate SEO meta suggestions. Kindly share a brief overview of the page to help us provide you with the best recommendations.', 'yoda-tools'); ?></p>
                                <p><label>
                                    Overview
                                    <textarea id="yoda_seo_action_overview" rows="8" placeholder="e.g. My portfolios about visual arts."></textarea>
                                </label>

                                </p>
                            </div>
                            <div class="footer_content">
                                <button id="yoda_seo_save_no_generate" class="yoda_seo_modal_button yoda_seo_modal_button_secondary yoda_seo_save_no_generate" data-generate="no" disabled><span class="dashicons dashicons-saved"></span> <?php esc_attr_e('Save overview without generating SEO','yoda-tools'); ?></button>&nbsp;<button id="yoda_seo_save_generate" class="yoda_seo_modal_button yoda_seo_save_no_generate" data-generate="yes" disabled><span class="dashicons dashicons-text"></span> <?php esc_html_e('Save overview and generate SEO', 'yoda-tools'); ?></button>
                            </div>
                        </div>
                    </div>

                <?php } ?>

                <?php
                break;
            case 'robots': 
                    require_once( YODA_TOOLS_PATH . '/views/robots-txt.php' );
                break;
            default:
                ?>
                    <form action='options.php' method='post' >
                        <?php 
                        settings_fields( 'yoda-tools_page' );
                        do_settings_sections( 'yoda_tools_settings' );
                        ?>
                        <table class="form-table">
                            <tr>
                                <th></th>
                                <td class="submit">
                                    <?php 
                                    if($mode_seo_tools == 'on'){
                                        submit_button(); 
                                    }
                                    ?>
                                </td>
                            </tr>
                        </table>
                    </form>
                <?php
                break;
                
        endswitch;
    ?>
</div><!-- end .yoda-tab-content -->