<?php
	use yoda_tools\includes\core\settings;
	use yoda_tools\includes\cron_tracking;

    /* Does User Belong Here */
	if ( ! defined( 'YODA_TOOLS_VERSION' ) ) {
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    global $wpdb;

    //Get the active tab from the $_GET param
    $tab = isset($_GET['tab']) ? $_GET['tab'] : '';
    
    wp_enqueue_script('admin_js');
    wp_localize_script( 'admin_js', 'yoda_tools_ajax', array( 'ajax_url' => admin_url('admin-ajax.php')) );

	$this->options = settings::getInstance();
    $mode_server = $this->options->getOption('mode-server-tools');
	
?>

<h2 class="yoda-nav-title">Server Tools <span class="use-with-caution">(USE WITH CAUTION)</span></h2>
<?php
if($mode_server == 'on'){
?>
<ul class="yoda-tab yoda-dflex">
    <li><a href="?page=yoda-tools-server-tools" class="<?php echo ($tab == '' ? 'active' : ''); ?>">Server</a></li>
    <li><a href="?page=yoda-tools-server-tools&tab=database" class="<?php echo ($tab == 'database' ? 'active' : ''); ?>">Database</a></li>
    <li><a href="?page=yoda-tools-server-tools&tab=cron" class="<?php echo ($tab == 'cron' ? 'active' : ''); ?>">WordPress Schedules (Cron)</a></li>
</ul>
<?php
}
?>

<div class="yoda-notice">
    <?php settings_errors(); ?>
</div>

<div class="yoda-tab-content yoda-server-tools no-border">
	<?php
        switch($tab):
        	case 'cron':
				if($mode_server == 'on'){
        		?>
        			<h2>WP Cron Schedule</h2>
        			<?php
        				$date_time_formatted = current_time('M j, Y g:i:s a');
                        $timestamp           = current_time( 'timestamp' );

                        $thisCron = NEW cron_tracking();
                        if(!empty($thisCron->get_last_cron_run()) && is_numeric($thisCron->get_last_cron_run())){
                            $lastCronRun = date('M j, Y g:i:s a', $thisCron->get_last_cron_run());
                        }else{
                            if(!empty($thisCron->get_last_cron_run())){
                                $lastCronRun = $thisCron->get_last_cron_run();
                            } else {
                                $lastCronRun = "N/A";
                            }
                        }

						$schedule_intervals = wp_get_schedules();

					?>
						<!-- Filter area -->
						<div class="yoda_tools_cron_filter">
							<ul>
								<li><?php esc_html_e('Filter', 'yoda-tools'); ?>&nbsp;&nbsp;&nbsp;<input type="text" id="yoda_tools_cron_filter_name_field" placeholder="<?php esc_html_e('Search cron name','yoda-tools'); ?>"></li>
								<li>
									<select id="yoda_tools_filter_cron_period">
										<option value=""><?php esc_html_e('Period','yoda-tools'); ?></option>
										<?php 
										if (!empty($schedule_intervals)) { 
											foreach ($schedule_intervals as $index => $period) {
										?>
												<option value="<?php echo esc_attr($period['display']); ?>"><?php echo esc_html($period['display']); ?></option>
										<?php
											}	
										} 
										?>
										
									</select>
								</li>
							</ul>
							
						</div>
					<?php
						
                        printf( '<div class="txt-center current-time"><b>Current Time:</b></a> %s</div>', esc_html( $date_time_formatted ) );
					
					?> 
						<!-- table wrapper of schedules <HEAD>-->
						<div class="yoda_cron_schedule_table">
					<?php
                        // Display CRON table.
                        $tmpTableDataRender = NEW settings();
                        $tmpTableDataRender->table_data_render(
                            $thisCron->cron_array,
                            array(
                                'action'                  => '',
                                'columns'                 => array(
                                    'Scheduled Events',
                                    'Next Run',
                                    'Period',
                                    'Interval',
                                    'Arguments',
                                    'Run'
                                ),
                                'css'                     => 'width: 100%;',
                                'hover_actions'           => array(),
                                'bulk_actions'            => array(),
                                'hover_action_column_key' => '0',
                            )
                        );

					?> 
						<!-- table wrapper of schedules <TAIL>-->
						</div> 
					<?php 

						
                        // Display time intervals table.
                        $pretty_intervals   = array();
                        

                        foreach ( $schedule_intervals as $interval_tag => $schedule_interval ) {
                            $pretty_intervals[ $schedule_interval['interval'] ][] = array(
                                $schedule_interval['display'],
                                $interval_tag,
                                $schedule_interval['interval'],
                            );
                        }
                        ksort( $pretty_intervals );
                        $pretty_intervals = array_reverse( $pretty_intervals );

                        // Now flatten the array.
                        $display_intervals = array();
                        foreach ( $pretty_intervals as $interval ) {
                            if ( is_array( $interval[0] ) ) {
                                // Array of arrays of schedule details.
                                foreach ( $interval as $interval2 ) {
                                    $display_intervals[] = $interval2;
                                }
                            } else {
                                // Just array of schedule details.
                                $display_intervals[] = $interval;
                            }
                        }

                        $tmpTableDataRender->table_data_render(
                            $display_intervals, // Array of cron items set in code section above.
                            array(
                                'columns' => array(
                                    'Schedule Periods',
                                    'Tag',
                                    'Interval',
                                ),
                                'css'     => 'width: 100%;',
                            )
                        );
				}else{
				?>
					<p>Make sure you enable option in <a href="?page=yoda-tools-dashboard">Dashboard</a> page</p>
				<?php
				}
        		break;

        	case 'database':
				if($mode_server == 'on'){
        		?>
        			<h2>Database Information</h2>

        			<?php
                        global $wpdb;

                        $tables_query = "
                            SELECT TABLE_NAME AS `table`,
                              TABLE_ROWS as `rows`,
                              ROUND((DATA_LENGTH + INDEX_LENGTH) / 1024 ) AS `size`
                            FROM information_schema.TABLES
                            WHERE TABLE_SCHEMA = '".$wpdb->dbname."'
                            ORDER BY TABLE_NAME ASC 
                        ";
                        $rows = $wpdb->get_results($tables_query);
                    ?>
                    <div class="yoda-database-info">
                    	<table class="widefat striped">
	                        <thead>
	                        <tr class="thead">
	                            <th>Table Name</th>
	                            <th># of Rows</th>
	                            <th>Size (KB)</th>
	                        </tr>
	                        </thead>
	                        <tbody>
	                            <?php foreach( $rows as $row ): ?>
	                                <tr>
	                                    <td class="entry-row"><?php echo $row->table?></td>
	                                    <td class="entry-row"><?php echo $row->rows?></td>
	                                    <td class="entry-row"><?php echo $row->size?></td>
	                                </tr>
	                            <?php endforeach; ?>
	                        </tbody>
	                    </table>
                    </div>
        		<?php
				}else{
				?>
					<p>Make sure you enable option in <a href="?page=yoda-tools-dashboard">Dashboard</a> page</p>
				<?php
				}
        		break;
        	default:
        		?>
        			<h2>Server Information</h2>
		            <div class="yoda-server-info">
		                <h1>Wordpress</h1>
		                <table>
		                    <tbody>
		                        <tr>
		                            <td class="e">Wordpress URL</td>
		                            <td class="v"><?php echo get_bloginfo('url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Version</td>
		                            <td class="v"><?php echo get_bloginfo('version', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Administrator Email</td>
		                            <td class="v"><?php echo get_bloginfo('admin_email', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Page Encoding</td>
		                            <td class="v"><?php echo get_bloginfo('charset', 'display')?></td>
		                        </tr>

		                        <tr>
		                            <td class="e">Content Type</td>
		                            <td class="v"><?php echo get_bloginfo('html_type', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Text Direction</td>
		                            <td class="v"><?php echo get_bloginfo('text_direction', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Language</td>
		                            <td class="v"><?php echo get_bloginfo('language', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Stylesheet URL</td>
		                            <td class="v"><?php echo get_bloginfo('stylesheet_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Stylesheet Folder</td>
		                            <td class="v"><?php echo get_bloginfo('stylesheet_directory', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Template URL</td>
		                            <td class="v"><?php echo get_bloginfo('template_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Ping back URL</td>
		                            <td class="v"><?php echo get_bloginfo('pingback_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Atom Feed URL</td>
		                            <td class="v"><?php echo get_bloginfo('atom_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">RDF/RSS 1.0 feed URL</td>
		                            <td class="v"><?php echo get_bloginfo('rdf_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">RSS 0.92 feed URL</td>
		                            <td class="v"><?php echo get_bloginfo('rss_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">RSS 2.0 feed URL</td>
		                            <td class="v"><?php echo get_bloginfo('rss2_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Comments Atom feed URL</td>
		                            <td class="v"><?php echo get_bloginfo('comments_atom_url', 'display')?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">Comments RSS 2.0 feed URL</td>
		                            <td class="v"><?php echo get_bloginfo('comments_rss2_url', 'display')?></td>
		                        </tr>
		                    </tbody>
		                </table>
						<?php
						if($mode_server == 'on'){
						?>
		                <h1>Database Info</h1>
		                <table>
		                    <tbody>
		                        <tr>
		                            <td class="e">DB Client Info</td>
		                            <td class="v"><?php echo mysqli_get_client_info()?></td>
		                        </tr>
		                        <tr>
		                            <td class="e">MySQL/MariaDB Version</td>
		                            <td class="v"><?php echo $wpdb->db_server_info()?></td>
		                        </tr>
		                    </tbody>
		                </table>

		                <?php
		                    ob_start();
		                    phpinfo();
		                    $pinfo = ob_get_contents();
		                    ob_end_clean();

		                    // the name attribute "module_Zend Optimizer" of an anker-tag is not xhtml valide, so replace it with "module_Zend_Optimizer"
		                    echo ( str_replace ( "module_Zend Optimizer", "module_Zend_Optimizer", preg_replace ( '%^.*<body>(.*)</body>.*$%ms', '$1', $pinfo ) ) );
		                ?>
		            </div>
        		<?php
				}else{
					?>
						<p>Make sure you enable option in <a href="?page=yoda-tools-dashboard">Dashboard</a> page to see more details..</p>
					<?php
				}
        		break;

       	endswitch;
   	?>
</div><!-- end .yoda-tab-content -->