<?php 
     use yoda_tools\includes\core\settings;
     /* Does User Belong Here */
     if ( ! defined( 'YODA_TOOLS_VERSION' ) ) {
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }


    $yoda_options = settings::getInstance();

    //let's get all available gravity forms

    $gravity_forms = [];
    $cf7forms = [];

    //get gravity forms
    if (class_exists('GFAPI')) {
        $gforms = GFAPI::get_forms();
        if (!empty($gforms)) {
            foreach ($gforms as $gform) {
                $gravity_forms[$gform['id']] = $gform['title'];
            }
        }
    }

    //get cf7 forms
    $args = [
        'posts_per_page'    => -1,
        'post_type'         => 'wpcf7_contact_form',
        'post_status'       => 'publish'
    ];
    $cfforms = get_posts($args);
    if (!empty($cfforms)) {
        foreach ($cfforms as $cf7form) {
            $cf7forms[$cf7form->ID] = $cf7form->post_title;
        }
    }

    $allforms = [
      'Gravity Forms' => $gravity_forms,
      'Contact Form 7'  => $cf7forms
    ];

    $forms = [];
    $comments = $yoda_options->getOption('spam-advanced-comments');
    $registration = $yoda_options->getOption('spam-advanced-registration');
    $get_forms = $yoda_options->getOption('spam-advanced-single-forms');

    $advancedspam_status = (!empty($yoda_options->getOption('spam-advanced-spamguard')))? 'checked' : null;
    if (!$yoda_options->getOption('apikey')) {
        $advancedspam_status = 'disabled';
    }

    if (!empty($get_forms)) {
        $forms = $get_forms;
    }

?>
<h2 class="yoda-nav-title"><?php esc_html_e('Spam Guard','yoda-tools'); ?></h2>

<?php if(isset($_GET['success'])): ?>
    <div class="notice notice-success is-dismissible">
        <p><?php esc_html_e( 'Changes has been saved', 'yoda-tools' ); ?></p>
    </div>
<?php endif; ?>

<div class="yoda-tab-content yoda-settings">
    <form method="post" action="">
        <table class="wp-list-table yoda_spamguard_settings_table">
            <tbody data-form-type="other">
                <tr>
                    <td colspan="2"><strong>
                        <?php esc_html_e('This plugin should work with default settings, however if you begin to get spam, update the field name below.', 'yoda-tools'); ?>
                    </strong></td>
                </tr>
                <tr>
                    <td width="250"><?php esc_html_e('Advanced Spamguard', 'yoda-tools'); ?></td>
                    <td>
                        <label><input id="yoda_sg_advanced_spamguard" name="yoda_sg_advanced_spamguard" class="yoda_sg_field_name" type="checkbox" value="Y" <?php echo $advancedspam_status; ?>>
                        <em><?php esc_html_e('Utilizing advanced AI technology to automatically detect and filter out a comprehensive range of spam messages.','yoda-tools'); ?></em></label>
                    </td>
                </tr>
                <?php
                    $row_status = null;
                    if ($advancedspam_status === 'checked') {
                        $row_status = "style=\"display:table-row;\"";
                    }
                ?>
                <tr class="yoda_sg_advanced_inside_options" <?php echo $row_status; ?>>
                    <td width="250"></td>
                    <td>
                        <p>
                            <label>
                                <?php
                                $check_status = null;
                                if ($comments === 'Y') {
                                    $check_status = "checked";
                                }
                                ?>
                                <input type="checkbox" id="yoda_sg_advanced_comments" name="yoda_sg_advanced_comments" value="Y" <?php echo esc_attr($check_status); ?>> <?php esc_html_e('Advanced SpamGuard Filtering for Comments', 'yoda-tools'); ?>
                            </label>
                        </p>
                        <p>
                            <label>
                                <?php
                                $check_status = null;
                                if ($registration === 'Y') {
                                    $check_status = "checked";
                                }
                                ?>
                                <input type="checkbox" id="yoda_sg_advanced_registration" name="yoda_sg_advanced_registration" value="Y" <?php echo esc_attr($check_status); ?>> <?php esc_html_e('Advanced SpamGuard Filtering for Registrations', 'yoda-tools'); ?>
                            </label>
                        </p>
                        <p>
                            <label>
                                <?php
                                $check_status = null;
                                $show_form_list = null;
                                if (!empty($forms)) {
                                    $check_status = "checked";
                                    $show_form_list = "style=\"display: block;\"";
                                }
                                ?>
                                <input type="checkbox" name="yoda_sg_advanced_by_forms" id="yoda_sg_advanced_by_forms" value="Y" <?php echo esc_attr($check_status); ?>> <?php esc_html_e('Advanced SpamGuard Filtering by form', 'yoda-tools'); ?>
                            </label>
                        </p>

                        <div class="yoda_sg_advanced_forms_options_wrapper" data-no-data="<?php echo empty($forms); ?>" <?php echo $show_form_list; ?>>
                            <?php foreach ($allforms as $pluging => $aforms) {
                                if (!empty($aforms)) {
                            ?>
                                <ul class="yoda_sg_advanced_forms_options"><h3><?php echo esc_html($pluging); ?></h3>
                            <?php foreach ($aforms as $id => $form) {
                                    $status_form = (in_array($id, $forms))? 'checked' : null;
                            ?>
                                    <li><label><input type="checkbox" name="yoda_sg_advanced_single_form[]" value="<?php echo $id; ?>" <?php echo esc_attr($status_form); ?>><?php echo $form; ?></label></li>
                            <?php } ?>
                                </ul>
                            <?php }
                            } ?>
                        </div>
                    </td>
                </tr>
                <tr>
                    <td width="250"><?php esc_html_e('Honey Pot Field Name', 'yoda-tools'); ?></td>
                    <td>
                        <input id="yoda_sg_field_name" name="yoda_sg_field_name" style="width:300px;" value="<?php echo esc_attr($yoda_options->getOption('spam-guard-name')); ?>" type="text" readonly="readonly" data-form-type="name">

                        <span class="dashicons dashicons-update yoda_spam_generate_name" style="font-size: 28px; cursor: pointer;"></span>

                        <br>
                        <em><?php esc_html_e('Changing the field name regularly is a good idea. Please click on icon above to generate new field name.','yoda-tools'); ?></em>
                    </td>
                </tr>
                <tr>
                    <td><?php esc_html_e('Honey Pot Error Message', 'yoda-tools'); ?></td>
                    <td>
                        <input name="yoda_sg_error_message" id="yoda_sg_error_message" style="width:300px;" value="<?php echo esc_attr($yoda_options->getOption('spam-guard-message')); ?>" type="text" data-form-type="other"><br>
                        <em><?php esc_html_e('Mesage for bots. No average human users will see though.', 'yoda-tools'); ?></em>
                    </td>
                </tr>

                <tr>
                    <td><?php esc_html_e('Disable Honeypot Test Widget', 'yoda-tools'); ?></td>
                    <td>
                        <select name="yoda_sg_disable_test_widget" data-form-type="other">
                                <option value="no" <?php echo ($yoda_options->getOption('spam-guard-widget') === 'no')? esc_attr('selected') : null; ?>><?php esc_html_e('No', 'yoda-tools'); ?></option>
                                <option value="yes" <?php echo ($yoda_options->getOption('spam-guard-widget') === 'yes' || !$yoda_options->getOption('spam-guard-widget'))? esc_attr('selected') : null; ?>><?php esc_html_e('Yes', 'yoda-tools'); ?></option>
                        </select>
                        <em><?php esc_html_e('Only visible when Admin user is logged in.', 'yoda-tools'); ?></em>
                    </td>
                </tr>
                <tr>
                    <td colspan="2">
                        <input type="hidden" id="yoda_sg_nonce" name="yoda_sg_nonce" value="<?php echo wp_create_nonce('yoda-spam-guard'); ?>">
                        <input type="submit" name="yoda_save_sg_settings" class="button-primary" value="<?php esc_attr_e('Save Changes','yoda-tools'); ?>" data-form-type="action">
                    </td>
                </tr>
            </tbody>
        </table>
    </form>
</div>