<?php
    /*
        Plugin Name: Maennche Tools
        Plugin URI:  https://www.maennche.com/
        Description: Easily add Google Analytics, Microsoft Clarity, and other analytics to your website. This plugin helps track and record visitor events
        Version:     1.7.4
        Author:      Team Maennche
        Author URI:  https://maennche.com/
        License:     GPLv3
        License URI: https://www.gnu.org/licenses/gpl.html

        Maennche Tools Plugin is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 2 of the License, or
        any later version.

        Maennche Tools is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with Site Health. If not, see https://www.gnu.org/licenses/gpl.html.
    */
    namespace yoda_tools;

    use WP_Query;
    use yoda_tools\includes\core\adminNotices;
    use yoda_tools\includes\core\settings;
    use yoda_tools\includes\core\utilities;
    use yoda_tools\includes\core\upgrader;
    use yoda_tools\includes\core\Mail_Handler;
    use yoda_tools\includes\core\spamGuard;
    use yoda_tools\includes\scripts;
    use yoda_tools\includes\core\seoSmartValidator;

    if ( !function_exists( 'add_action' ) ){
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    if ( !function_exists( 'add_filter' ) ){
        header( 'Status: 403 Forbidden' );
        header( 'HTTP/1.1 403 Forbidden' );
        exit();
    }

    define( 'YODA_TOOLS_VERSION', '1.7.4' );
    class yoda_tools {
        private string $file = __file__;
        private string $url;

        public $options;
        public $settings;
        public $util;

        private $notification_count = [];

        /**
         * Loading all dependencies
         * @return void
         */
        public function __construct() {
            global $wpdb;

            if ( !defined( 'YODA_TOOLS_PATH' ) ) {
                define( 'YODA_TOOLS_PATH', plugin_dir_path(__file__));
            }
            if ( !defined( 'YODA_TOOLS_URL' ) ) {
                define( 'YODA_TOOLS_URL', plugin_dir_url(__file__));
            }
            if ( !defined( 'YODA_TOOLS_SLUG' ) ) {
                define( 'YODA_TOOLS_SLUG', 'yoda-tools');
            }
            if ( !defined( 'YODA_TOOLS_NAMESPACE' ) ) {
                define( 'YODA_TOOLS_NAMESPACE', 'yoda_tools');
            }
            if ( !defined('YODA_OPENSSL_KEY') ) {
                define( 'YODA_OPENSSL_KEY', 'yoda-tools-openssl');
            }

            /* ***************************** Init *************************** */
            $this->url = plugin_dir_url(__file__);

            /* ***************************** Establish Hooks *************************** */
            add_action( 'init', [$this, 'initialize'] );
            register_activation_hook( __file__,[$this, 'activate'] );
            register_deactivation_hook( __file__,[$this, 'deactivate'] );
            add_action('admin_init', [$this,'settings'] );
            add_action('admin_menu', [$this,'register_menu']);

            // Include the autoloader, so we can dynamically include the rest of the classes.
            require_once( YODA_TOOLS_PATH.'includes/core/autoloader.php' );
            spl_autoload_register( 'yoda_tools_autoloader' );

            //load libraries
            add_action( 'admin_enqueue_scripts', [$this,'load_libraries'] );

        }

        public function load_libraries()
        {
            //load libraries in specific pages only
            if (isset($_GET["page"]) && ($_GET["page"] === "yoda-tools-admin-notices" || $_GET["page"] === "yoda-tools-scripts")) {
                wp_enqueue_style( YODA_TOOLS_SLUG.'--css', YODA_TOOLS_URL.'includes/lib/select2/select2.min.css' );
                wp_enqueue_script( YODA_TOOLS_SLUG.'-select2-js', YODA_TOOLS_URL.'includes/lib/select2/select2.min.js', array( 'jquery' ), '1.6.12', true );
	            wp_add_inline_script( YODA_TOOLS_SLUG.'-select2-js', 'yodaANnonce', wp_create_nonce('yoda-an-nonce'));
            }
        }

        function activate(): void {
            $this->initialize();

            flush_rewrite_rules();
        }

        function deactivate(): void { }

        public function initialize(): void {
            //Version #
            if ((int)get_option(YODA_TOOLS_SLUG.'_version', 0) === 0) {
                update_option(YODA_TOOLS_SLUG.'_version', '1.7.4');
            }

            //Site Id
            if (get_option(YODA_TOOLS_SLUG.'-site-id', 0) === 0) {
                update_option(YODA_TOOLS_SLUG . '-site-id', '');
            }

            if (get_option(YODA_TOOLS_SLUG.'_version') !== YODA_TOOLS_VERSION) {
                //Update DB

                //RUN DB UPDATES HERE
                update_option(YODA_TOOLS_SLUG.'_version', get_option('yoda_analytics_version'));
                update_option(YODA_TOOLS_SLUG.'_defaults', get_option('yoda_analytics_defaults'));
                update_option(YODA_TOOLS_SLUG.'-site-id', get_option('yoda-analytics-site-id'));
                update_option(YODA_TOOLS_SLUG.'_settings', str_replace('yoda_analytics','yoda_tools',get_option('yoda_analytics_settings')));

                delete_option('yoda_analytics_version');
                delete_option('yoda_analytics_settings');
                delete_option('yoda_analytics_defaults');
                delete_option('yoda-analytics-site-id');
                delete_option('_transient_yoda_analytics_plugin_update');
                delete_option('_transient_timeout_yoda_analytics_plugin_update');

                update_option(YODA_TOOLS_SLUG.'_version', YODA_TOOLS_VERSION);
                
            }
            
            $settings = settings::getInstance();
            $settings->checkTableLogs();

            /** Dashboard options */
            $this->options = settings::getInstance();
            $mode_scripts = $this->options->getOption('mode-scripts');
            $mode_server = $this->options->getOption('mode-server-tools');
            $mode_smtp_tools = $this->options->getOption('mode-smtp-tools');
            $mode_seo_tools = $this->options->getOption('mode-seo-tools');

            if(!$mode_scripts){
                update_option(YODA_TOOLS_SLUG.'-mode-scripts', 'on');
            }
            if(!$mode_server){
                update_option(YODA_TOOLS_SLUG.'-mode-server-tools', 'off');
            }
            if(!$mode_smtp_tools){
                update_option(YODA_TOOLS_SLUG.'-mode-smtp-tools', 'off');
            }
            if(!$mode_seo_tools){
                update_option(YODA_TOOLS_SLUG.'-mode-seo-tools', 'on');
            }

            scripts::register();
            $this->util = new utilities();

            add_filter('plugin_action_links', [$this,'plugin_links'], 10, 2);
	        $updater = NEW upgrader();

	        if($mode_smtp_tools === 'on') {
                global $phpmailer;
                $phpmailer = new Mail_Handler(true);
	        }

            //Load Front Script
            //update @since 1.6.12
            if (is_admin()) {
                $this->initialize_front_end();
            } else {
                add_action('template_redirect',[$this, 'initialize_front_end']);
            }
            

            $mode_spam_guard_tool = $this->options->getOption('mode-spam-guard');
            //load spam guard if enabled
            if ($mode_spam_guard_tool === 'on') {
                new spamGuard();
            }

            $mode_admin_notices = $this->options->getOption('mode-admin-notices');
            //load Admin Notices if enabled
            if ($mode_admin_notices === 'on') {
                new adminNotices();
            }

            if ($this->options->getOption('apikey')) {
                //initialize smart validator
                new seoSmartValidator($this->options->getOption('apikey'), $this->options->getOption('credits'));
            }
            
        }


        //Admin Menus
        public function register_menu(): void {
            global $admin_page_hooks;
            $this->options = settings::getInstance();
            $mode_scripts = $this->options->getOption('mode-scripts');
            $mode_server = $this->options->getOption('mode-server-tools');
            $mode_seo_tools = $this->options->getOption('mode-seo-tools');
            $mode_smtp_tools = $this->options->getOption('mode-smtp-tools');
            $mode_spam_guard = $this->options->getOption('mode-spam-guard');
            $mode_admin_notices = $this->options->getOption('mode-admin-notices');

            //notification count
            $this->side_notification_count();


            //Check to see if YODA Tools menu has been created by another YODA plugin
            if (!isset($admin_page_hooks['yoda_tools_dashboard'])) {
                //If Not Register it
                add_menu_page('yoda', 'Maennche Tools '.$this->notification_count['all'], 'administrator', YODA_TOOLS_SLUG.'-dashboard', [$this,'dashboard_page'], $this->url . 'images/favicon.png');
            }

            //Register Dashboard Page
            add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'Dashboard', 'Dashboard', 'administrator', YODA_TOOLS_SLUG.'-dashboard', [$this,'dashboard_page']);

            if($mode_scripts == 'on'){
                //Register Scripts Settings Page
                add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'Scripts', 'Scripts', 'administrator', YODA_TOOLS_SLUG.'-scripts', [$this,'scripts_page']);
            }

            if($mode_seo_tools == 'on'){
                
                //Register Tracking Page
                add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'SEO Tools', 'SEO Tools '.$this->notification_count['requires-attention'], 'administrator', YODA_TOOLS_SLUG.'-seo-tools', [$this,'settings_page']);
                
            }

            if($mode_server == 'on'){
                //Register Server Tools Page
                add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'Server Tools', 'Server Tools', 'administrator', YODA_TOOLS_SLUG.'-server-tools', [$this,'server_tools_page']);
            }

            if($mode_smtp_tools == 'on'){
                //Register SMTP
                add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'SMTP', 'SMTP', 'administrator', YODA_TOOLS_SLUG.'-smtp', [$this,'smtp_menu']);
            }

            if($mode_spam_guard === 'on'){
                //Register SMTP
                add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'Spam Guard', 'Spam Guard', 'administrator', YODA_TOOLS_SLUG.'-spam-guard', [$this,'spamguard_page']);
            }

            if($mode_admin_notices === 'on'){
                //Register SMTP
                add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'Admin Notices', 'Admin Notices', 'administrator', YODA_TOOLS_SLUG.'-admin-notices', [$this,'admin_notices_page']);
            }

            //add settings without toggle
            add_submenu_page(YODA_TOOLS_SLUG.'-dashboard', 'Settings', 'Settings', 'administrator', YODA_TOOLS_SLUG.'-settings', [$this,'yoda_version']);
          
        }


        //Dashboard View
        function dashboard_page(): void {
            $this->render_menu_page( 'dashboard' );
        }

        //Settings View
        public function settings_page(): void {
            $this->render_menu_page( 'seo-settings' );
        }

        //Scripts View
        public function scripts_page(): void {
            $this->render_menu_page( 'scripts' );
        }

        //Server Tools View
        public function server_tools_page(): void {
            $this->render_menu_page( 'server-tools' );
        }
        
        //Server Tools View
        public function smtp_menu(): void {
            $this->render_menu_page( 'smtp-config' );
        }

        //Robots.txt View
        public function robots_txt(): void {
            $this->render_menu_page( 'robots-txt' );
        }

        //spam guard
        public function spamguard_page(): void {
            $this->render_menu_page( 'spam-guard' );
        }

        //admin notices
        public function admin_notices_page(): void {
            $this->render_menu_page( 'admin-notices' );
        }

        //preview version
        public function yoda_version(): void {
            $this->render_menu_page( 'settings' );
        }

        

        //Render Admin Page View
        public function render_menu_page($menupage): void {

            require_once( YODA_TOOLS_PATH . '/views/header.php' );
            require_once( YODA_TOOLS_PATH . "/views/$menupage.php" );
            require_once( YODA_TOOLS_PATH . '/views/footer.php' );

        }

        //Setting Definitions
        public function settings(): void {
            $fnSettings = array(
                array(
                    'settings_type' => 'section',
                    'id' => 'yoda_tools_section_settings_general',
                    'title' => __('Tracking', 'yoda_tools'),
                    'callback' => [__NAMESPACE__ . '\includes\core\settings','description_section_callback'],
                    'page' => 'yoda_tools_settings'
                ),
                array(
                    'settings_type' => 'field',
                    'id' => 'yoda_tools_userid',
                    'title' => __('User ID', 'yoda_tools'),
                    'callback' => [__NAMESPACE__ . '\includes\core\settings','settings_field_render'],
                    'page' => 'yoda_tools_settings',
                    'section' => 'yoda_tools_section_settings_general',
                    'args' => array(
                        'id' => 'yoda_tools_userid',
                        'type' => 'checkbox',
                        'class' => '',
                        'name' => 'yoda_tools_userid',
                        'value' => 1,
                        'initial' => 1,
                        'label_for' => '',
                        'description' => __('This is intended to be a known identifier for a user provided by the site owner/tracking library user.', 'yoda_tools'),
                    )
                ),
                array(
                    'settings_type' => 'field',
                    'id' => 'yoda_tools_anonymizeip',
                    'title' => __('Anonymize IP', 'yoda_tools'),
                    'callback' => [__NAMESPACE__ . '\includes\core\settings','settings_field_render'],
                    'page' => 'yoda_tools_settings',
                    'section' => 'yoda_tools_section_settings_general',
                    'args' => array(
                        'id' => 'yoda_tools_anonymizeip',
                        'type' => 'checkbox',
                        'class' => '',
                        'name' => 'yoda_tools_anonymizeip',
                        'value' => 1,
                        'label_for' => '',
                        'description' => __('The IP address of the sender will be anonymized', 'yoda_tools'),
                    )
                ),
                array(
                    'settings_type' => 'field',
                    'id' => 'yoda_tools_displayfeatures',
                    'title' => __('Display Features', 'yoda_tools'),
                    'callback' => [__NAMESPACE__ . '\includes\core\settings','settings_field_render'],
                    'page' => 'yoda_tools_settings',
                    'section' => 'yoda_tools_section_settings_general',
                    'args' => array(
                        'id' => 'yoda_tools_displayfeatures',
                        'type' => 'checkbox',
                        'class' => '',
                        'name' => 'yoda_tools_displayfeatures',
                        'value' => 1,
                        'initial' => 1,
                        'label_for' => '',
                        'description' => __('The plugin works by sending an additional request to stats.g.doubleclick.net that is used to provide advertising features like remarketing and demographics and interest reporting in Google Analytics.', 'yoda_tools'),
                    )
                ),
                array(
                    'settings_type' => 'field',
                    'id' => 'yoda_tools_enhancedlinkatt',
                    'title' => __('Enhanced Link Attribution', 'yoda_tools'),
                    'callback' => [__NAMESPACE__ . '\includes\core\settings','settings_field_render'],
                    'page' => 'yoda_tools_settings',
                    'section' => 'yoda_tools_section_settings_general',
                    'args' => array(
                        'id' => 'yoda_tools_enhancedlinkatt',
                        'type' => 'checkbox',
                        'class' => '',
                        'name' => 'yoda_tools_enhancedlinkatt',
                        'value' => 1,
                        'initial' => 1,
                        'label_for' => '',
                        'description' => __('Enhanced Link Attribution improves the accuracy of your In-Page Analytics report by automatically differentiating between multiple links to the same URL on a single page by using link element IDs.', 'yoda_tools'),
                    )
                ),
                array(
                    'settings_type' => 'field',
                    'id' => 'yoda_tools_events',
                    'title' => __('Event Tracking', 'yoda_tools'),
                    'callback' => [__NAMESPACE__ . '\includes\core\settings','settings_field_render'],
                    'page' => 'yoda_tools_settings',
                    'section' => 'yoda_tools_section_settings_general',
                    'args' => array(
                        'id' => 'yoda_tools_events',
                        'type' => 'checkbox',
                        'class' => '',
                        'name' => 'yoda_tools_events',
                        'value' => 1,
                        'initial' => 1,
                        'label_for' => '',
                        'description' => __('Track events feature: Downloads, Emails, Phone numbers Error 404, Search and Outbound links, Add a comment, Scroll Depth', 'yoda_tools'),
                    )
                )
            );

            settings::register_settings($fnSettings,YODA_TOOLS_SLUG);
        }


        /**
         * Add settings link to plugin actions
         *
         * @param  array  $plugin_actions
         * @param  string $plugin_file
         * @since  1.1
         * @return array
         */
        public static function plugin_links($plugin_actions, $plugin_file): array
        {
            $new_actions = array();

            if ('yoda-tools/yoda_tools.php' === $plugin_file) {
                $new_actions['yoda_tools_dashboard'] = sprintf(__('<a href="%s">Dashboard</a>', 'yoda_tools'), esc_url(admin_url('admin.php?page=yoda-tools-dashboard')));
                $new_actions['yoda_tools_settings'] = sprintf(__('<a href="%s">Settings</a>', 'yoda_tools'), esc_url(admin_url('admin.php?page=yoda-tools-settings')));
            }

            return array_merge($new_actions, $plugin_actions);
        }


        //Front End Scripts
        function initialize_front_end(): void {
            
            $this->options = settings::getInstance();
            $mode_scripts = $this->options->getOption('mode-scripts');

            if (!is_admin()) {
                if($mode_scripts == 'on'){

                    //get current post details
                    global $post;
                    $current_post = $post;

                    //Enqueue User Defined Custom Meta
                    $args_meta = array(
                        'post_type' => 'yoda-meta',
                        'posts_per_page' => -1,
                        'post_status' => 'publish',
                    );
                    $queryMeta = new WP_Query($args_meta);
                    if ($queryMeta->have_posts()) {
                        while ($queryMeta->have_posts()) {
                            $queryMeta->the_post();
                            $fnContent = get_the_content();
                            add_action('wp_head', function () use ($fnContent) {
                                echo $fnContent;
                            }, 1, 1);
                        }
                    }

                    //Enqueue User Defined Scripts
                    $args_script = array(
                        'post_type' => 'yoda-script',
                        'posts_per_page' => -1,
                        'post_status' => 'publish',
                    );
                    $queryScript = new WP_Query($args_script);

                    $gtagExist = false; 
                    $gtagPlacement = 'wp_head';

                    if ($queryScript->have_posts()) {
                        while ($queryScript->have_posts()) {
                            $queryScript->the_post();
                            $fnContent = get_the_content();

                            //Check if gtag scripts exist
                            if( strpos( $fnContent, "gtag()" ) !== false ) 
                                $gtagExist = true;


                            //location filter will go here
                            //@since 1.6.12
                            if (!is_admin()) {
                                $do_load = $this->util->is_load_script(get_the_ID(), $current_post);
                                if (!$do_load) {
                                    //don't load scripts
                                    continue;
                                }
                            }
                            

                            switch (get_post_meta(get_the_ID(), 'yoda-tools-script-location', true)) {
                                case '1':
                                    add_action('wp_head', function () use ($fnContent) {
                                        echo $fnContent;
                                    }, 5, 1);

                                    break;
                                case '2':
                                    add_action('wp_footer', function () use ($fnContent) {
                                        echo $fnContent;
                                    }, 5, 1);

                                    $gtagPlacement = 'wp_footer';
                                    break;
                                default:
                                    //if no location set, set one.
                                    update_post_meta(get_the_ID(), 'yoda-tools-script-location', 1);
                                    add_action('wp_head', function () use ($fnContent) {
                                        echo $fnContent;
                                    }, 5, 1);
                            }
                        }
                    }

                    //Enqueue User Defined Custom CSS
                    $args_css = array(
                        'post_type' => 'yoda-css',
                        'posts_per_page' => -1,
                        'post_status' => 'publish',
                    );
                    $queryCSS = new WP_Query($args_css);


                    if ($queryCSS->have_posts()) {
                        while ($queryCSS->have_posts()) {
                            $queryCSS->the_post();
                            $fnContent = get_the_content();

                            //check for location condition
                            //@since 1.6.12
                            if (!is_admin()) {
                                $do_load = $this->util->is_load_script(get_the_ID(), $current_post);
                                if (!$do_load) {
                                    //don't load scripts
                                    continue;
                                }
                            }

                            //css location
                            switch (get_post_meta(get_the_ID(), 'yoda-tools-script-location', true)) {
                                case '2':
                                    add_action('wp_footer', function () use ($fnContent) {
                                        echo "<style>".$fnContent."</style>";
                                    }, 5, 1);
                                    break;
                                case '1':
                                default:
                                    update_post_meta(get_the_ID(), 'yoda-tools-script-location', 1);
                                    add_action('wp_head', function () use ($fnContent) {
                                        echo "<style>".$fnContent."</style>";
                                    }, 5, 1);
                                break;
                            }

                        }
                    }

                    $args_php = array(
                        'post_type' => 'yoda-php',
                        'posts_per_page' => -1,
                        'post_status' => 'publish',
                    );
                    $queryPHP = new WP_Query($args_php);
                    if ($queryPHP->have_posts()) {
                        while ($queryPHP->have_posts()) {
                            $queryPHP->the_post();
                            $phpContent = get_the_content();
                            $this->settings = settings::getInstance();
                            $this->settings->customPhp($phpContent, ['id' => get_the_ID(), 'title' => get_the_title(), 'content' => esc_html(get_the_content())]);
                        }
                    }

                    //Tracking Analytics
                    $googleSettingArray = null;
                    $saved_options = get_option('yoda_tools_settings');

                    //yoda_tools_userid
                    if (is_user_logged_in()) {
                        $current_user = wp_get_current_user();

                        if (isset($saved_options['yoda_tools_userid']) && (int)$saved_options['yoda_tools_userid'] === 1) {
                            $googleSettingArray['user_id'] = $current_user->ID;
                        }
                    }

                    //yoda_tools_anonymizeip
                    if (isset($saved_options['yoda_tools_anonymizeip']) && (int)$saved_options['yoda_tools_anonymizeip'] === 1) {
                        $googleSettingArray['anonymize_ip'] = true;
                    }

                    //yoda_tools_displayfeatures
                    if (!isset($saved_options['yoda_tools_displayfeatures'])) {
                        $googleSettingArray['allow_display_features'] = false;
                    }

                    //yoda_tools_enhancedlinkatt
                    if (isset($saved_options['yoda_tools_enhancedlinkatt']) && (int)$saved_options['yoda_tools_enhancedlinkatt'] === 1) {
                        $googleSettingArray['link_attribution'] = true;
                    }

                    if ($gtagExist && !empty($googleSettingArray) && is_array($googleSettingArray)) {
                        add_action($gtagPlacement, function () use ($googleSettingArray) {
                            ?>
                            <script>
                                gtag("config", <?php echo str_replace(array('[', ']'), '', htmlspecialchars(wp_json_encode($googleSettingArray), ENT_NOQUOTES)); ?>);
                            </script>
                            <?php
                        });
                    }

                    //Event Tracking
                    $saved_options = get_option('yoda_tools_settings');
                    $yodaEvents = null;
                    if (isset($saved_options['yoda_tools_events'])) {
                        $yodaEvents = $saved_options['yoda_tools_events'];
                    }
                    
                    if ((int)$yodaEvents === 1) {
                        //Load YODA Tools Analytics Tracking Script
                        wp_enqueue_script( 'yoda-tools', $this->url . 'js/analytics.min.js',array('jquery') );
                        wp_enqueue_script( 'yoda-tools-scroll', $this->url . 'js/jquery.scrolldepth.min.js',array('jquery') );
                    }
                }
            } else {
	            if(isset($_GET["page"]) && strpos(" ".$_GET["page"],"yoda") > 0) {
		            wp_register_style('admin_css', $this->url . 'css/admin_css.css');
		            wp_enqueue_script('admin_js', $this->url . 'js/admin_js.js', array('jquery'));

                    wp_localize_script('admin_js', 'ajaxurl_yoda', array(
			            'url' => admin_url('admin-ajax.php'),
			            'path_url' => YODA_TOOLS_URL
		            ));

		            $plugin['prefix'] = 'yoda';
		            $plugin['url'] = $this->url;
		            $plugin['version'] = YODA_TOOLS_VERSION;
	            }
                if(isset($_GET["page"]) && $_GET["page"] == "yoda-tools-scripts"){
                    wp_enqueue_style( $plugin['prefix'] . '-codemirror-css', $plugin['url'] . 'includes/lib/codemirror/lib/codemirror.css', array(), $plugin['version'], 'all' );
                    wp_enqueue_script( $plugin['prefix'] . '-codemirror-js', $plugin['url'] . 'includes/lib/codemirror/lib/codemirror.js', array(), $plugin['version'], false );

                    // Enqueue addons
                    $addons = array(
                        'display' => array( 'autorefresh', 'placeholder' ),
                        'lint' => array( 'lint', 'html-lint', 'javascript-lint', 'css-lint' ),
                        'hint' => array( 'show-hint', 'jshint', 'javascript-hint', 'css-hint' ),
                        'selection' => array( 'active-line' )
                    );
                    
                    foreach ( $addons as $addons_group_name => $addons_group ) {
                        foreach ( $addons_group as $addon ) {
                            wp_enqueue_script( $plugin['prefix'] . '-codemirror-addon-' . $addon . '-js', $plugin['url'] . 'includes/lib/codemirror/addon/' . $addons_group_name . '/' . $addon . '.js', array(), $plugin['version'], false );
                        }
                    }

                    // Enqueue modes
                    $modes = array(
                        'htmlmixed',
                        'xml',
                        'javascript',
                        'css',
                        'clike',
                        'markdown',
                        'php'
                    );
                    foreach ( $modes as $mode ) {
                        wp_enqueue_script( $plugin['prefix'] . '-codemirror-mode-' . $mode . '-js', $plugin['url'] . 'includes/lib/codemirror/mode/' . $mode . '/' . $mode . '.js', array(), $plugin['version'], true );
                    }

                    
                    // Enqueue modes
                    $themes = array(
                        'dark'
                    );

                    foreach ( $themes as $data ) {
                        wp_enqueue_style( $plugin['prefix'] . '-codemirror-theme-' . $data . '-css', $plugin['url'] . 'includes/lib/codemirror/theme/' . $data . '.css', array(), $plugin['version'], 'all' );
                    }

                    $themeData = settings::getInstance()->getThemeOption();

                    // Enqueue settings file
                    wp_enqueue_script( $plugin['prefix'] . '-codemirror-settings-js', $plugin['url'] . 'includes/js/codemirror-settings.js', array(), $plugin['version'], true );

                    wp_localize_script( $plugin['prefix'] . '-codemirror-settings-js', 'themedata', array( 'theme' => $themeData ) );
                }
            }

            //reset the query to avoid conflict to other plugins
            //always do this right after using WP_Query else use get_posts() instead.
            //@since 1.6.12
            wp_reset_query();

        }


        /**
         * 
         * 
         * side_notification_count
         * Display notication count to any menu in YODA settings
         * @since 1.6.12
         * 
         * 
         */
        public function side_notification_count()
        {

            if (!$this->options->getOption('apikey')) {
                $this->notification_count['requires-attention'] = null;
                $this->notification_count['all'] = null;
                return;
            }

            $args = [
                'posts_per_page'    => -1,
                'post_type'         => 'yoda-seo',
                'post_status'       => 'requires-attention'
            ];
            $data = get_posts($args);
            $req_count = count($data);


            //add other notification count here in the future
            $this->notification_count['requires-attention'] = "<span class=\"update-plugins count-{$req_count}\"><span class=\"plugin-count\">{$req_count}</span></span>";;
            
            //edit here if you have other notification count
            $all_count = $req_count;
            $this->notification_count['all'] = "<span class=\"update-plugins count-{$all_count}\"><span class=\"plugin-count\">{$all_count}</span></span>";


        }
    }

    $yoda_tools = NEW yoda_tools();
